#!/usr/bin/env python3.6
import re
import logging
import sys
from typing import IO, NamedTuple, List

logging.basicConfig(level=logging.DEBUG)

Tag = NamedTuple('Tag', [
    ('group', int),
    ('elem', int),
    ('vr', str),
    ('name', str),
    ('vm', str)])

def list_tags() -> List[Tag]:
    global DATA

    ok = True
    tags: List[Tag] = []
    for line in DATA.split("\n"):
        if re.match(r'\s*#', line) or re.match(r'^\s*$', line):
            continue
        m = re.match(r'\(([^,]+),([^,]+)\)\s+(\w\w)\s+([^\t]+)\s+([^\t]+)', line)
        if not m:
            logging.error("Invalid line: %s", line)
            ok = False
            continue

        vr=m.group(3).upper()
        if vr == "XS":
            # Its generally safe to treat XS as unsigned.  See
            # https://github.com/dgobbi/vtk-dicom/issues/38 for
	    # some discussions.
            vr = "US"
        elif vr == "OX":
	    # TODO(saito) I'm less sure about the OX rule. Where is
	    # this crap defined in the standard??
            vr = "OW"

        tag = Tag(group=m.group(1),
                  elem=m.group(2),
                  vr=vr,
                  name=m.group(4),
                  vm=m.group(5))


        if not re.match('^[0-9A-Fa-f]+$', tag.group) or not re.match('^[0-9A-Fa-f]+$', tag.elem):
            continue
        tags.append(tag)
    if not ok:
        sys.exit(1)

    return tags

def generate(out: IO[str]):
    tags = list_tags()

    print("package dicomtag", file=out)
    print("", file=out)
    print("// Code generated from generate_tag_definitions.py. DO NOT EDIT.", file=out)
    for t in tags:
        if t.name.find("RETIRED") >= 0:
            continue
        print(f'var {t.name} = Tag{{0x{t.group}, 0x{t.elem}}}', file=out)

    print("var tagDict map[Tag]TagInfo", file=out)
    print("", file=out)
    print("func init() {", file=out)
    print("	maybeInitTagDict()", file=out)
    print("}", file=out)
    print("func maybeInitTagDict() {", file=out)
    print("	if len(tagDict) > 0 {", file=out)
    print("		return", file=out)
    print("	}", file=out)
    print("	tagDict = make(map[Tag]TagInfo)", file=out)
    for t in tags:
        print(f'	tagDict[Tag{{0x{t.group}, 0x{t.elem}}}] = TagInfo{{Tag{{0x{t.group}, 0x{t.elem}}}, "{t.vr}", "{t.name}", "{t.vm}"}}', file=out)
    print("}", file=out)


#(0000,0000)	UL	CommandGroupLength	1	DICOM_2011


DATA = """#
#  Copyright (C) 1994-2012, OFFIS e.V.
#  All rights reserved.  See COPYRIGHT file for details.
#
#  This software and supporting documentation were developed by
#
#    OFFIS e.V.
#    R&D Division Health
#    Escherweg 2
#    D-26121 Oldenburg, Germany
#
#
#  Module:  dcmdata
#
#  Author:  Andrew Hewett, Marco Eichelberg, Joerg Riesmeier
#
#  Purpose: This is the global standard DICOM data dictionary for the DCMTK.
#
# This file contains the complete data dictionary from the 2011 edition of the
# DICOM standard.  This also includes the non-private definitions from the
# DICONDE (Digital Imaging and Communication in Nondestructive Evaluation) and
# DICOS (Digital Imaging and Communications in Security) standard.
#
# In addition, the data dictionary entries from the following final text
# supplements and correction items have been incorporated:
# - Supplement 152, 158.
# - CP 1064, 1123, 1137, 1138, 1147, 1188, 1200, 1204, 1216.
#
# Each line represents an entry in the data dictionary.  Each line has 5 fields
# (Tag, VR, Name, VM, Version).  Entries need not be in ascending tag order.
#
# Entries may override existing entries.
#
# Each field must be separated by a single tab.  The tag values (gggg,eeee)
# must be in hexedecimal and must be surrounded by parentheses.  Repeating
# groups are represented by indicating the range (gggg-gggg,eeee).  By default
# the repeating notation only represents even numbers.  A range where only
# odd numbers are valid is represented using the notation (gggg-o-gggg,eeee).
# A range can represent both even and odd numbers using the notation
# (gggg-u-gggg,eeee).  The element part of the tag can also be a range.
#
# Comments have a '#' at the beginning of the line.
#
# Tag		VR	Name			VM	Version
#
(0000,0000)	UL	CommandGroupLength	1	DICOM_2011
(0000,0002)	UI	AffectedSOPClassUID	1	DICOM_2011
(0000,0003)	UI	RequestedSOPClassUID	1	DICOM_2011
(0000,0100)	US	CommandField	1	DICOM_2011
(0000,0110)	US	MessageID	1	DICOM_2011
(0000,0120)	US	MessageIDBeingRespondedTo	1	DICOM_2011
(0000,0600)	AE	MoveDestination	1	DICOM_2011
(0000,0700)	US	Priority	1	DICOM_2011
(0000,0800)	US	CommandDataSetType	1	DICOM_2011
(0000,0900)	US	Status	1	DICOM_2011
(0000,0901)	AT	OffendingElement	1-n	DICOM_2011
(0000,0902)	LO	ErrorComment	1	DICOM_2011
(0000,0903)	US	ErrorID	1	DICOM_2011
(0000,1000)	UI	AffectedSOPInstanceUID	1	DICOM_2011
(0000,1001)	UI	RequestedSOPInstanceUID	1	DICOM_2011
(0000,1002)	US	EventTypeID	1	DICOM_2011
(0000,1005)	AT	AttributeIdentifierList	1-n	DICOM_2011
(0000,1008)	US	ActionTypeID	1	DICOM_2011
(0000,1020)	US	NumberOfRemainingSuboperations	1	DICOM_2011
(0000,1021)	US	NumberOfCompletedSuboperations	1	DICOM_2011
(0000,1022)	US	NumberOfFailedSuboperations	1	DICOM_2011
(0000,1023)	US	NumberOfWarningSuboperations	1	DICOM_2011
(0000,1030)	AE	MoveOriginatorApplicationEntityTitle	1	DICOM_2011
(0000,1031)	US	MoveOriginatorMessageID	1	DICOM_2011
(0002,0000)	UL	FileMetaInformationGroupLength	1	DICOM_2011
(0002,0001)	OB	FileMetaInformationVersion	1	DICOM_2011
(0002,0002)	UI	MediaStorageSOPClassUID	1	DICOM_2011
(0002,0003)	UI	MediaStorageSOPInstanceUID	1	DICOM_2011
(0002,0010)	UI	TransferSyntaxUID	1	DICOM_2011
(0002,0012)	UI	ImplementationClassUID	1	DICOM_2011
(0002,0013)	SH	ImplementationVersionName	1	DICOM_2011
(0002,0016)	AE	SourceApplicationEntityTitle	1	DICOM_2011
(0002,0100)	UI	PrivateInformationCreatorUID	1	DICOM_2011
(0002,0102)	OB	PrivateInformation	1	DICOM_2011
(0004,1130)	CS	FileSetID	1	DICOM_2011
(0004,1141)	CS	FileSetDescriptorFileID	1-8	DICOM_2011
(0004,1142)	CS	SpecificCharacterSetOfFileSetDescriptorFile	1	DICOM_2011
(0004,1200)	up	OffsetOfTheFirstDirectoryRecordOfTheRootDirectoryEntity	1	DICOM_2011
(0004,1202)	up	OffsetOfTheLastDirectoryRecordOfTheRootDirectoryEntity	1	DICOM_2011
(0004,1212)	US	FileSetConsistencyFlag	1	DICOM_2011
(0004,1220)	SQ	DirectoryRecordSequence	1	DICOM_2011
(0004,1400)	up	OffsetOfTheNextDirectoryRecord	1	DICOM_2011
(0004,1410)	US	RecordInUseFlag	1	DICOM_2011
(0004,1420)	up	OffsetOfReferencedLowerLevelDirectoryEntity	1	DICOM_2011
(0004,1430)	CS	DirectoryRecordType	1	DICOM_2011
(0004,1432)	UI	PrivateRecordUID	1	DICOM_2011
(0004,1500)	CS	ReferencedFileID	1-8	DICOM_2011
(0004,1510)	UI	ReferencedSOPClassUIDInFile	1	DICOM_2011
(0004,1511)	UI	ReferencedSOPInstanceUIDInFile	1	DICOM_2011
(0004,1512)	UI	ReferencedTransferSyntaxUIDInFile	1	DICOM_2011
(0004,151A)	UI	ReferencedRelatedGeneralSOPClassUIDInFile	1-n	DICOM_2011
(0008,0005)	CS	SpecificCharacterSet	1-n	DICOM_2011
(0008,0006)	SQ	LanguageCodeSequence	1	DICOM_2011
(0008,0008)	CS	ImageType	2-n	DICOM_2011
(0008,0012)	DA	InstanceCreationDate	1	DICOM_2011
(0008,0013)	TM	InstanceCreationTime	1	DICOM_2011
(0008,0014)	UI	InstanceCreatorUID	1	DICOM_2011
(0008,0016)	UI	SOPClassUID	1	DICOM_2011
(0008,0018)	UI	SOPInstanceUID	1	DICOM_2011
(0008,001A)	UI	RelatedGeneralSOPClassUID	1-n	DICOM_2011
(0008,001B)	UI	OriginalSpecializedSOPClassUID	1	DICOM_2011
(0008,0020)	DA	StudyDate	1	DICOM_2011
(0008,0021)	DA	SeriesDate	1	DICOM_2011
(0008,0022)	DA	AcquisitionDate	1	DICOM_2011
(0008,0023)	DA	ContentDate	1	DICOM_2011
(0008,002A)	DT	AcquisitionDateTime	1	DICOM_2011
(0008,0030)	TM	StudyTime	1	DICOM_2011
(0008,0031)	TM	SeriesTime	1	DICOM_2011
(0008,0032)	TM	AcquisitionTime	1	DICOM_2011
(0008,0033)	TM	ContentTime	1	DICOM_2011
(0008,0050)	SH	AccessionNumber	1	DICOM_2011
(0008,0051)	SQ	IssuerOfAccessionNumberSequence	1	DICOM_2011
(0008,0052)	CS	QueryRetrieveLevel	1	DICOM_2011
(0008,0054)	AE	RetrieveAETitle	1-n	DICOM_2011
(0008,0056)	CS	InstanceAvailability	1	DICOM_2011
(0008,0058)	UI	FailedSOPInstanceUIDList	1-n	DICOM_2011
(0008,0060)	CS	Modality	1	DICOM_2011
(0008,0061)	CS	ModalitiesInStudy	1-n	DICOM_2011
(0008,0062)	UI	SOPClassesInStudy	1-n	DICOM_2011
(0008,0064)	CS	ConversionType	1	DICOM_2011
(0008,0068)	CS	PresentationIntentType	1	DICOM_2011
(0008,0070)	LO	Manufacturer	1	DICOM_2011
(0008,0080)	LO	InstitutionName	1	DICOM_2011
(0008,0081)	ST	InstitutionAddress	1	DICOM_2011
(0008,0082)	SQ	InstitutionCodeSequence	1	DICOM_2011
(0008,0090)	PN	ReferringPhysicianName	1	DICOM_2011
(0008,0092)	ST	ReferringPhysicianAddress	1	DICOM_2011
(0008,0094)	SH	ReferringPhysicianTelephoneNumbers	1-n	DICOM_2011
(0008,0096)	SQ	ReferringPhysicianIdentificationSequence	1	DICOM_2011
(0008,0100)	SH	CodeValue	1	DICOM_2011
(0008,0102)	SH	CodingSchemeDesignator	1	DICOM_2011
(0008,0103)	SH	CodingSchemeVersion	1	DICOM_2011
(0008,0104)	LO	CodeMeaning	1	DICOM_2011
(0008,0105)	CS	MappingResource	1	DICOM_2011
(0008,0106)	DT	ContextGroupVersion	1	DICOM_2011
(0008,0107)	DT	ContextGroupLocalVersion	1	DICOM_2011
(0008,010B)	CS	ContextGroupExtensionFlag	1	DICOM_2011
(0008,010C)	UI	CodingSchemeUID	1	DICOM_2011
(0008,010D)	UI	ContextGroupExtensionCreatorUID	1	DICOM_2011
(0008,010F)	CS	ContextIdentifier	1	DICOM_2011
(0008,0110)	SQ	CodingSchemeIdentificationSequence	1	DICOM_2011
(0008,0112)	LO	CodingSchemeRegistry	1	DICOM_2011
(0008,0114)	ST	CodingSchemeExternalID	1	DICOM_2011
(0008,0115)	ST	CodingSchemeName	1	DICOM_2011
(0008,0116)	ST	CodingSchemeResponsibleOrganization	1	DICOM_2011
(0008,0117)	UI	ContextUID	1	DICOM_2011
(0008,0201)	SH	TimezoneOffsetFromUTC	1	DICOM_2011
(0008,1010)	SH	StationName	1	DICOM_2011
(0008,1030)	LO	StudyDescription	1	DICOM_2011
(0008,1032)	SQ	ProcedureCodeSequence	1	DICOM_2011
(0008,103E)	LO	SeriesDescription	1	DICOM_2011
(0008,103F)	SQ	SeriesDescriptionCodeSequence	1	DICOM_2011
(0008,1040)	LO	InstitutionalDepartmentName	1	DICOM_2011
(0008,1048)	PN	PhysiciansOfRecord	1-n	DICOM_2011
(0008,1049)	SQ	PhysiciansOfRecordIdentificationSequence	1	DICOM_2011
(0008,1050)	PN	PerformingPhysicianName	1-n	DICOM_2011
(0008,1052)	SQ	PerformingPhysicianIdentificationSequence	1	DICOM_2011
(0008,1060)	PN	NameOfPhysiciansReadingStudy	1-n	DICOM_2011
(0008,1062)	SQ	PhysiciansReadingStudyIdentificationSequence	1	DICOM_2011
(0008,1070)	PN	OperatorsName	1-n	DICOM_2011
(0008,1072)	SQ	OperatorIdentificationSequence	1	DICOM_2011
(0008,1080)	LO	AdmittingDiagnosesDescription	1-n	DICOM_2011
(0008,1084)	SQ	AdmittingDiagnosesCodeSequence	1	DICOM_2011
(0008,1090)	LO	ManufacturerModelName	1	DICOM_2011
(0008,1110)	SQ	ReferencedStudySequence	1	DICOM_2011
(0008,1111)	SQ	ReferencedPerformedProcedureStepSequence	1	DICOM_2011
(0008,1115)	SQ	ReferencedSeriesSequence	1	DICOM_2011
(0008,1120)	SQ	ReferencedPatientSequence	1	DICOM_2011
(0008,1125)	SQ	ReferencedVisitSequence	1	DICOM_2011
(0008,1134)	SQ	ReferencedStereometricInstanceSequence	1	DICOM_2011
(0008,113A)	SQ	ReferencedWaveformSequence	1	DICOM_2011
(0008,1140)	SQ	ReferencedImageSequence	1	DICOM_2011
(0008,114A)	SQ	ReferencedInstanceSequence	1	DICOM_2011
(0008,114B)	SQ	ReferencedRealWorldValueMappingInstanceSequence	1	DICOM_2011
(0008,1150)	UI	ReferencedSOPClassUID	1	DICOM_2011
(0008,1155)	UI	ReferencedSOPInstanceUID	1	DICOM_2011
(0008,115A)	UI	SOPClassesSupported	1-n	DICOM_2011
(0008,1160)	IS	ReferencedFrameNumber	1-n	DICOM_2011
(0008,1161)	UL	SimpleFrameList	1-n	DICOM_2011
(0008,1162)	UL	CalculatedFrameList	3-3n	DICOM_2011
(0008,1163)	FD	TimeRange	2	DICOM_2011
(0008,1164)	SQ	FrameExtractionSequence	1	DICOM_2011
(0008,1167)	UI	MultiFrameSourceSOPInstanceUID	1	DICOM_2011
(0008,1195)	UI	TransactionUID	1	DICOM_2011
(0008,1197)	US	FailureReason	1	DICOM_2011
(0008,1198)	SQ	FailedSOPSequence	1	DICOM_2011
(0008,1199)	SQ	ReferencedSOPSequence	1	DICOM_2011
(0008,1200)	SQ	StudiesContainingOtherReferencedInstancesSequence	1	DICOM_2011
(0008,1250)	SQ	RelatedSeriesSequence	1	DICOM_2011
(0008,2111)	ST	DerivationDescription	1	DICOM_2011
(0008,2112)	SQ	SourceImageSequence	1	DICOM_2011
(0008,2120)	SH	StageName	1	DICOM_2011
(0008,2122)	IS	StageNumber	1	DICOM_2011
(0008,2124)	IS	NumberOfStages	1	DICOM_2011
(0008,2127)	SH	ViewName	1	DICOM_2011
(0008,2128)	IS	ViewNumber	1	DICOM_2011
(0008,2129)	IS	NumberOfEventTimers	1	DICOM_2011
(0008,212A)	IS	NumberOfViewsInStage	1	DICOM_2011
(0008,2130)	DS	EventElapsedTimes	1-n	DICOM_2011
(0008,2132)	LO	EventTimerNames	1-n	DICOM_2011
(0008,2133)	SQ	EventTimerSequence	1	DICOM_2011
(0008,2134)	FD	EventTimeOffset	1	DICOM_2011
(0008,2135)	SQ	EventCodeSequence	1	DICOM_2011
(0008,2142)	IS	StartTrim	1	DICOM_2011
(0008,2143)	IS	StopTrim	1	DICOM_2011
(0008,2144)	IS	RecommendedDisplayFrameRate	1	DICOM_2011
(0008,2218)	SQ	AnatomicRegionSequence	1	DICOM_2011
(0008,2220)	SQ	AnatomicRegionModifierSequence	1	DICOM_2011
(0008,2228)	SQ	PrimaryAnatomicStructureSequence	1	DICOM_2011
(0008,2229)	SQ	AnatomicStructureSpaceOrRegionSequence	1	DICOM_2011
(0008,2230)	SQ	PrimaryAnatomicStructureModifierSequence	1	DICOM_2011
(0008,3001)	SQ	AlternateRepresentationSequence	1	DICOM_2011
(0008,3010)	UI	IrradiationEventUID	1	DICOM_2011
(0008,9007)	CS	FrameType	4	DICOM_2011
(0008,9092)	SQ	ReferencedImageEvidenceSequence	1	DICOM_2011
(0008,9121)	SQ	ReferencedRawDataSequence	1	DICOM_2011
(0008,9123)	UI	CreatorVersionUID	1	DICOM_2011
(0008,9124)	SQ	DerivationImageSequence	1	DICOM_2011
(0008,9154)	SQ	SourceImageEvidenceSequence	1	DICOM_2011
(0008,9205)	CS	PixelPresentation	1	DICOM_2011
(0008,9206)	CS	VolumetricProperties	1	DICOM_2011
(0008,9207)	CS	VolumeBasedCalculationTechnique	1	DICOM_2011
(0008,9208)	CS	ComplexImageComponent	1	DICOM_2011
(0008,9209)	CS	AcquisitionContrast	1	DICOM_2011
(0008,9215)	SQ	DerivationCodeSequence	1	DICOM_2011
(0008,9237)	SQ	ReferencedPresentationStateSequence	1	DICOM_2011
(0008,9410)	SQ	ReferencedOtherPlaneSequence	1	DICOM_2011
(0008,9458)	SQ	FrameDisplaySequence	1	DICOM_2011
(0008,9459)	FL	RecommendedDisplayFrameRateInFloat	1	DICOM_2011
(0008,9460)	CS	SkipFrameRangeFlag	1	DICOM_2011
(0010,0010)	PN	PatientName	1	DICOM_2011
(0010,0020)	LO	PatientID	1	DICOM_2011
(0010,0021)	LO	IssuerOfPatientID	1	DICOM_2011
(0010,0022)	CS	TypeOfPatientID	1	DICOM_2011
(0010,0024)	SQ	IssuerOfPatientIDQualifiersSequence	1	DICOM_2011
(0010,0030)	DA	PatientBirthDate	1	DICOM_2011
(0010,0032)	TM	PatientBirthTime	1	DICOM_2011
(0010,0040)	CS	PatientSex	1	DICOM_2011
(0010,0050)	SQ	PatientInsurancePlanCodeSequence	1	DICOM_2011
(0010,0101)	SQ	PatientPrimaryLanguageCodeSequence	1	DICOM_2011
(0010,0102)	SQ	PatientPrimaryLanguageModifierCodeSequence	1	DICOM_2011
(0010,1000)	LO	OtherPatientIDs	1-n	DICOM_2011
(0010,1001)	PN	OtherPatientNames	1-n	DICOM_2011
(0010,1002)	SQ	OtherPatientIDsSequence	1	DICOM_2011
(0010,1005)	PN	PatientBirthName	1	DICOM_2011
(0010,1010)	AS	PatientAge	1	DICOM_2011
(0010,1020)	DS	PatientSize	1	DICOM_2011
(0010,1021)	SQ	PatientSizeCodeSequence	1	DICOM_2011
(0010,1030)	DS	PatientWeight	1	DICOM_2011
(0010,1040)	LO	PatientAddress	1	DICOM_2011
(0010,1060)	PN	PatientMotherBirthName	1	DICOM_2011
(0010,1080)	LO	MilitaryRank	1	DICOM_2011
(0010,1081)	LO	BranchOfService	1	DICOM_2011
(0010,1090)	LO	MedicalRecordLocator	1	DICOM_2011
(0010,2000)	LO	MedicalAlerts	1-n	DICOM_2011
(0010,2110)	LO	Allergies	1-n	DICOM_2011
(0010,2150)	LO	CountryOfResidence	1	DICOM_2011
(0010,2152)	LO	RegionOfResidence	1	DICOM_2011
(0010,2154)	SH	PatientTelephoneNumbers	1-n	DICOM_2011
(0010,2160)	SH	EthnicGroup	1	DICOM_2011
(0010,2180)	SH	Occupation	1	DICOM_2011
(0010,21A0)	CS	SmokingStatus	1	DICOM_2011
(0010,21B0)	LT	AdditionalPatientHistory	1	DICOM_2011
(0010,21C0)	US	PregnancyStatus	1	DICOM_2011
(0010,21D0)	DA	LastMenstrualDate	1	DICOM_2011
(0010,21F0)	LO	PatientReligiousPreference	1	DICOM_2011
(0010,2201)	LO	PatientSpeciesDescription	1	DICOM_2011
(0010,2202)	SQ	PatientSpeciesCodeSequence	1	DICOM_2011
(0010,2203)	CS	PatientSexNeutered	1	DICOM_2011
(0010,2210)	CS	AnatomicalOrientationType	1	DICOM_2011
(0010,2292)	LO	PatientBreedDescription	1	DICOM_2011
(0010,2293)	SQ	PatientBreedCodeSequence	1	DICOM_2011
(0010,2294)	SQ	BreedRegistrationSequence	1	DICOM_2011
(0010,2295)	LO	BreedRegistrationNumber	1	DICOM_2011
(0010,2296)	SQ	BreedRegistryCodeSequence	1	DICOM_2011
(0010,2297)	PN	ResponsiblePerson	1	DICOM_2011
(0010,2298)	CS	ResponsiblePersonRole	1	DICOM_2011
(0010,2299)	LO	ResponsibleOrganization	1	DICOM_2011
(0010,4000)	LT	PatientComments	1	DICOM_2011
(0010,9431)	FL	ExaminedBodyThickness	1	DICOM_2011
(0012,0010)	LO	ClinicalTrialSponsorName	1	DICOM_2011
(0012,0020)	LO	ClinicalTrialProtocolID	1	DICOM_2011
(0012,0021)	LO	ClinicalTrialProtocolName	1	DICOM_2011
(0012,0030)	LO	ClinicalTrialSiteID	1	DICOM_2011
(0012,0031)	LO	ClinicalTrialSiteName	1	DICOM_2011
(0012,0040)	LO	ClinicalTrialSubjectID	1	DICOM_2011
(0012,0042)	LO	ClinicalTrialSubjectReadingID	1	DICOM_2011
(0012,0050)	LO	ClinicalTrialTimePointID	1	DICOM_2011
(0012,0051)	ST	ClinicalTrialTimePointDescription	1	DICOM_2011
(0012,0060)	LO	ClinicalTrialCoordinatingCenterName	1	DICOM_2011
(0012,0062)	CS	PatientIdentityRemoved	1	DICOM_2011
(0012,0063)	LO	DeidentificationMethod	1-n	DICOM_2011
(0012,0064)	SQ	DeidentificationMethodCodeSequence	1	DICOM_2011
(0012,0071)	LO	ClinicalTrialSeriesID	1	DICOM_2011
(0012,0072)	LO	ClinicalTrialSeriesDescription	1	DICOM_2011
(0012,0081)	LO	ClinicalTrialProtocolEthicsCommitteeName	1	DICOM_2011
(0012,0082)	LO	ClinicalTrialProtocolEthicsCommitteeApprovalNumber	1	DICOM_2011
(0012,0083)	SQ	ConsentForClinicalTrialUseSequence	1	DICOM_2011
(0012,0084)	CS	DistributionType	1	DICOM_2011
(0012,0085)	CS	ConsentForDistributionFlag	1	DICOM_2011
(0014,0023)	ST	CADFileFormat	1-n	DICOM_2011
(0014,0024)	ST	ComponentReferenceSystem	1-n	DICOM_2011
(0014,0025)	ST	ComponentManufacturingProcedure	1-n	DICOM_2011
(0014,0028)	ST	ComponentManufacturer	1-n	DICOM_2011
(0014,0030)	DS	MaterialThickness	1-n	DICOM_2011
(0014,0032)	DS	MaterialPipeDiameter	1-n	DICOM_2011
(0014,0034)	DS	MaterialIsolationDiameter	1-n	DICOM_2011
(0014,0042)	ST	MaterialGrade	1-n	DICOM_2011
(0014,0044)	ST	MaterialPropertiesFileID	1-n	DICOM_2011
(0014,0045)	ST	MaterialPropertiesFileFormat	1-n	DICOM_2011
(0014,0046)	LT	MaterialNotes	1	DICOM_2011
(0014,0050)	CS	ComponentShape	1	DICOM_2011
(0014,0052)	CS	CurvatureType	1	DICOM_2011
(0014,0054)	DS	OuterDiameter	1	DICOM_2011
(0014,0056)	DS	InnerDiameter	1	DICOM_2011
(0014,1010)	ST	ActualEnvironmentalConditions	1	DICOM_2011
(0014,1020)	DA	ExpiryDate	1	DICOM_2011
(0014,1040)	ST	EnvironmentalConditions	1	DICOM_2011
(0014,2002)	SQ	EvaluatorSequence	1	DICOM_2011
(0014,2004)	IS	EvaluatorNumber	1	DICOM_2011
(0014,2006)	PN	EvaluatorName	1	DICOM_2011
(0014,2008)	IS	EvaluationAttempt	1	DICOM_2011
(0014,2012)	SQ	IndicationSequence	1	DICOM_2011
(0014,2014)	IS	IndicationNumber	1	DICOM_2011
(0014,2016)	SH	IndicationLabel	1	DICOM_2011
(0014,2018)	ST	IndicationDescription	1	DICOM_2011
(0014,201A)	CS	IndicationType	1-n	DICOM_2011
(0014,201C)	CS	IndicationDisposition	1	DICOM_2011
(0014,201E)	SQ	IndicationROISequence	1	DICOM_2011
(0014,2030)	SQ	IndicationPhysicalPropertySequence	1	DICOM_2011
(0014,2032)	SH	PropertyLabel	1	DICOM_2011
(0014,2202)	IS	CoordinateSystemNumberOfAxes	1	DICOM_2011
(0014,2204)	SQ	CoordinateSystemAxesSequence	1	DICOM_2011
(0014,2206)	ST	CoordinateSystemAxisDescription	1	DICOM_2011
(0014,2208)	CS	CoordinateSystemDataSetMapping	1	DICOM_2011
(0014,220A)	IS	CoordinateSystemAxisNumber	1	DICOM_2011
(0014,220C)	CS	CoordinateSystemAxisType	1	DICOM_2011
(0014,220E)	CS	CoordinateSystemAxisUnits	1	DICOM_2011
(0014,2210)	OB	CoordinateSystemAxisValues	1	DICOM_2011
(0014,2220)	SQ	CoordinateSystemTransformSequence	1	DICOM_2011
(0014,2222)	ST	TransformDescription	1	DICOM_2011
(0014,2224)	IS	TransformNumberOfAxes	1	DICOM_2011
(0014,2226)	IS	TransformOrderOfAxes	1-n	DICOM_2011
(0014,2228)	CS	TransformedAxisUnits	1	DICOM_2011
(0014,222A)	DS	CoordinateSystemTransformRotationAndScaleMatrix	1-n	DICOM_2011
(0014,222C)	DS	CoordinateSystemTransformTranslationMatrix	1-n	DICOM_2011
(0014,3011)	DS	InternalDetectorFrameTime	1	DICOM_2011
(0014,3012)	DS	NumberOfFramesIntegrated	1	DICOM_2011
(0014,3020)	SQ	DetectorTemperatureSequence	1	DICOM_2011
# the VR of the following attribute is "ST" according to ASTM E2699-11 (DICONDE),
# but "DS" according to DICOM PS 3.6-2011; the former makes more sense and is correct
(0014,3022)	ST	SensorName	1	DICOM_2011
(0014,3024)	DS	HorizontalOffsetOfSensor	1	DICOM_2011
(0014,3026)	DS	VerticalOffsetOfSensor	1	DICOM_2011
(0014,3028)	DS	SensorTemperature	1	DICOM_2011
(0014,3040)	SQ	DarkCurrentSequence	1	DICOM_2011
(0014,3050)	ox	DarkCurrentCounts	1	DICOM_2011
(0014,3060)	SQ	GainCorrectionReferenceSequence	1	DICOM_2011
(0014,3070)	ox	AirCounts	1	DICOM_2011
(0014,3071)	DS	KVUsedInGainCalibration	1	DICOM_2011
(0014,3072)	DS	MAUsedInGainCalibration	1	DICOM_2011
(0014,3073)	DS	NumberOfFramesUsedForIntegration	1	DICOM_2011
(0014,3074)	LO	FilterMaterialUsedInGainCalibration	1	DICOM_2011
(0014,3075)	DS	FilterThicknessUsedInGainCalibration	1	DICOM_2011
(0014,3076)	DA	DateOfGainCalibration	1	DICOM_2011
(0014,3077)	TM	TimeOfGainCalibration	1	DICOM_2011
(0014,3080)	OB	BadPixelImage	1	DICOM_2011
(0014,3099)	LT	CalibrationNotes	1	DICOM_2011
(0014,4002)	SQ	PulserEquipmentSequence	1	DICOM_2011
(0014,4004)	CS	PulserType	1	DICOM_2011
(0014,4006)	LT	PulserNotes	1	DICOM_2011
(0014,4008)	SQ	ReceiverEquipmentSequence	1	DICOM_2011
(0014,400A)	CS	AmplifierType	1	DICOM_2011
(0014,400C)	LT	ReceiverNotes	1	DICOM_2011
(0014,400E)	SQ	PreAmplifierEquipmentSequence	1	DICOM_2011
(0014,400F)	LT	PreAmplifierNotes	1	DICOM_2011
(0014,4010)	SQ	TransmitTransducerSequence	1	DICOM_2011
(0014,4011)	SQ	ReceiveTransducerSequence	1	DICOM_2011
(0014,4012)	US	NumberOfElements	1	DICOM_2011
(0014,4013)	CS	ElementShape	1	DICOM_2011
(0014,4014)	DS	ElementDimensionA	1	DICOM_2011
(0014,4015)	DS	ElementDimensionB	1	DICOM_2011
(0014,4016)	DS	ElementPitch	1	DICOM_2011
(0014,4017)	DS	MeasuredBeamDimensionA	1	DICOM_2011
(0014,4018)	DS	MeasuredBeamDimensionB	1	DICOM_2011
(0014,4019)	DS	LocationOfMeasuredBeamDiameter	1	DICOM_2011
(0014,401A)	DS	NominalFrequency	1	DICOM_2011
(0014,401B)	DS	MeasuredCenterFrequency	1	DICOM_2011
(0014,401C)	DS	MeasuredBandwidth	1	DICOM_2011
(0014,4020)	SQ	PulserSettingsSequence	1	DICOM_2011
(0014,4022)	DS	PulseWidth	1	DICOM_2011
(0014,4024)	DS	ExcitationFrequency	1	DICOM_2011
(0014,4026)	CS	ModulationType	1	DICOM_2011
(0014,4028)	DS	Damping	1	DICOM_2011
(0014,4030)	SQ	ReceiverSettingsSequence	1	DICOM_2011
(0014,4031)	DS	AcquiredSoundpathLength	1	DICOM_2011
(0014,4032)	CS	AcquisitionCompressionType	1	DICOM_2011
(0014,4033)	IS	AcquisitionSampleSize	1	DICOM_2011
(0014,4034)	DS	RectifierSmoothing	1	DICOM_2011
(0014,4035)	SQ	DACSequence	1	DICOM_2011
(0014,4036)	CS	DACType	1	DICOM_2011
(0014,4038)	DS	DACGainPoints	1-n	DICOM_2011
(0014,403A)	DS	DACTimePoints	1-n	DICOM_2011
(0014,403C)	DS	DACAmplitude	1-n	DICOM_2011
(0014,4040)	SQ	PreAmplifierSettingsSequence	1	DICOM_2011
(0014,4050)	SQ	TransmitTransducerSettingsSequence	1	DICOM_2011
(0014,4051)	SQ	ReceiveTransducerSettingsSequence	1	DICOM_2011
(0014,4052)	DS	IncidentAngle	1	DICOM_2011
(0014,4054)	ST	CouplingTechnique	1	DICOM_2011
(0014,4056)	ST	CouplingMedium	1	DICOM_2011
(0014,4057)	DS	CouplingVelocity	1	DICOM_2011
(0014,4058)	DS	CrystalCenterLocationX	1	DICOM_2011
(0014,4059)	DS	CrystalCenterLocationZ	1	DICOM_2011
(0014,405A)	DS	SoundPathLength	1	DICOM_2011
(0014,405C)	ST	DelayLawIdentifier	1	DICOM_2011
(0014,4060)	SQ	GateSettingsSequence	1	DICOM_2011
(0014,4062)	DS	GateThreshold	1	DICOM_2011
(0014,4064)	DS	VelocityOfSound	1	DICOM_2011
(0014,4070)	SQ	CalibrationSettingsSequence	1	DICOM_2011
(0014,4072)	ST	CalibrationProcedure	1	DICOM_2011
(0014,4074)	SH	ProcedureVersion	1	DICOM_2011
(0014,4076)	DA	ProcedureCreationDate	1	DICOM_2011
(0014,4078)	DA	ProcedureExpirationDate	1	DICOM_2011
(0014,407A)	DA	ProcedureLastModifiedDate	1	DICOM_2011
(0014,407C)	TM	CalibrationTime	1-n	DICOM_2011
(0014,407E)	DA	CalibrationDate	1-n	DICOM_2011
(0014,5002)	IS	LINACEnergy	1	DICOM_2011
(0014,5004)	IS	LINACOutput	1	DICOM_2011
(0018,0010)	LO	ContrastBolusAgent	1	DICOM_2011
(0018,0012)	SQ	ContrastBolusAgentSequence	1	DICOM_2011
(0018,0014)	SQ	ContrastBolusAdministrationRouteSequence	1	DICOM_2011
(0018,0015)	CS	BodyPartExamined	1	DICOM_2011
(0018,0020)	CS	ScanningSequence	1-n	DICOM_2011
(0018,0021)	CS	SequenceVariant	1-n	DICOM_2011
(0018,0022)	CS	ScanOptions	1-n	DICOM_2011
(0018,0023)	CS	MRAcquisitionType	1	DICOM_2011
(0018,0024)	SH	SequenceName	1	DICOM_2011
(0018,0025)	CS	AngioFlag	1	DICOM_2011
(0018,0026)	SQ	InterventionDrugInformationSequence	1	DICOM_2011
(0018,0027)	TM	InterventionDrugStopTime	1	DICOM_2011
(0018,0028)	DS	InterventionDrugDose	1	DICOM_2011
(0018,0029)	SQ	InterventionDrugCodeSequence	1	DICOM_2011
(0018,002A)	SQ	AdditionalDrugSequence	1	DICOM_2011
(0018,0031)	LO	Radiopharmaceutical	1	DICOM_2011
(0018,0034)	LO	InterventionDrugName	1	DICOM_2011
(0018,0035)	TM	InterventionDrugStartTime	1	DICOM_2011
(0018,0036)	SQ	InterventionSequence	1	DICOM_2011
(0018,0038)	CS	InterventionStatus	1	DICOM_2011
(0018,003A)	ST	InterventionDescription	1	DICOM_2011
(0018,0040)	IS	CineRate	1	DICOM_2011
(0018,0042)	CS	InitialCineRunState	1	DICOM_2011
(0018,0050)	DS	SliceThickness	1	DICOM_2011
(0018,0060)	DS	KVP	1	DICOM_2011
(0018,0070)	IS	CountsAccumulated	1	DICOM_2011
(0018,0071)	CS	AcquisitionTerminationCondition	1	DICOM_2011
(0018,0072)	DS	EffectiveDuration	1	DICOM_2011
(0018,0073)	CS	AcquisitionStartCondition	1	DICOM_2011
(0018,0074)	IS	AcquisitionStartConditionData	1	DICOM_2011
(0018,0075)	IS	AcquisitionTerminationConditionData	1	DICOM_2011
(0018,0080)	DS	RepetitionTime	1	DICOM_2011
(0018,0081)	DS	EchoTime	1	DICOM_2011
(0018,0082)	DS	InversionTime	1	DICOM_2011
(0018,0083)	DS	NumberOfAverages	1	DICOM_2011
(0018,0084)	DS	ImagingFrequency	1	DICOM_2011
(0018,0085)	SH	ImagedNucleus	1	DICOM_2011
(0018,0086)	IS	EchoNumbers	1-n	DICOM_2011
(0018,0087)	DS	MagneticFieldStrength	1	DICOM_2011
(0018,0088)	DS	SpacingBetweenSlices	1	DICOM_2011
(0018,0089)	IS	NumberOfPhaseEncodingSteps	1	DICOM_2011
(0018,0090)	DS	DataCollectionDiameter	1	DICOM_2011
(0018,0091)	IS	EchoTrainLength	1	DICOM_2011
(0018,0093)	DS	PercentSampling	1	DICOM_2011
(0018,0094)	DS	PercentPhaseFieldOfView	1	DICOM_2011
(0018,0095)	DS	PixelBandwidth	1	DICOM_2011
(0018,1000)	LO	DeviceSerialNumber	1	DICOM_2011
(0018,1002)	UI	DeviceUID	1	DICOM_2011
(0018,1003)	LO	DeviceID	1	DICOM_2011
(0018,1004)	LO	PlateID	1	DICOM_2011
(0018,1005)	LO	GeneratorID	1	DICOM_2011
(0018,1006)	LO	GridID	1	DICOM_2011
(0018,1007)	LO	CassetteID	1	DICOM_2011
(0018,1008)	LO	GantryID	1	DICOM_2011
(0018,1010)	LO	SecondaryCaptureDeviceID	1	DICOM_2011
(0018,1012)	DA	DateOfSecondaryCapture	1	DICOM_2011
(0018,1014)	TM	TimeOfSecondaryCapture	1	DICOM_2011
(0018,1016)	LO	SecondaryCaptureDeviceManufacturer	1	DICOM_2011
(0018,1018)	LO	SecondaryCaptureDeviceManufacturerModelName	1	DICOM_2011
(0018,1019)	LO	SecondaryCaptureDeviceSoftwareVersions	1-n	DICOM_2011
(0018,1020)	LO	SoftwareVersions	1-n	DICOM_2011
(0018,1022)	SH	VideoImageFormatAcquired	1	DICOM_2011
(0018,1023)	LO	DigitalImageFormatAcquired	1	DICOM_2011
(0018,1030)	LO	ProtocolName	1	DICOM_2011
(0018,1040)	LO	ContrastBolusRoute	1	DICOM_2011
(0018,1041)	DS	ContrastBolusVolume	1	DICOM_2011
(0018,1042)	TM	ContrastBolusStartTime	1	DICOM_2011
(0018,1043)	TM	ContrastBolusStopTime	1	DICOM_2011
(0018,1044)	DS	ContrastBolusTotalDose	1	DICOM_2011
(0018,1045)	IS	SyringeCounts	1	DICOM_2011
(0018,1046)	DS	ContrastFlowRate	1-n	DICOM_2011
(0018,1047)	DS	ContrastFlowDuration	1-n	DICOM_2011
(0018,1048)	CS	ContrastBolusIngredient	1	DICOM_2011
(0018,1049)	DS	ContrastBolusIngredientConcentration	1	DICOM_2011
(0018,1050)	DS	SpatialResolution	1	DICOM_2011
(0018,1060)	DS	TriggerTime	1	DICOM_2011
(0018,1061)	LO	TriggerSourceOrType	1	DICOM_2011
(0018,1062)	IS	NominalInterval	1	DICOM_2011
(0018,1063)	DS	FrameTime	1	DICOM_2011
(0018,1064)	LO	CardiacFramingType	1	DICOM_2011
(0018,1065)	DS	FrameTimeVector	1-n	DICOM_2011
(0018,1066)	DS	FrameDelay	1	DICOM_2011
(0018,1067)	DS	ImageTriggerDelay	1	DICOM_2011
(0018,1068)	DS	MultiplexGroupTimeOffset	1	DICOM_2011
(0018,1069)	DS	TriggerTimeOffset	1	DICOM_2011
(0018,106A)	CS	SynchronizationTrigger	1	DICOM_2011
(0018,106C)	US	SynchronizationChannel	2	DICOM_2011
(0018,106E)	UL	TriggerSamplePosition	1	DICOM_2011
(0018,1070)	LO	RadiopharmaceuticalRoute	1	DICOM_2011
(0018,1071)	DS	RadiopharmaceuticalVolume	1	DICOM_2011
(0018,1072)	TM	RadiopharmaceuticalStartTime	1	DICOM_2011
(0018,1073)	TM	RadiopharmaceuticalStopTime	1	DICOM_2011
(0018,1074)	DS	RadionuclideTotalDose	1	DICOM_2011
(0018,1075)	DS	RadionuclideHalfLife	1	DICOM_2011
(0018,1076)	DS	RadionuclidePositronFraction	1	DICOM_2011
(0018,1077)	DS	RadiopharmaceuticalSpecificActivity	1	DICOM_2011
(0018,1078)	DT	RadiopharmaceuticalStartDateTime	1	DICOM_2011
(0018,1079)	DT	RadiopharmaceuticalStopDateTime	1	DICOM_2011
(0018,1080)	CS	BeatRejectionFlag	1	DICOM_2011
(0018,1081)	IS	LowRRValue	1	DICOM_2011
(0018,1082)	IS	HighRRValue	1	DICOM_2011
(0018,1083)	IS	IntervalsAcquired	1	DICOM_2011
(0018,1084)	IS	IntervalsRejected	1	DICOM_2011
(0018,1085)	LO	PVCRejection	1	DICOM_2011
(0018,1086)	IS	SkipBeats	1	DICOM_2011
(0018,1088)	IS	HeartRate	1	DICOM_2011
(0018,1090)	IS	CardiacNumberOfImages	1	DICOM_2011
(0018,1094)	IS	TriggerWindow	1	DICOM_2011
(0018,1100)	DS	ReconstructionDiameter	1	DICOM_2011
(0018,1110)	DS	DistanceSourceToDetector	1	DICOM_2011
(0018,1111)	DS	DistanceSourceToPatient	1	DICOM_2011
(0018,1114)	DS	EstimatedRadiographicMagnificationFactor	1	DICOM_2011
(0018,1120)	DS	GantryDetectorTilt	1	DICOM_2011
(0018,1121)	DS	GantryDetectorSlew	1	DICOM_2011
(0018,1130)	DS	TableHeight	1	DICOM_2011
(0018,1131)	DS	TableTraverse	1	DICOM_2011
(0018,1134)	CS	TableMotion	1	DICOM_2011
(0018,1135)	DS	TableVerticalIncrement	1-n	DICOM_2011
(0018,1136)	DS	TableLateralIncrement	1-n	DICOM_2011
(0018,1137)	DS	TableLongitudinalIncrement	1-n	DICOM_2011
(0018,1138)	DS	TableAngle	1	DICOM_2011
(0018,113A)	CS	TableType	1	DICOM_2011
(0018,1140)	CS	RotationDirection	1	DICOM_2011
(0018,1142)	DS	RadialPosition	1-n	DICOM_2011
(0018,1143)	DS	ScanArc	1	DICOM_2011
(0018,1144)	DS	AngularStep	1	DICOM_2011
(0018,1145)	DS	CenterOfRotationOffset	1	DICOM_2011
(0018,1147)	CS	FieldOfViewShape	1	DICOM_2011
(0018,1149)	IS	FieldOfViewDimensions	1-2	DICOM_2011
(0018,1150)	IS	ExposureTime	1	DICOM_2011
(0018,1151)	IS	XRayTubeCurrent	1	DICOM_2011
(0018,1152)	IS	Exposure	1	DICOM_2011
(0018,1153)	IS	ExposureInuAs	1	DICOM_2011
(0018,1154)	DS	AveragePulseWidth	1	DICOM_2011
(0018,1155)	CS	RadiationSetting	1	DICOM_2011
(0018,1156)	CS	RectificationType	1	DICOM_2011
(0018,115A)	CS	RadiationMode	1	DICOM_2011
(0018,115E)	DS	ImageAndFluoroscopyAreaDoseProduct	1	DICOM_2011
(0018,1160)	SH	FilterType	1	DICOM_2011
(0018,1161)	LO	TypeOfFilters	1-n	DICOM_2011
(0018,1162)	DS	IntensifierSize	1	DICOM_2011
(0018,1164)	DS	ImagerPixelSpacing	2	DICOM_2011
(0018,1166)	CS	Grid	1-n	DICOM_2011
(0018,1170)	IS	GeneratorPower	1	DICOM_2011
(0018,1180)	SH	CollimatorGridName	1	DICOM_2011
(0018,1181)	CS	CollimatorType	1	DICOM_2011
(0018,1182)	IS	FocalDistance	1-2	DICOM_2011
(0018,1183)	DS	XFocusCenter	1-2	DICOM_2011
(0018,1184)	DS	YFocusCenter	1-2	DICOM_2011
(0018,1190)	DS	FocalSpots	1-n	DICOM_2011
(0018,1191)	CS	AnodeTargetMaterial	1	DICOM_2011
(0018,11A0)	DS	BodyPartThickness	1	DICOM_2011
(0018,11A2)	DS	CompressionForce	1	DICOM_2011
(0018,1200)	DA	DateOfLastCalibration	1-n	DICOM_2011
(0018,1201)	TM	TimeOfLastCalibration	1-n	DICOM_2011
(0018,1210)	SH	ConvolutionKernel	1-n	DICOM_2011
(0018,1242)	IS	ActualFrameDuration	1	DICOM_2011
(0018,1243)	IS	CountRate	1	DICOM_2011
(0018,1244)	US	PreferredPlaybackSequencing	1	DICOM_2011
(0018,1250)	SH	ReceiveCoilName	1	DICOM_2011
(0018,1251)	SH	TransmitCoilName	1	DICOM_2011
(0018,1260)	SH	PlateType	1	DICOM_2011
(0018,1261)	LO	PhosphorType	1	DICOM_2011
(0018,1300)	DS	ScanVelocity	1	DICOM_2011
(0018,1301)	CS	WholeBodyTechnique	1-n	DICOM_2011
(0018,1302)	IS	ScanLength	1	DICOM_2011
(0018,1310)	US	AcquisitionMatrix	4	DICOM_2011
(0018,1312)	CS	InPlanePhaseEncodingDirection	1	DICOM_2011
(0018,1314)	DS	FlipAngle	1	DICOM_2011
(0018,1315)	CS	VariableFlipAngleFlag	1	DICOM_2011
(0018,1316)	DS	SAR	1	DICOM_2011
(0018,1318)	DS	dBdt	1	DICOM_2011
(0018,1400)	LO	AcquisitionDeviceProcessingDescription	1	DICOM_2011
(0018,1401)	LO	AcquisitionDeviceProcessingCode	1	DICOM_2011
(0018,1402)	CS	CassetteOrientation	1	DICOM_2011
(0018,1403)	CS	CassetteSize	1	DICOM_2011
(0018,1404)	US	ExposuresOnPlate	1	DICOM_2011
(0018,1405)	IS	RelativeXRayExposure	1	DICOM_2011
(0018,1411)	DS	ExposureIndex	1	DICOM_2011
(0018,1412)	DS	TargetExposureIndex	1	DICOM_2011
(0018,1413)	DS	DeviationIndex	1	DICOM_2011
(0018,1450)	DS	ColumnAngulation	1	DICOM_2011
(0018,1460)	DS	TomoLayerHeight	1	DICOM_2011
(0018,1470)	DS	TomoAngle	1	DICOM_2011
(0018,1480)	DS	TomoTime	1	DICOM_2011
(0018,1490)	CS	TomoType	1	DICOM_2011
(0018,1491)	CS	TomoClass	1	DICOM_2011
(0018,1495)	IS	NumberOfTomosynthesisSourceImages	1	DICOM_2011
(0018,1500)	CS	PositionerMotion	1	DICOM_2011
(0018,1508)	CS	PositionerType	1	DICOM_2011
(0018,1510)	DS	PositionerPrimaryAngle	1	DICOM_2011
(0018,1511)	DS	PositionerSecondaryAngle	1	DICOM_2011
(0018,1520)	DS	PositionerPrimaryAngleIncrement	1-n	DICOM_2011
(0018,1521)	DS	PositionerSecondaryAngleIncrement	1-n	DICOM_2011
(0018,1530)	DS	DetectorPrimaryAngle	1	DICOM_2011
(0018,1531)	DS	DetectorSecondaryAngle	1	DICOM_2011
(0018,1600)	CS	ShutterShape	1-3	DICOM_2011
(0018,1602)	IS	ShutterLeftVerticalEdge	1	DICOM_2011
(0018,1604)	IS	ShutterRightVerticalEdge	1	DICOM_2011
(0018,1606)	IS	ShutterUpperHorizontalEdge	1	DICOM_2011
(0018,1608)	IS	ShutterLowerHorizontalEdge	1	DICOM_2011
(0018,1610)	IS	CenterOfCircularShutter	2	DICOM_2011
(0018,1612)	IS	RadiusOfCircularShutter	1	DICOM_2011
(0018,1620)	IS	VerticesOfThePolygonalShutter	2-2n	DICOM_2011
(0018,1622)	US	ShutterPresentationValue	1	DICOM_2011
(0018,1623)	US	ShutterOverlayGroup	1	DICOM_2011
(0018,1624)	US	ShutterPresentationColorCIELabValue	3	DICOM_2011
(0018,1700)	CS	CollimatorShape	1-3	DICOM_2011
(0018,1702)	IS	CollimatorLeftVerticalEdge	1	DICOM_2011
(0018,1704)	IS	CollimatorRightVerticalEdge	1	DICOM_2011
(0018,1706)	IS	CollimatorUpperHorizontalEdge	1	DICOM_2011
(0018,1708)	IS	CollimatorLowerHorizontalEdge	1	DICOM_2011
(0018,1710)	IS	CenterOfCircularCollimator	2	DICOM_2011
(0018,1712)	IS	RadiusOfCircularCollimator	1	DICOM_2011
(0018,1720)	IS	VerticesOfThePolygonalCollimator	2-2n	DICOM_2011
(0018,1800)	CS	AcquisitionTimeSynchronized	1	DICOM_2011
(0018,1801)	SH	TimeSource	1	DICOM_2011
(0018,1802)	CS	TimeDistributionProtocol	1	DICOM_2011
(0018,1803)	LO	NTPSourceAddress	1	DICOM_2011
(0018,2001)	IS	PageNumberVector	1-n	DICOM_2011
(0018,2002)	SH	FrameLabelVector	1-n	DICOM_2011
(0018,2003)	DS	FramePrimaryAngleVector	1-n	DICOM_2011
(0018,2004)	DS	FrameSecondaryAngleVector	1-n	DICOM_2011
(0018,2005)	DS	SliceLocationVector	1-n	DICOM_2011
(0018,2006)	SH	DisplayWindowLabelVector	1-n	DICOM_2011
(0018,2010)	DS	NominalScannedPixelSpacing	2	DICOM_2011
(0018,2020)	CS	DigitizingDeviceTransportDirection	1	DICOM_2011
(0018,2030)	DS	RotationOfScannedFilm	1	DICOM_2011
(0018,3100)	CS	IVUSAcquisition	1	DICOM_2011
(0018,3101)	DS	IVUSPullbackRate	1	DICOM_2011
(0018,3102)	DS	IVUSGatedRate	1	DICOM_2011
(0018,3103)	IS	IVUSPullbackStartFrameNumber	1	DICOM_2011
(0018,3104)	IS	IVUSPullbackStopFrameNumber	1	DICOM_2011
(0018,3105)	IS	LesionNumber	1-n	DICOM_2011
(0018,5000)	SH	OutputPower	1-n	DICOM_2011
(0018,5010)	LO	TransducerData	1-n	DICOM_2011
(0018,5012)	DS	FocusDepth	1	DICOM_2011
(0018,5020)	LO	ProcessingFunction	1	DICOM_2011
(0018,5022)	DS	MechanicalIndex	1	DICOM_2011
(0018,5024)	DS	BoneThermalIndex	1	DICOM_2011
(0018,5026)	DS	CranialThermalIndex	1	DICOM_2011
(0018,5027)	DS	SoftTissueThermalIndex	1	DICOM_2011
(0018,5028)	DS	SoftTissueFocusThermalIndex	1	DICOM_2011
(0018,5029)	DS	SoftTissueSurfaceThermalIndex	1	DICOM_2011
(0018,5050)	IS	DepthOfScanField	1	DICOM_2011
(0018,5100)	CS	PatientPosition	1	DICOM_2011
(0018,5101)	CS	ViewPosition	1	DICOM_2011
(0018,5104)	SQ	ProjectionEponymousNameCodeSequence	1	DICOM_2011
(0018,6000)	DS	Sensitivity	1	DICOM_2011
(0018,6011)	SQ	SequenceOfUltrasoundRegions	1	DICOM_2011
(0018,6012)	US	RegionSpatialFormat	1	DICOM_2011
(0018,6014)	US	RegionDataType	1	DICOM_2011
(0018,6016)	UL	RegionFlags	1	DICOM_2011
(0018,6018)	UL	RegionLocationMinX0	1	DICOM_2011
(0018,601A)	UL	RegionLocationMinY0	1	DICOM_2011
(0018,601C)	UL	RegionLocationMaxX1	1	DICOM_2011
(0018,601E)	UL	RegionLocationMaxY1	1	DICOM_2011
(0018,6020)	SL	ReferencePixelX0	1	DICOM_2011
(0018,6022)	SL	ReferencePixelY0	1	DICOM_2011
(0018,6024)	US	PhysicalUnitsXDirection	1	DICOM_2011
(0018,6026)	US	PhysicalUnitsYDirection	1	DICOM_2011
(0018,6028)	FD	ReferencePixelPhysicalValueX	1	DICOM_2011
(0018,602A)	FD	ReferencePixelPhysicalValueY	1	DICOM_2011
(0018,602C)	FD	PhysicalDeltaX	1	DICOM_2011
(0018,602E)	FD	PhysicalDeltaY	1	DICOM_2011
(0018,6030)	UL	TransducerFrequency	1	DICOM_2011
(0018,6031)	CS	TransducerType	1	DICOM_2011
(0018,6032)	UL	PulseRepetitionFrequency	1	DICOM_2011
(0018,6034)	FD	DopplerCorrectionAngle	1	DICOM_2011
(0018,6036)	FD	SteeringAngle	1	DICOM_2011
(0018,6039)	SL	DopplerSampleVolumeXPosition	1	DICOM_2011
(0018,603B)	SL	DopplerSampleVolumeYPosition	1	DICOM_2011
(0018,603D)	SL	TMLinePositionX0	1	DICOM_2011
(0018,603F)	SL	TMLinePositionY0	1	DICOM_2011
(0018,6041)	SL	TMLinePositionX1	1	DICOM_2011
(0018,6043)	SL	TMLinePositionY1	1	DICOM_2011
(0018,6044)	US	PixelComponentOrganization	1	DICOM_2011
(0018,6046)	UL	PixelComponentMask	1	DICOM_2011
(0018,6048)	UL	PixelComponentRangeStart	1	DICOM_2011
(0018,604A)	UL	PixelComponentRangeStop	1	DICOM_2011
(0018,604C)	US	PixelComponentPhysicalUnits	1	DICOM_2011
(0018,604E)	US	PixelComponentDataType	1	DICOM_2011
(0018,6050)	UL	NumberOfTableBreakPoints	1	DICOM_2011
(0018,6052)	UL	TableOfXBreakPoints	1-n	DICOM_2011
(0018,6054)	FD	TableOfYBreakPoints	1-n	DICOM_2011
(0018,6056)	UL	NumberOfTableEntries	1	DICOM_2011
(0018,6058)	UL	TableOfPixelValues	1-n	DICOM_2011
(0018,605A)	FL	TableOfParameterValues	1-n	DICOM_2011
(0018,6060)	FL	RWaveTimeVector	1-n	DICOM_2011
(0018,7000)	CS	DetectorConditionsNominalFlag	1	DICOM_2011
(0018,7001)	DS	DetectorTemperature	1	DICOM_2011
(0018,7004)	CS	DetectorType	1	DICOM_2011
(0018,7005)	CS	DetectorConfiguration	1	DICOM_2011
(0018,7006)	LT	DetectorDescription	1	DICOM_2011
(0018,7008)	LT	DetectorMode	1	DICOM_2011
(0018,700A)	SH	DetectorID	1	DICOM_2011
(0018,700C)	DA	DateOfLastDetectorCalibration	1	DICOM_2011
(0018,700E)	TM	TimeOfLastDetectorCalibration	1	DICOM_2011
(0018,7010)	IS	ExposuresOnDetectorSinceLastCalibration	1	DICOM_2011
(0018,7011)	IS	ExposuresOnDetectorSinceManufactured	1	DICOM_2011
(0018,7012)	DS	DetectorTimeSinceLastExposure	1	DICOM_2011
(0018,7014)	DS	DetectorActiveTime	1	DICOM_2011
(0018,7016)	DS	DetectorActivationOffsetFromExposure	1	DICOM_2011
(0018,701A)	DS	DetectorBinning	2	DICOM_2011
(0018,7020)	DS	DetectorElementPhysicalSize	2	DICOM_2011
(0018,7022)	DS	DetectorElementSpacing	2	DICOM_2011
(0018,7024)	CS	DetectorActiveShape	1	DICOM_2011
(0018,7026)	DS	DetectorActiveDimensions	1-2	DICOM_2011
(0018,7028)	DS	DetectorActiveOrigin	2	DICOM_2011
(0018,702A)	LO	DetectorManufacturerName	1	DICOM_2011
(0018,702B)	LO	DetectorManufacturerModelName	1	DICOM_2011
(0018,7030)	DS	FieldOfViewOrigin	2	DICOM_2011
(0018,7032)	DS	FieldOfViewRotation	1	DICOM_2011
(0018,7034)	CS	FieldOfViewHorizontalFlip	1	DICOM_2011
(0018,7036)	FL	PixelDataAreaOriginRelativeToFOV	2	DICOM_2011
(0018,7038)	FL	PixelDataAreaRotationAngleRelativeToFOV	1	DICOM_2011
(0018,7040)	LT	GridAbsorbingMaterial	1	DICOM_2011
(0018,7041)	LT	GridSpacingMaterial	1	DICOM_2011
(0018,7042)	DS	GridThickness	1	DICOM_2011
(0018,7044)	DS	GridPitch	1	DICOM_2011
(0018,7046)	IS	GridAspectRatio	2	DICOM_2011
(0018,7048)	DS	GridPeriod	1	DICOM_2011
(0018,704C)	DS	GridFocalDistance	1	DICOM_2011
(0018,7050)	CS	FilterMaterial	1-n	DICOM_2011
(0018,7052)	DS	FilterThicknessMinimum	1-n	DICOM_2011
(0018,7054)	DS	FilterThicknessMaximum	1-n	DICOM_2011
(0018,7056)	FL	FilterBeamPathLengthMinimum	1-n	DICOM_2011
(0018,7058)	FL	FilterBeamPathLengthMaximum	1-n	DICOM_2011
(0018,7060)	CS	ExposureControlMode	1	DICOM_2011
(0018,7062)	LT	ExposureControlModeDescription	1	DICOM_2011
(0018,7064)	CS	ExposureStatus	1	DICOM_2011
(0018,7065)	DS	PhototimerSetting	1	DICOM_2011
(0018,8150)	DS	ExposureTimeInuS	1	DICOM_2011
(0018,8151)	DS	XRayTubeCurrentInuA	1	DICOM_2011
(0018,9004)	CS	ContentQualification	1	DICOM_2011
(0018,9005)	SH	PulseSequenceName	1	DICOM_2011
(0018,9006)	SQ	MRImagingModifierSequence	1	DICOM_2011
(0018,9008)	CS	EchoPulseSequence	1	DICOM_2011
(0018,9009)	CS	InversionRecovery	1	DICOM_2011
(0018,9010)	CS	FlowCompensation	1	DICOM_2011
(0018,9011)	CS	MultipleSpinEcho	1	DICOM_2011
(0018,9012)	CS	MultiPlanarExcitation	1	DICOM_2011
(0018,9014)	CS	PhaseContrast	1	DICOM_2011
(0018,9015)	CS	TimeOfFlightContrast	1	DICOM_2011
(0018,9016)	CS	Spoiling	1	DICOM_2011
(0018,9017)	CS	SteadyStatePulseSequence	1	DICOM_2011
(0018,9018)	CS	EchoPlanarPulseSequence	1	DICOM_2011
(0018,9019)	FD	TagAngleFirstAxis	1	DICOM_2011
(0018,9020)	CS	MagnetizationTransfer	1	DICOM_2011
(0018,9021)	CS	T2Preparation	1	DICOM_2011
(0018,9022)	CS	BloodSignalNulling	1	DICOM_2011
(0018,9024)	CS	SaturationRecovery	1	DICOM_2011
(0018,9025)	CS	SpectrallySelectedSuppression	1	DICOM_2011
(0018,9026)	CS	SpectrallySelectedExcitation	1	DICOM_2011
(0018,9027)	CS	SpatialPresaturation	1	DICOM_2011
(0018,9028)	CS	Tagging	1	DICOM_2011
(0018,9029)	CS	OversamplingPhase	1	DICOM_2011
(0018,9030)	FD	TagSpacingFirstDimension	1	DICOM_2011
(0018,9032)	CS	GeometryOfKSpaceTraversal	1	DICOM_2011
(0018,9033)	CS	SegmentedKSpaceTraversal	1	DICOM_2011
(0018,9034)	CS	RectilinearPhaseEncodeReordering	1	DICOM_2011
(0018,9035)	FD	TagThickness	1	DICOM_2011
(0018,9036)	CS	PartialFourierDirection	1	DICOM_2011
(0018,9037)	CS	CardiacSynchronizationTechnique	1	DICOM_2011
(0018,9041)	LO	ReceiveCoilManufacturerName	1	DICOM_2011
(0018,9042)	SQ	MRReceiveCoilSequence	1	DICOM_2011
(0018,9043)	CS	ReceiveCoilType	1	DICOM_2011
(0018,9044)	CS	QuadratureReceiveCoil	1	DICOM_2011
(0018,9045)	SQ	MultiCoilDefinitionSequence	1	DICOM_2011
(0018,9046)	LO	MultiCoilConfiguration	1	DICOM_2011
(0018,9047)	SH	MultiCoilElementName	1	DICOM_2011
(0018,9048)	CS	MultiCoilElementUsed	1	DICOM_2011
(0018,9049)	SQ	MRTransmitCoilSequence	1	DICOM_2011
(0018,9050)	LO	TransmitCoilManufacturerName	1	DICOM_2011
(0018,9051)	CS	TransmitCoilType	1	DICOM_2011
(0018,9052)	FD	SpectralWidth	1-2	DICOM_2011
(0018,9053)	FD	ChemicalShiftReference	1-2	DICOM_2011
(0018,9054)	CS	VolumeLocalizationTechnique	1	DICOM_2011
(0018,9058)	US	MRAcquisitionFrequencyEncodingSteps	1	DICOM_2011
(0018,9059)	CS	Decoupling	1	DICOM_2011
(0018,9060)	CS	DecoupledNucleus	1-2	DICOM_2011
(0018,9061)	FD	DecouplingFrequency	1-2	DICOM_2011
(0018,9062)	CS	DecouplingMethod	1	DICOM_2011
(0018,9063)	FD	DecouplingChemicalShiftReference	1-2	DICOM_2011
(0018,9064)	CS	KSpaceFiltering	1	DICOM_2011
(0018,9065)	CS	TimeDomainFiltering	1-2	DICOM_2011
(0018,9066)	US	NumberOfZeroFills	1-2	DICOM_2011
(0018,9067)	CS	BaselineCorrection	1	DICOM_2011
(0018,9069)	FD	ParallelReductionFactorInPlane	1	DICOM_2011
(0018,9070)	FD	CardiacRRIntervalSpecified	1	DICOM_2011
(0018,9073)	FD	AcquisitionDuration	1	DICOM_2011
(0018,9074)	DT	FrameAcquisitionDateTime	1	DICOM_2011
(0018,9075)	CS	DiffusionDirectionality	1	DICOM_2011
(0018,9076)	SQ	DiffusionGradientDirectionSequence	1	DICOM_2011
(0018,9077)	CS	ParallelAcquisition	1	DICOM_2011
(0018,9078)	CS	ParallelAcquisitionTechnique	1	DICOM_2011
(0018,9079)	FD	InversionTimes	1-n	DICOM_2011
(0018,9080)	ST	MetaboliteMapDescription	1	DICOM_2011
(0018,9081)	CS	PartialFourier	1	DICOM_2011
(0018,9082)	FD	EffectiveEchoTime	1	DICOM_2011
(0018,9083)	SQ	MetaboliteMapCodeSequence	1	DICOM_2011
(0018,9084)	SQ	ChemicalShiftSequence	1	DICOM_2011
(0018,9085)	CS	CardiacSignalSource	1	DICOM_2011
(0018,9087)	FD	DiffusionBValue	1	DICOM_2011
(0018,9089)	FD	DiffusionGradientOrientation	3	DICOM_2011
(0018,9090)	FD	VelocityEncodingDirection	3	DICOM_2011
(0018,9091)	FD	VelocityEncodingMinimumValue	1	DICOM_2011
(0018,9092)	SQ	VelocityEncodingAcquisitionSequence	1	DICOM_2011
(0018,9093)	US	NumberOfKSpaceTrajectories	1	DICOM_2011
(0018,9094)	CS	CoverageOfKSpace	1	DICOM_2011
(0018,9095)	UL	SpectroscopyAcquisitionPhaseRows	1	DICOM_2011
(0018,9098)	FD	TransmitterFrequency	1-2	DICOM_2011
(0018,9100)	CS	ResonantNucleus	1-2	DICOM_2011
(0018,9101)	CS	FrequencyCorrection	1	DICOM_2011
(0018,9103)	SQ	MRSpectroscopyFOVGeometrySequence	1	DICOM_2011
(0018,9104)	FD	SlabThickness	1	DICOM_2011
(0018,9105)	FD	SlabOrientation	3	DICOM_2011
(0018,9106)	FD	MidSlabPosition	3	DICOM_2011
(0018,9107)	SQ	MRSpatialSaturationSequence	1	DICOM_2011
(0018,9112)	SQ	MRTimingAndRelatedParametersSequence	1	DICOM_2011
(0018,9114)	SQ	MREchoSequence	1	DICOM_2011
(0018,9115)	SQ	MRModifierSequence	1	DICOM_2011
(0018,9117)	SQ	MRDiffusionSequence	1	DICOM_2011
(0018,9118)	SQ	CardiacSynchronizationSequence	1	DICOM_2011
(0018,9119)	SQ	MRAveragesSequence	1	DICOM_2011
(0018,9125)	SQ	MRFOVGeometrySequence	1	DICOM_2011
(0018,9126)	SQ	VolumeLocalizationSequence	1	DICOM_2011
(0018,9127)	UL	SpectroscopyAcquisitionDataColumns	1	DICOM_2011
(0018,9147)	CS	DiffusionAnisotropyType	1	DICOM_2011
(0018,9151)	DT	FrameReferenceDateTime	1	DICOM_2011
(0018,9152)	SQ	MRMetaboliteMapSequence	1	DICOM_2011
(0018,9155)	FD	ParallelReductionFactorOutOfPlane	1	DICOM_2011
(0018,9159)	UL	SpectroscopyAcquisitionOutOfPlanePhaseSteps	1	DICOM_2011
(0018,9168)	FD	ParallelReductionFactorSecondInPlane	1	DICOM_2011
(0018,9169)	CS	CardiacBeatRejectionTechnique	1	DICOM_2011
(0018,9170)	CS	RespiratoryMotionCompensationTechnique	1	DICOM_2011
(0018,9171)	CS	RespiratorySignalSource	1	DICOM_2011
(0018,9172)	CS	BulkMotionCompensationTechnique	1	DICOM_2011
(0018,9173)	CS	BulkMotionSignalSource	1	DICOM_2011
(0018,9174)	CS	ApplicableSafetyStandardAgency	1	DICOM_2011
(0018,9175)	LO	ApplicableSafetyStandardDescription	1	DICOM_2011
(0018,9176)	SQ	OperatingModeSequence	1	DICOM_2011
(0018,9177)	CS	OperatingModeType	1	DICOM_2011
(0018,9178)	CS	OperatingMode	1	DICOM_2011
(0018,9179)	CS	SpecificAbsorptionRateDefinition	1	DICOM_2011
(0018,9180)	CS	GradientOutputType	1	DICOM_2011
(0018,9181)	FD	SpecificAbsorptionRateValue	1	DICOM_2011
(0018,9182)	FD	GradientOutput	1	DICOM_2011
(0018,9183)	CS	FlowCompensationDirection	1	DICOM_2011
(0018,9184)	FD	TaggingDelay	1	DICOM_2011
(0018,9185)	ST	RespiratoryMotionCompensationTechniqueDescription	1	DICOM_2011
(0018,9186)	SH	RespiratorySignalSourceID	1	DICOM_2011
(0018,9197)	SQ	MRVelocityEncodingSequence	1	DICOM_2011
(0018,9198)	CS	FirstOrderPhaseCorrection	1	DICOM_2011
(0018,9199)	CS	WaterReferencedPhaseCorrection	1	DICOM_2011
(0018,9200)	CS	MRSpectroscopyAcquisitionType	1	DICOM_2011
(0018,9214)	CS	RespiratoryCyclePosition	1	DICOM_2011
(0018,9217)	FD	VelocityEncodingMaximumValue	1	DICOM_2011
(0018,9218)	FD	TagSpacingSecondDimension	1	DICOM_2011
(0018,9219)	SS	TagAngleSecondAxis	1	DICOM_2011
(0018,9220)	FD	FrameAcquisitionDuration	1	DICOM_2011
(0018,9226)	SQ	MRImageFrameTypeSequence	1	DICOM_2011
(0018,9227)	SQ	MRSpectroscopyFrameTypeSequence	1	DICOM_2011
(0018,9231)	US	MRAcquisitionPhaseEncodingStepsInPlane	1	DICOM_2011
(0018,9232)	US	MRAcquisitionPhaseEncodingStepsOutOfPlane	1	DICOM_2011
(0018,9234)	UL	SpectroscopyAcquisitionPhaseColumns	1	DICOM_2011
(0018,9236)	CS	CardiacCyclePosition	1	DICOM_2011
(0018,9239)	SQ	SpecificAbsorptionRateSequence	1	DICOM_2011
(0018,9240)	US	RFEchoTrainLength	1	DICOM_2011
(0018,9241)	US	GradientEchoTrainLength	1	DICOM_2011
(0018,9250)	CS	ArterialSpinLabelingContrast	1	DICOM_2011
(0018,9251)	SQ	MRArterialSpinLabelingSequence	1	DICOM_2011
(0018,9252)	LO	ASLTechniqueDescription	1	DICOM_2011
(0018,9253)	US	ASLSlabNumber	1	DICOM_2011
(0018,9254)	FD	ASLSlabThickness	1	DICOM_2011
(0018,9255)	FD	ASLSlabOrientation	3	DICOM_2011
(0018,9256)	FD	ASLMidSlabPosition	3	DICOM_2011
(0018,9257)	CS	ASLContext	1	DICOM_2011
(0018,9258)	UL	ASLPulseTrainDuration	1	DICOM_2011
(0018,9259)	CS	ASLCrusherFlag	1	DICOM_2011
(0018,925A)	FD	ASLCrusherFlow	1	DICOM_2011
(0018,925B)	LO	ASLCrusherDescription	1	DICOM_2011
(0018,925C)	CS	ASLBolusCutoffFlag	1	DICOM_2011
(0018,925D)	SQ	ASLBolusCutoffTimingSequence	1	DICOM_2011
(0018,925E)	LO	ASLBolusCutoffTechnique	1	DICOM_2011
(0018,925F)	UL	ASLBolusCutoffDelayTime	1	DICOM_2011
(0018,9260)	SQ	ASLSlabSequence	1	DICOM_2011
(0018,9295)	FD	ChemicalShiftMinimumIntegrationLimitInppm	1	DICOM_2011
(0018,9296)	FD	ChemicalShiftMaximumIntegrationLimitInppm	1	DICOM_2011
(0018,9301)	SQ	CTAcquisitionTypeSequence	1	DICOM_2011
(0018,9302)	CS	AcquisitionType	1	DICOM_2011
(0018,9303)	FD	TubeAngle	1	DICOM_2011
(0018,9304)	SQ	CTAcquisitionDetailsSequence	1	DICOM_2011
(0018,9305)	FD	RevolutionTime	1	DICOM_2011
(0018,9306)	FD	SingleCollimationWidth	1	DICOM_2011
(0018,9307)	FD	TotalCollimationWidth	1	DICOM_2011
(0018,9308)	SQ	CTTableDynamicsSequence	1	DICOM_2011
(0018,9309)	FD	TableSpeed	1	DICOM_2011
(0018,9310)	FD	TableFeedPerRotation	1	DICOM_2011
(0018,9311)	FD	SpiralPitchFactor	1	DICOM_2011
(0018,9312)	SQ	CTGeometrySequence	1	DICOM_2011
(0018,9313)	FD	DataCollectionCenterPatient	3	DICOM_2011
(0018,9314)	SQ	CTReconstructionSequence	1	DICOM_2011
(0018,9315)	CS	ReconstructionAlgorithm	1	DICOM_2011
(0018,9316)	CS	ConvolutionKernelGroup	1	DICOM_2011
(0018,9317)	FD	ReconstructionFieldOfView	2	DICOM_2011
(0018,9318)	FD	ReconstructionTargetCenterPatient	3	DICOM_2011
(0018,9319)	FD	ReconstructionAngle	1	DICOM_2011
(0018,9320)	SH	ImageFilter	1	DICOM_2011
(0018,9321)	SQ	CTExposureSequence	1	DICOM_2011
(0018,9322)	FD	ReconstructionPixelSpacing	2	DICOM_2011
(0018,9323)	CS	ExposureModulationType	1	DICOM_2011
(0018,9324)	FD	EstimatedDoseSaving	1	DICOM_2011
(0018,9325)	SQ	CTXRayDetailsSequence	1	DICOM_2011
(0018,9326)	SQ	CTPositionSequence	1	DICOM_2011
(0018,9327)	FD	TablePosition	1	DICOM_2011
(0018,9328)	FD	ExposureTimeInms	1	DICOM_2011
(0018,9329)	SQ	CTImageFrameTypeSequence	1	DICOM_2011
(0018,9330)	FD	XRayTubeCurrentInmA	1	DICOM_2011
(0018,9332)	FD	ExposureInmAs	1	DICOM_2011
(0018,9333)	CS	ConstantVolumeFlag	1	DICOM_2011
(0018,9334)	CS	FluoroscopyFlag	1	DICOM_2011
(0018,9335)	FD	DistanceSourceToDataCollectionCenter	1	DICOM_2011
(0018,9337)	US	ContrastBolusAgentNumber	1	DICOM_2011
(0018,9338)	SQ	ContrastBolusIngredientCodeSequence	1	DICOM_2011
(0018,9340)	SQ	ContrastAdministrationProfileSequence	1	DICOM_2011
(0018,9341)	SQ	ContrastBolusUsageSequence	1	DICOM_2011
(0018,9342)	CS	ContrastBolusAgentAdministered	1	DICOM_2011
(0018,9343)	CS	ContrastBolusAgentDetected	1	DICOM_2011
(0018,9344)	CS	ContrastBolusAgentPhase	1	DICOM_2011
(0018,9345)	FD	CTDIvol	1	DICOM_2011
(0018,9346)	SQ	CTDIPhantomTypeCodeSequence	1	DICOM_2011
(0018,9351)	FL	CalciumScoringMassFactorPatient	1	DICOM_2011
(0018,9352)	FL	CalciumScoringMassFactorDevice	3	DICOM_2011
(0018,9353)	FL	EnergyWeightingFactor	1	DICOM_2011
(0018,9360)	SQ	CTAdditionalXRaySourceSequence	1	DICOM_2011
(0018,9401)	SQ	ProjectionPixelCalibrationSequence	1	DICOM_2011
(0018,9402)	FL	DistanceSourceToIsocenter	1	DICOM_2011
(0018,9403)	FL	DistanceObjectToTableTop	1	DICOM_2011
(0018,9404)	FL	ObjectPixelSpacingInCenterOfBeam	2	DICOM_2011
(0018,9405)	SQ	PositionerPositionSequence	1	DICOM_2011
(0018,9406)	SQ	TablePositionSequence	1	DICOM_2011
(0018,9407)	SQ	CollimatorShapeSequence	1	DICOM_2011
(0018,9410)	CS	PlanesInAcquisition	1	DICOM_2011
(0018,9412)	SQ	XAXRFFrameCharacteristicsSequence	1	DICOM_2011
(0018,9417)	SQ	FrameAcquisitionSequence	1	DICOM_2011
(0018,9420)	CS	XRayReceptorType	1	DICOM_2011
(0018,9423)	LO	AcquisitionProtocolName	1	DICOM_2011
(0018,9424)	LT	AcquisitionProtocolDescription	1	DICOM_2011
(0018,9425)	CS	ContrastBolusIngredientOpaque	1	DICOM_2011
(0018,9426)	FL	DistanceReceptorPlaneToDetectorHousing	1	DICOM_2011
(0018,9427)	CS	IntensifierActiveShape	1	DICOM_2011
(0018,9428)	FL	IntensifierActiveDimensions	1-2	DICOM_2011
(0018,9429)	FL	PhysicalDetectorSize	2	DICOM_2011
(0018,9430)	FL	PositionOfIsocenterProjection	2	DICOM_2011
(0018,9432)	SQ	FieldOfViewSequence	1	DICOM_2011
(0018,9433)	LO	FieldOfViewDescription	1	DICOM_2011
(0018,9434)	SQ	ExposureControlSensingRegionsSequence	1	DICOM_2011
(0018,9435)	CS	ExposureControlSensingRegionShape	1	DICOM_2011
(0018,9436)	SS	ExposureControlSensingRegionLeftVerticalEdge	1	DICOM_2011
(0018,9437)	SS	ExposureControlSensingRegionRightVerticalEdge	1	DICOM_2011
(0018,9438)	SS	ExposureControlSensingRegionUpperHorizontalEdge	1	DICOM_2011
(0018,9439)	SS	ExposureControlSensingRegionLowerHorizontalEdge	1	DICOM_2011
(0018,9440)	SS	CenterOfCircularExposureControlSensingRegion	2	DICOM_2011
(0018,9441)	US	RadiusOfCircularExposureControlSensingRegion	1	DICOM_2011
(0018,9442)	SS	VerticesOfThePolygonalExposureControlSensingRegion	2-n	DICOM_2011
(0018,9447)	FL	ColumnAngulationPatient	1	DICOM_2011
(0018,9449)	FL	BeamAngle	1	DICOM_2011
(0018,9451)	SQ	FrameDetectorParametersSequence	1	DICOM_2011
(0018,9452)	FL	CalculatedAnatomyThickness	1	DICOM_2011
(0018,9455)	SQ	CalibrationSequence	1	DICOM_2011
(0018,9456)	SQ	ObjectThicknessSequence	1	DICOM_2011
(0018,9457)	CS	PlaneIdentification	1	DICOM_2011
(0018,9461)	FL	FieldOfViewDimensionsInFloat	1-2	DICOM_2011
(0018,9462)	SQ	IsocenterReferenceSystemSequence	1	DICOM_2011
(0018,9463)	FL	PositionerIsocenterPrimaryAngle	1	DICOM_2011
(0018,9464)	FL	PositionerIsocenterSecondaryAngle	1	DICOM_2011
(0018,9465)	FL	PositionerIsocenterDetectorRotationAngle	1	DICOM_2011
(0018,9466)	FL	TableXPositionToIsocenter	1	DICOM_2011
(0018,9467)	FL	TableYPositionToIsocenter	1	DICOM_2011
(0018,9468)	FL	TableZPositionToIsocenter	1	DICOM_2011
(0018,9469)	FL	TableHorizontalRotationAngle	1	DICOM_2011
(0018,9470)	FL	TableHeadTiltAngle	1	DICOM_2011
(0018,9471)	FL	TableCradleTiltAngle	1	DICOM_2011
(0018,9472)	SQ	FrameDisplayShutterSequence	1	DICOM_2011
(0018,9473)	FL	AcquiredImageAreaDoseProduct	1	DICOM_2011
(0018,9474)	CS	CArmPositionerTabletopRelationship	1	DICOM_2011
(0018,9476)	SQ	XRayGeometrySequence	1	DICOM_2011
(0018,9477)	SQ	IrradiationEventIdentificationSequence	1	DICOM_2011
(0018,9504)	SQ	XRay3DFrameTypeSequence	1	DICOM_2011
(0018,9506)	SQ	ContributingSourcesSequence	1	DICOM_2011
(0018,9507)	SQ	XRay3DAcquisitionSequence	1	DICOM_2011
(0018,9508)	FL	PrimaryPositionerScanArc	1	DICOM_2011
(0018,9509)	FL	SecondaryPositionerScanArc	1	DICOM_2011
(0018,9510)	FL	PrimaryPositionerScanStartAngle	1	DICOM_2011
(0018,9511)	FL	SecondaryPositionerScanStartAngle	1	DICOM_2011
(0018,9514)	FL	PrimaryPositionerIncrement	1	DICOM_2011
(0018,9515)	FL	SecondaryPositionerIncrement	1	DICOM_2011
(0018,9516)	DT	StartAcquisitionDateTime	1	DICOM_2011
(0018,9517)	DT	EndAcquisitionDateTime	1	DICOM_2011
(0018,9524)	LO	ApplicationName	1	DICOM_2011
(0018,9525)	LO	ApplicationVersion	1	DICOM_2011
(0018,9526)	LO	ApplicationManufacturer	1	DICOM_2011
(0018,9527)	CS	AlgorithmType	1	DICOM_2011
(0018,9528)	LO	AlgorithmDescription	1	DICOM_2011
(0018,9530)	SQ	XRay3DReconstructionSequence	1	DICOM_2011
(0018,9531)	LO	ReconstructionDescription	1	DICOM_2011
(0018,9538)	SQ	PerProjectionAcquisitionSequence	1	DICOM_2011
(0018,9601)	SQ	DiffusionBMatrixSequence	1	DICOM_2011
(0018,9602)	FD	DiffusionBValueXX	1	DICOM_2011
(0018,9603)	FD	DiffusionBValueXY	1	DICOM_2011
(0018,9604)	FD	DiffusionBValueXZ	1	DICOM_2011
(0018,9605)	FD	DiffusionBValueYY	1	DICOM_2011
(0018,9606)	FD	DiffusionBValueYZ	1	DICOM_2011
(0018,9607)	FD	DiffusionBValueZZ	1	DICOM_2011
(0018,9701)	DT	DecayCorrectionDateTime	1	DICOM_2011
(0018,9715)	FD	StartDensityThreshold	1	DICOM_2011
(0018,9716)	FD	StartRelativeDensityDifferenceThreshold	1	DICOM_2011
(0018,9717)	FD	StartCardiacTriggerCountThreshold	1	DICOM_2011
(0018,9718)	FD	StartRespiratoryTriggerCountThreshold	1	DICOM_2011
(0018,9719)	FD	TerminationCountsThreshold	1	DICOM_2011
(0018,9720)	FD	TerminationDensityThreshold	1	DICOM_2011
(0018,9721)	FD	TerminationRelativeDensityThreshold	1	DICOM_2011
(0018,9722)	FD	TerminationTimeThreshold	1	DICOM_2011
(0018,9723)	FD	TerminationCardiacTriggerCountThreshold	1	DICOM_2011
(0018,9724)	FD	TerminationRespiratoryTriggerCountThreshold	1	DICOM_2011
(0018,9725)	CS	DetectorGeometry	1	DICOM_2011
(0018,9726)	FD	TransverseDetectorSeparation	1	DICOM_2011
(0018,9727)	FD	AxialDetectorDimension	1	DICOM_2011
(0018,9729)	US	RadiopharmaceuticalAgentNumber	1	DICOM_2011
(0018,9732)	SQ	PETFrameAcquisitionSequence	1	DICOM_2011
(0018,9733)	SQ	PETDetectorMotionDetailsSequence	1	DICOM_2011
(0018,9734)	SQ	PETTableDynamicsSequence	1	DICOM_2011
(0018,9735)	SQ	PETPositionSequence	1	DICOM_2011
(0018,9736)	SQ	PETFrameCorrectionFactorsSequence	1	DICOM_2011
(0018,9737)	SQ	RadiopharmaceuticalUsageSequence	1	DICOM_2011
(0018,9738)	CS	AttenuationCorrectionSource	1	DICOM_2011
(0018,9739)	US	NumberOfIterations	1	DICOM_2011
(0018,9740)	US	NumberOfSubsets	1	DICOM_2011
(0018,9749)	SQ	PETReconstructionSequence	1	DICOM_2011
(0018,9751)	SQ	PETFrameTypeSequence	1	DICOM_2011
(0018,9755)	CS	TimeOfFlightInformationUsed	1	DICOM_2011
(0018,9756)	CS	ReconstructionType	1	DICOM_2011
(0018,9758)	CS	DecayCorrected	1	DICOM_2011
(0018,9759)	CS	AttenuationCorrected	1	DICOM_2011
(0018,9760)	CS	ScatterCorrected	1	DICOM_2011
(0018,9761)	CS	DeadTimeCorrected	1	DICOM_2011
(0018,9762)	CS	GantryMotionCorrected	1	DICOM_2011
(0018,9763)	CS	PatientMotionCorrected	1	DICOM_2011
(0018,9764)	CS	CountLossNormalizationCorrected	1	DICOM_2011
(0018,9765)	CS	RandomsCorrected	1	DICOM_2011
(0018,9766)	CS	NonUniformRadialSamplingCorrected	1	DICOM_2011
(0018,9767)	CS	SensitivityCalibrated	1	DICOM_2011
(0018,9768)	CS	DetectorNormalizationCorrection	1	DICOM_2011
(0018,9769)	CS	IterativeReconstructionMethod	1	DICOM_2011
(0018,9770)	CS	AttenuationCorrectionTemporalRelationship	1	DICOM_2011
(0018,9771)	SQ	PatientPhysiologicalStateSequence	1	DICOM_2011
(0018,9772)	SQ	PatientPhysiologicalStateCodeSequence	1	DICOM_2011
(0018,9801)	FD	DepthsOfFocus	1-n	DICOM_2011
(0018,9803)	SQ	ExcludedIntervalsSequence	1	DICOM_2011
(0018,9804)	DT	ExclusionStartDatetime	1	DICOM_2011
(0018,9805)	FD	ExclusionDuration	1	DICOM_2011
(0018,9806)	SQ	USImageDescriptionSequence	1	DICOM_2011
(0018,9807)	SQ	ImageDataTypeSequence	1	DICOM_2011
(0018,9808)	CS	DataType	1	DICOM_2011
(0018,9809)	SQ	TransducerScanPatternCodeSequence	1	DICOM_2011
(0018,980B)	CS	AliasedDataType	1	DICOM_2011
(0018,980C)	CS	PositionMeasuringDeviceUsed	1	DICOM_2011
(0018,980D)	SQ	TransducerGeometryCodeSequence	1	DICOM_2011
(0018,980E)	SQ	TransducerBeamSteeringCodeSequence	1	DICOM_2011
(0018,980F)	SQ	TransducerApplicationCodeSequence	1	DICOM_2011
(0018,A001)	SQ	ContributingEquipmentSequence	1	DICOM_2011
(0018,A002)	DT	ContributionDateTime	1	DICOM_2011
(0018,A003)	ST	ContributionDescription	1	DICOM_2011
(0020,000D)	UI	StudyInstanceUID	1	DICOM_2011
(0020,000E)	UI	SeriesInstanceUID	1	DICOM_2011
(0020,0010)	SH	StudyID	1	DICOM_2011
(0020,0011)	IS	SeriesNumber	1	DICOM_2011
(0020,0012)	IS	AcquisitionNumber	1	DICOM_2011
(0020,0013)	IS	InstanceNumber	1	DICOM_2011
(0020,0019)	IS	ItemNumber	1	DICOM_2011
(0020,0020)	CS	PatientOrientation	2	DICOM_2011
(0020,0032)	DS	ImagePositionPatient	3	DICOM_2011
(0020,0037)	DS	ImageOrientationPatient	6	DICOM_2011
(0020,0052)	UI	FrameOfReferenceUID	1	DICOM_2011
(0020,0060)	CS	Laterality	1	DICOM_2011
(0020,0062)	CS	ImageLaterality	1	DICOM_2011
(0020,0100)	IS	TemporalPositionIdentifier	1	DICOM_2011
(0020,0105)	IS	NumberOfTemporalPositions	1	DICOM_2011
(0020,0110)	DS	TemporalResolution	1	DICOM_2011
(0020,0200)	UI	SynchronizationFrameOfReferenceUID	1	DICOM_2011
(0020,0242)	UI	SOPInstanceUIDOfConcatenationSource	1	DICOM_2011
(0020,1002)	IS	ImagesInAcquisition	1	DICOM_2011
(0020,1040)	LO	PositionReferenceIndicator	1	DICOM_2011
(0020,1041)	DS	SliceLocation	1	DICOM_2011
(0020,1200)	IS	NumberOfPatientRelatedStudies	1	DICOM_2011
(0020,1202)	IS	NumberOfPatientRelatedSeries	1	DICOM_2011
(0020,1204)	IS	NumberOfPatientRelatedInstances	1	DICOM_2011
(0020,1206)	IS	NumberOfStudyRelatedSeries	1	DICOM_2011
(0020,1208)	IS	NumberOfStudyRelatedInstances	1	DICOM_2011
(0020,1209)	IS	NumberOfSeriesRelatedInstances	1	DICOM_2011
(0020,4000)	LT	ImageComments	1	DICOM_2011
(0020,9056)	SH	StackID	1	DICOM_2011
(0020,9057)	UL	InStackPositionNumber	1	DICOM_2011
(0020,9071)	SQ	FrameAnatomySequence	1	DICOM_2011
(0020,9072)	CS	FrameLaterality	1	DICOM_2011
(0020,9111)	SQ	FrameContentSequence	1	DICOM_2011
(0020,9113)	SQ	PlanePositionSequence	1	DICOM_2011
(0020,9116)	SQ	PlaneOrientationSequence	1	DICOM_2011
(0020,9128)	UL	TemporalPositionIndex	1	DICOM_2011
(0020,9153)	FD	NominalCardiacTriggerDelayTime	1	DICOM_2011
(0020,9154)	FL	NominalCardiacTriggerTimePriorToRPeak	1	DICOM_2011
(0020,9155)	FL	ActualCardiacTriggerTimePriorToRPeak	1	DICOM_2011
(0020,9156)	US	FrameAcquisitionNumber	1	DICOM_2011
(0020,9157)	UL	DimensionIndexValues	1-n	DICOM_2011
(0020,9158)	LT	FrameComments	1	DICOM_2011
(0020,9161)	UI	ConcatenationUID	1	DICOM_2011
(0020,9162)	US	InConcatenationNumber	1	DICOM_2011
(0020,9163)	US	InConcatenationTotalNumber	1	DICOM_2011
(0020,9164)	UI	DimensionOrganizationUID	1	DICOM_2011
(0020,9165)	AT	DimensionIndexPointer	1	DICOM_2011
(0020,9167)	AT	FunctionalGroupPointer	1	DICOM_2011
(0020,9213)	LO	DimensionIndexPrivateCreator	1	DICOM_2011
(0020,9221)	SQ	DimensionOrganizationSequence	1	DICOM_2011
(0020,9222)	SQ	DimensionIndexSequence	1	DICOM_2011
(0020,9228)	UL	ConcatenationFrameOffsetNumber	1	DICOM_2011
(0020,9238)	LO	FunctionalGroupPrivateCreator	1	DICOM_2011
(0020,9241)	FL	NominalPercentageOfCardiacPhase	1	DICOM_2011
(0020,9245)	FL	NominalPercentageOfRespiratoryPhase	1	DICOM_2011
(0020,9246)	FL	StartingRespiratoryAmplitude	1	DICOM_2011
(0020,9247)	CS	StartingRespiratoryPhase	1	DICOM_2011
(0020,9248)	FL	EndingRespiratoryAmplitude	1	DICOM_2011
(0020,9249)	CS	EndingRespiratoryPhase	1	DICOM_2011
(0020,9250)	CS	RespiratoryTriggerType	1	DICOM_2011
(0020,9251)	FD	RRIntervalTimeNominal	1	DICOM_2011
(0020,9252)	FD	ActualCardiacTriggerDelayTime	1	DICOM_2011
(0020,9253)	SQ	RespiratorySynchronizationSequence	1	DICOM_2011
(0020,9254)	FD	RespiratoryIntervalTime	1	DICOM_2011
(0020,9255)	FD	NominalRespiratoryTriggerDelayTime	1	DICOM_2011
(0020,9256)	FD	RespiratoryTriggerDelayThreshold	1	DICOM_2011
(0020,9257)	FD	ActualRespiratoryTriggerDelayTime	1	DICOM_2011
(0020,9301)	FD	ImagePositionVolume	3	DICOM_2011
(0020,9302)	FD	ImageOrientationVolume	6	DICOM_2011
(0020,9307)	CS	UltrasoundAcquisitionGeometry	1	DICOM_2011
(0020,9308)	FD	ApexPosition	3	DICOM_2011
(0020,9309)	FD	VolumeToTransducerMappingMatrix	16	DICOM_2011
(0020,930A)	FD	VolumeToTableMappingMatrix	16	DICOM_2011
(0020,930C)	CS	PatientFrameOfReferenceSource	1	DICOM_2011
(0020,930D)	FD	TemporalPositionTimeOffset	1	DICOM_2011
(0020,930E)	SQ	PlanePositionVolumeSequence	1	DICOM_2011
(0020,930F)	SQ	PlaneOrientationVolumeSequence	1	DICOM_2011
(0020,9310)	SQ	TemporalPositionSequence	1	DICOM_2011
(0020,9311)	CS	DimensionOrganizationType	1	DICOM_2011
(0020,9312)	UI	VolumeFrameOfReferenceUID	1	DICOM_2011
(0020,9313)	UI	TableFrameOfReferenceUID	1	DICOM_2011
(0020,9421)	LO	DimensionDescriptionLabel	1	DICOM_2011
(0020,9450)	SQ	PatientOrientationInFrameSequence	1	DICOM_2011
(0020,9453)	LO	FrameLabel	1	DICOM_2011
(0020,9518)	US	AcquisitionIndex	1-n	DICOM_2011
(0020,9529)	SQ	ContributingSOPInstancesReferenceSequence	1	DICOM_2011
(0020,9536)	US	ReconstructionIndex	1	DICOM_2011
(0022,0001)	US	LightPathFilterPassThroughWavelength	1	DICOM_2011
(0022,0002)	US	LightPathFilterPassBand	2	DICOM_2011
(0022,0003)	US	ImagePathFilterPassThroughWavelength	1	DICOM_2011
(0022,0004)	US	ImagePathFilterPassBand	2	DICOM_2011
(0022,0005)	CS	PatientEyeMovementCommanded	1	DICOM_2011
(0022,0006)	SQ	PatientEyeMovementCommandCodeSequence	1	DICOM_2011
(0022,0007)	FL	SphericalLensPower	1	DICOM_2011
(0022,0008)	FL	CylinderLensPower	1	DICOM_2011
(0022,0009)	FL	CylinderAxis	1	DICOM_2011
(0022,000A)	FL	EmmetropicMagnification	1	DICOM_2011
(0022,000B)	FL	IntraOcularPressure	1	DICOM_2011
(0022,000C)	FL	HorizontalFieldOfView	1	DICOM_2011
(0022,000D)	CS	PupilDilated	1	DICOM_2011
(0022,000E)	FL	DegreeOfDilation	1	DICOM_2011
(0022,0010)	FL	StereoBaselineAngle	1	DICOM_2011
(0022,0011)	FL	StereoBaselineDisplacement	1	DICOM_2011
(0022,0012)	FL	StereoHorizontalPixelOffset	1	DICOM_2011
(0022,0013)	FL	StereoVerticalPixelOffset	1	DICOM_2011
(0022,0014)	FL	StereoRotation	1	DICOM_2011
(0022,0015)	SQ	AcquisitionDeviceTypeCodeSequence	1	DICOM_2011
(0022,0016)	SQ	IlluminationTypeCodeSequence	1	DICOM_2011
(0022,0017)	SQ	LightPathFilterTypeStackCodeSequence	1	DICOM_2011
(0022,0018)	SQ	ImagePathFilterTypeStackCodeSequence	1	DICOM_2011
(0022,0019)	SQ	LensesCodeSequence	1	DICOM_2011
(0022,001A)	SQ	ChannelDescriptionCodeSequence	1	DICOM_2011
(0022,001B)	SQ	RefractiveStateSequence	1	DICOM_2011
(0022,001C)	SQ	MydriaticAgentCodeSequence	1	DICOM_2011
(0022,001D)	SQ	RelativeImagePositionCodeSequence	1	DICOM_2011
(0022,001E)	FL	CameraAngleOfView	1	DICOM_2011
(0022,0020)	SQ	StereoPairsSequence	1	DICOM_2011
(0022,0021)	SQ	LeftImageSequence	1	DICOM_2011
(0022,0022)	SQ	RightImageSequence	1	DICOM_2011
(0022,0030)	FL	AxialLengthOfTheEye	1	DICOM_2011
(0022,0031)	SQ	OphthalmicFrameLocationSequence	1	DICOM_2011
(0022,0032)	FL	ReferenceCoordinates	2-2n	DICOM_2011
(0022,0035)	FL	DepthSpatialResolution	1	DICOM_2011
(0022,0036)	FL	MaximumDepthDistortion	1	DICOM_2011
(0022,0037)	FL	AlongScanSpatialResolution	1	DICOM_2011
(0022,0038)	FL	MaximumAlongScanDistortion	1	DICOM_2011
(0022,0039)	CS	OphthalmicImageOrientation	1	DICOM_2011
(0022,0041)	FL	DepthOfTransverseImage	1	DICOM_2011
(0022,0042)	SQ	MydriaticAgentConcentrationUnitsSequence	1	DICOM_2011
(0022,0048)	FL	AcrossScanSpatialResolution	1	DICOM_2011
(0022,0049)	FL	MaximumAcrossScanDistortion	1	DICOM_2011
(0022,004E)	DS	MydriaticAgentConcentration	1	DICOM_2011
(0022,0055)	FL	IlluminationWaveLength	1	DICOM_2011
(0022,0056)	FL	IlluminationPower	1	DICOM_2011
(0022,0057)	FL	IlluminationBandwidth	1	DICOM_2011
(0022,0058)	SQ	MydriaticAgentSequence	1	DICOM_2011
(0022,1007)	SQ	OphthalmicAxialMeasurementsRightEyeSequence	1	DICOM_2011
(0022,1008)	SQ	OphthalmicAxialMeasurementsLeftEyeSequence	1	DICOM_2011
(0022,1009)	CS	OphthalmicAxialMeasurementsDeviceType	1	DICOM_2011
(0022,1010)	CS	OphthalmicAxialLengthMeasurementsType	1	DICOM_2011
(0022,1012)	SQ	OphthalmicAxialLengthSequence	1	DICOM_2011
(0022,1019)	FL	OphthalmicAxialLength	1	DICOM_2011
(0022,1024)	SQ	LensStatusCodeSequence	1	DICOM_2011
(0022,1025)	SQ	VitreousStatusCodeSequence	1	DICOM_2011
(0022,1028)	SQ	IOLFormulaCodeSequence	1	DICOM_2011
(0022,1029)	LO	IOLFormulaDetail	1	DICOM_2011
(0022,1033)	FL	KeratometerIndex	1	DICOM_2011
(0022,1035)	SQ	SourceOfOphthalmicAxialLengthCodeSequence	1	DICOM_2011
(0022,1037)	FL	TargetRefraction	1	DICOM_2011
(0022,1039)	CS	RefractiveProcedureOccurred	1	DICOM_2011
(0022,1040)	SQ	RefractiveSurgeryTypeCodeSequence	1	DICOM_2011
(0022,1044)	SQ	OphthalmicUltrasoundMethodCodeSequence	1	DICOM_2011
(0022,1050)	SQ	OphthalmicAxialLengthMeasurementsSequence	1	DICOM_2011
(0022,1053)	FL	IOLPower	1	DICOM_2011
(0022,1054)	FL	PredictedRefractiveError	1	DICOM_2011
(0022,1059)	FL	OphthalmicAxialLengthVelocity	1	DICOM_2011
(0022,1065)	LO	LensStatusDescription	1	DICOM_2011
(0022,1066)	LO	VitreousStatusDescription	1	DICOM_2011
(0022,1090)	SQ	IOLPowerSequence	1	DICOM_2011
(0022,1092)	SQ	LensConstantSequence	1	DICOM_2011
(0022,1093)	LO	IOLManufacturer	1	DICOM_2011
# the following attribute will be retired
(0022,1094)	LO	LensConstantDescription	1	DICOM_2011
(0022,1095)	LO	ImplantName	1	DICOM_2011
(0022,1096)	SQ	KeratometryMeasurementTypeCodeSequence	1	DICOM_2011
(0022,1097)	LO	ImplantPartNumber	1	DICOM_2011
(0022,1100)	SQ	ReferencedOphthalmicAxialMeasurementsSequence	1	DICOM_2011
(0022,1101)	SQ	OphthalmicAxialLengthMeasurementsSegmentNameCodeSequence	1	DICOM_2011
(0022,1103)	SQ	RefractiveErrorBeforeRefractiveSurgeryCodeSequence	1	DICOM_2011
(0022,1121)	FL	IOLPowerForExactEmmetropia	1	DICOM_2011
(0022,1122)	FL	IOLPowerForExactTargetRefraction	1	DICOM_2011
(0022,1125)	SQ	AnteriorChamberDepthDefinitionCodeSequence	1	DICOM_2011
(0022,1127)	SQ	LensThicknessSequence	1	DICOM_2011
(0022,1128)	SQ	AnteriorChamberDepthSequence	1	DICOM_2011
(0022,1130)	FL	LensThickness	1	DICOM_2011
(0022,1131)	FL	AnteriorChamberDepth	1	DICOM_2011
(0022,1132)	SQ	SourceOfLensThicknessDataCodeSequence	1	DICOM_2011
(0022,1133)	SQ	SourceOfAnteriorChamberDepthDataCodeSequence	1	DICOM_2011
(0022,1134)	SQ	SourceOfRefractiveMeasurementsSequence	1	DICOM_2011
(0022,1135)	SQ	SourceOfRefractiveMeasurementsCodeSequence	1	DICOM_2011
(0022,1140)	CS	OphthalmicAxialLengthMeasurementModified	1	DICOM_2011
(0022,1150)	SQ	OphthalmicAxialLengthDataSourceCodeSequence	1	DICOM_2011
# the following attribute will be retired
(0022,1153)	SQ	OphthalmicAxialLengthAcquisitionMethodCodeSequence	1	DICOM_2011
(0022,1155)	FL	SignalToNoiseRatio	1	DICOM_2011
(0022,1159)	LO	OphthalmicAxialLengthDataSourceDescription	1	DICOM_2011
(0022,1210)	SQ	OphthalmicAxialLengthMeasurementsTotalLengthSequence	1	DICOM_2011
(0022,1211)	SQ	OphthalmicAxialLengthMeasurementsSegmentalLengthSequence	1	DICOM_2011
(0022,1212)	SQ	OphthalmicAxialLengthMeasurementsLengthSummationSequence	1	DICOM_2011
(0022,1220)	SQ	UltrasoundOphthalmicAxialLengthMeasurementsSequence	1	DICOM_2011
(0022,1225)	SQ	OpticalOphthalmicAxialLengthMeasurementsSequence	1	DICOM_2011
(0022,1230)	SQ	UltrasoundSelectedOphthalmicAxialLengthSequence	1	DICOM_2011
(0022,1250)	SQ	OphthalmicAxialLengthSelectionMethodCodeSequence	1	DICOM_2011
(0022,1255)	SQ	OpticalSelectedOphthalmicAxialLengthSequence	1	DICOM_2011
(0022,1257)	SQ	SelectedSegmentalOphthalmicAxialLengthSequence	1	DICOM_2011
(0022,1260)	SQ	SelectedTotalOphthalmicAxialLengthSequence	1	DICOM_2011
(0022,1262)	SQ	OphthalmicAxialLengthQualityMetricSequence	1	DICOM_2011
# the following attribute will be retired
(0022,1265)	SQ	OphthalmicAxialLengthQualityMetricTypeCodeSequence	1	DICOM_2011
# the following attribute will be retired
(0022,1273)	LO	OphthalmicAxialLengthQualityMetricTypeDescription	1	DICOM_2011
(0022,1300)	SQ	IntraocularLensCalculationsRightEyeSequence	1	DICOM_2011
(0022,1310)	SQ	IntraocularLensCalculationsLeftEyeSequence	1	DICOM_2011
(0022,1330)	SQ	ReferencedOphthalmicAxialLengthMeasurementQCImageSequence	1	DICOM_2011
(0024,0010)	FL	VisualFieldHorizontalExtent	1	DICOM_2011
(0024,0011)	FL	VisualFieldVerticalExtent	1	DICOM_2011
(0024,0012)	CS	VisualFieldShape	1	DICOM_2011
(0024,0016)	SQ	ScreeningTestModeCodeSequence	1	DICOM_2011
(0024,0018)	FL	MaximumStimulusLuminance	1	DICOM_2011
(0024,0020)	FL	BackgroundLuminance	1	DICOM_2011
(0024,0021)	SQ	StimulusColorCodeSequence	1	DICOM_2011
(0024,0024)	SQ	BackgroundIlluminationColorCodeSequence	1	DICOM_2011
(0024,0025)	FL	StimulusArea	1	DICOM_2011
(0024,0028)	FL	StimulusPresentationTime	1	DICOM_2011
(0024,0032)	SQ	FixationSequence	1	DICOM_2011
(0024,0033)	SQ	FixationMonitoringCodeSequence	1	DICOM_2011
(0024,0034)	SQ	VisualFieldCatchTrialSequence	1	DICOM_2011
(0024,0035)	US	FixationCheckedQuantity	1	DICOM_2011
(0024,0036)	US	PatientNotProperlyFixatedQuantity	1	DICOM_2011
(0024,0037)	CS	PresentedVisualStimuliDataFlag	1	DICOM_2011
(0024,0038)	US	NumberOfVisualStimuli	1	DICOM_2011
(0024,0039)	CS	ExcessiveFixationLossesDataFlag	1	DICOM_2011
(0024,0040)	CS	ExcessiveFixationLosses	1	DICOM_2011
(0024,0042)	US	StimuliRetestingQuantity	1	DICOM_2011
(0024,0044)	LT	CommentsOnPatientPerformanceOfVisualField	1	DICOM_2011
(0024,0045)	CS	FalseNegativesEstimateFlag	1	DICOM_2011
(0024,0046)	FL	FalseNegativesEstimate	1	DICOM_2011
(0024,0048)	US	NegativeCatchTrialsQuantity	1	DICOM_2011
(0024,0050)	US	FalseNegativesQuantity	1	DICOM_2011
(0024,0051)	CS	ExcessiveFalseNegativesDataFlag	1	DICOM_2011
(0024,0052)	CS	ExcessiveFalseNegatives	1	DICOM_2011
(0024,0053)	CS	FalsePositivesEstimateFlag	1	DICOM_2011
(0024,0054)	FL	FalsePositivesEstimate	1	DICOM_2011
(0024,0055)	CS	CatchTrialsDataFlag	1	DICOM_2011
(0024,0056)	US	PositiveCatchTrialsQuantity	1	DICOM_2011
(0024,0057)	CS	TestPointNormalsDataFlag	1	DICOM_2011
(0024,0058)	SQ	TestPointNormalsSequence	1	DICOM_2011
(0024,0059)	CS	GlobalDeviationProbabilityNormalsFlag	1	DICOM_2011
(0024,0060)	US	FalsePositivesQuantity	1	DICOM_2011
(0024,0061)	CS	ExcessiveFalsePositivesDataFlag	1	DICOM_2011
(0024,0062)	CS	ExcessiveFalsePositives	1	DICOM_2011
(0024,0063)	CS	VisualFieldTestNormalsFlag	1	DICOM_2011
(0024,0064)	SQ	ResultsNormalsSequence	1	DICOM_2011
(0024,0065)	SQ	AgeCorrectedSensitivityDeviationAlgorithmSequence	1	DICOM_2011
(0024,0066)	FL	GlobalDeviationFromNormal	1	DICOM_2011
(0024,0067)	SQ	GeneralizedDefectSensitivityDeviationAlgorithmSequence	1	DICOM_2011
(0024,0068)	FL	LocalizedDeviationfromNormal	1	DICOM_2011
(0024,0069)	LO	PatientReliabilityIndicator	1	DICOM_2011
(0024,0070)	FL	VisualFieldMeanSensitivity	1	DICOM_2011
(0024,0071)	FL	GlobalDeviationProbability	1	DICOM_2011
(0024,0072)	CS	LocalDeviationProbabilityNormalsFlag	1	DICOM_2011
(0024,0073)	FL	LocalizedDeviationProbability	1	DICOM_2011
(0024,0074)	CS	ShortTermFluctuationCalculated	1	DICOM_2011
(0024,0075)	FL	ShortTermFluctuation	1	DICOM_2011
(0024,0076)	CS	ShortTermFluctuationProbabilityCalculated	1	DICOM_2011
(0024,0077)	FL	ShortTermFluctuationProbability	1	DICOM_2011
(0024,0078)	CS	CorrectedLocalizedDeviationFromNormalCalculated	1	DICOM_2011
(0024,0079)	FL	CorrectedLocalizedDeviationFromNormal	1	DICOM_2011
(0024,0080)	CS	CorrectedLocalizedDeviationFromNormalProbabilityCalculated	1	DICOM_2011
(0024,0081)	FL	CorrectedLocalizedDeviationFromNormalProbability	1	DICOM_2011
(0024,0083)	SQ	GlobalDeviationProbabilitySequence	1	DICOM_2011
(0024,0085)	SQ	LocalizedDeviationProbabilitySequence	1	DICOM_2011
(0024,0086)	CS	FovealSensitivityMeasured	1	DICOM_2011
(0024,0087)	FL	FovealSensitivity	1	DICOM_2011
(0024,0088)	FL	VisualFieldTestDuration	1	DICOM_2011
(0024,0089)	SQ	VisualFieldTestPointSequence	1	DICOM_2011
(0024,0090)	FL	VisualFieldTestPointXCoordinate	1	DICOM_2011
(0024,0091)	FL	VisualFieldTestPointYCoordinate	1	DICOM_2011
(0024,0092)	FL	AgeCorrectedSensitivityDeviationValue	1	DICOM_2011
(0024,0093)	CS	StimulusResults	1	DICOM_2011
(0024,0094)	FL	SensitivityValue	1	DICOM_2011
(0024,0095)	CS	RetestStimulusSeen	1	DICOM_2011
(0024,0096)	FL	RetestSensitivityValue	1	DICOM_2011
(0024,0097)	SQ	VisualFieldTestPointNormalsSequence	1	DICOM_2011
(0024,0098)	FL	QuantifiedDefect	1	DICOM_2011
(0024,0100)	FL	AgeCorrectedSensitivityDeviationProbabilityValue	1	DICOM_2011
(0024,0102)	CS	GeneralizedDefectCorrectedSensitivityDeviationFlag	1	DICOM_2011
(0024,0103)	FL	GeneralizedDefectCorrectedSensitivityDeviationValue	1	DICOM_2011
(0024,0104)	FL	GeneralizedDefectCorrectedSensitivityDeviationProbabilityValue	1	DICOM_2011
(0024,0105)	FL	MinimumSensitivityValue	1	DICOM_2011
(0024,0106)	CS	BlindSpotLocalized	1	DICOM_2011
(0024,0107)	FL	BlindSpotXCoordinate	1	DICOM_2011
(0024,0108)	FL	BlindSpotYCoordinate	1	DICOM_2011
(0024,0110)	SQ	VisualAcuityMeasurementSequence	1	DICOM_2011
(0024,0112)	SQ	RefractiveParametersUsedOnPatientSequence	1	DICOM_2011
(0024,0113)	CS	MeasurementLaterality	1	DICOM_2011
(0024,0114)	SQ	OphthalmicPatientClinicalInformationLeftEyeSequence	1	DICOM_2011
(0024,0115)	SQ	OphthalmicPatientClinicalInformationRightEyeSequence	1	DICOM_2011
(0024,0117)	CS	FovealPointNormativeDataFlag	1	DICOM_2011
(0024,0118)	FL	FovealPointProbabilityValue	1	DICOM_2011
(0024,0120)	CS	ScreeningBaselineMeasured	1	DICOM_2011
(0024,0122)	SQ	ScreeningBaselineMeasuredSequence	1	DICOM_2011
(0024,0124)	CS	ScreeningBaselineType	1	DICOM_2011
(0024,0126)	FL	ScreeningBaselineValue	1	DICOM_2011
(0024,0202)	LO	AlgorithmSource	1	DICOM_2011
(0024,0306)	LO	DataSetName	1	DICOM_2011
(0024,0307)	LO	DataSetVersion	1	DICOM_2011
(0024,0308)	LO	DataSetSource	1	DICOM_2011
(0024,0309)	LO	DataSetDescription	1	DICOM_2011
(0024,0317)	SQ	VisualFieldTestReliabilityGlobalIndexSequence	1	DICOM_2011
(0024,0320)	SQ	VisualFieldGlobalResultsIndexSequence	1	DICOM_2011
(0024,0325)	SQ	DataObservationSequence	1	DICOM_2011
(0024,0338)	CS	IndexNormalsFlag	1	DICOM_2011
(0024,0341)	FL	IndexProbability	1	DICOM_2011
(0024,0344)	SQ	IndexProbabilitySequence	1	DICOM_2011
(0028,0002)	US	SamplesPerPixel	1	DICOM_2011
(0028,0003)	US	SamplesPerPixelUsed	1	DICOM_2011
(0028,0004)	CS	PhotometricInterpretation	1	DICOM_2011
(0028,0006)	US	PlanarConfiguration	1	DICOM_2011
(0028,0008)	IS	NumberOfFrames	1	DICOM_2011
(0028,0009)	AT	FrameIncrementPointer	1-n	DICOM_2011
(0028,000A)	AT	FrameDimensionPointer	1-n	DICOM_2011
(0028,0010)	US	Rows	1	DICOM_2011
(0028,0011)	US	Columns	1	DICOM_2011
(0028,0014)	US	UltrasoundColorDataPresent	1	DICOM_2011
(0028,0030)	DS	PixelSpacing	2	DICOM_2011
(0028,0031)	DS	ZoomFactor	2	DICOM_2011
(0028,0032)	DS	ZoomCenter	2	DICOM_2011
(0028,0034)	IS	PixelAspectRatio	2	DICOM_2011
(0028,0051)	CS	CorrectedImage	1-n	DICOM_2011
(0028,0100)	US	BitsAllocated	1	DICOM_2011
(0028,0101)	US	BitsStored	1	DICOM_2011
(0028,0102)	US	HighBit	1	DICOM_2011
(0028,0103)	US	PixelRepresentation	1	DICOM_2011
(0028,0106)	xs	SmallestImagePixelValue	1	DICOM_2011
(0028,0107)	xs	LargestImagePixelValue	1	DICOM_2011
(0028,0108)	xs	SmallestPixelValueInSeries	1	DICOM_2011
(0028,0109)	xs	LargestPixelValueInSeries	1	DICOM_2011
(0028,0120)	xs	PixelPaddingValue	1	DICOM_2011
(0028,0121)	xs	PixelPaddingRangeLimit	1	DICOM_2011
(0028,0300)	CS	QualityControlImage	1	DICOM_2011
(0028,0301)	CS	BurnedInAnnotation	1	DICOM_2011
(0028,0302)	CS	RecognizableVisualFeatures	1	DICOM_2011
(0028,0303)	CS	LongitudinalTemporalInformationModified	1	DICOM_2011
(0028,0304)	UI	ReferencedColorPaletteInstanceUID	1	DICOM_2011
(0028,0A02)	CS	PixelSpacingCalibrationType	1	DICOM_2011
(0028,0A04)	LO	PixelSpacingCalibrationDescription	1	DICOM_2011
(0028,1040)	CS	PixelIntensityRelationship	1	DICOM_2011
(0028,1041)	SS	PixelIntensityRelationshipSign	1	DICOM_2011
(0028,1050)	DS	WindowCenter	1-n	DICOM_2011
(0028,1051)	DS	WindowWidth	1-n	DICOM_2011
(0028,1052)	DS	RescaleIntercept	1	DICOM_2011
(0028,1053)	DS	RescaleSlope	1	DICOM_2011
(0028,1054)	LO	RescaleType	1	DICOM_2011
(0028,1055)	LO	WindowCenterWidthExplanation	1-n	DICOM_2011
(0028,1056)	CS	VOILUTFunction	1	DICOM_2011
(0028,1090)	CS	RecommendedViewingMode	1	DICOM_2011
(0028,1101)	xs	RedPaletteColorLookupTableDescriptor	3	DICOM_2011
(0028,1102)	xs	GreenPaletteColorLookupTableDescriptor	3	DICOM_2011
(0028,1103)	xs	BluePaletteColorLookupTableDescriptor	3	DICOM_2011
(0028,1104)	US	AlphaPaletteColorLookupTableDescriptor	3	DICOM_2011
(0028,1199)	UI	PaletteColorLookupTableUID	1	DICOM_2011
(0028,1201)	OW	RedPaletteColorLookupTableData	1	DICOM_2011
(0028,1202)	OW	GreenPaletteColorLookupTableData	1	DICOM_2011
(0028,1203)	OW	BluePaletteColorLookupTableData	1	DICOM_2011
(0028,1204)	OW	AlphaPaletteColorLookupTableData	1	DICOM_2011
(0028,1221)	OW	SegmentedRedPaletteColorLookupTableData	1	DICOM_2011
(0028,1222)	OW	SegmentedGreenPaletteColorLookupTableData	1	DICOM_2011
(0028,1223)	OW	SegmentedBluePaletteColorLookupTableData	1	DICOM_2011
(0028,1300)	CS	BreastImplantPresent	1	DICOM_2011
(0028,1350)	CS	PartialView	1	DICOM_2011
(0028,1351)	ST	PartialViewDescription	1	DICOM_2011
(0028,1352)	SQ	PartialViewCodeSequence	1	DICOM_2011
(0028,135A)	CS	SpatialLocationsPreserved	1	DICOM_2011
(0028,1401)	SQ	DataFrameAssignmentSequence	1	DICOM_2011
(0028,1402)	CS	DataPathAssignment	1	DICOM_2011
(0028,1403)	US	BitsMappedToColorLookupTable	1	DICOM_2011
(0028,1404)	SQ	BlendingLUT1Sequence	1	DICOM_2011
(0028,1405)	CS	BlendingLUT1TransferFunction	1	DICOM_2011
(0028,1406)	FD	BlendingWeightConstant	1	DICOM_2011
(0028,1407)	US	BlendingLookupTableDescriptor	3	DICOM_2011
(0028,1408)	OW	BlendingLookupTableData	1	DICOM_2011
(0028,140B)	SQ	EnhancedPaletteColorLookupTableSequence	1	DICOM_2011
(0028,140C)	SQ	BlendingLUT2Sequence	1	DICOM_2011
(0028,140D)	CS	BlendingLUT2TransferFunction	1	DICOM_2011
(0028,140E)	CS	DataPathID	1	DICOM_2011
(0028,140F)	CS	RGBLUTTransferFunction	1	DICOM_2011
(0028,1410)	CS	AlphaLUTTransferFunction	1	DICOM_2011
(0028,2000)	OB	ICCProfile	1	DICOM_2011
(0028,2110)	CS	LossyImageCompression	1	DICOM_2011
(0028,2112)	DS	LossyImageCompressionRatio	1-n	DICOM_2011
(0028,2114)	CS	LossyImageCompressionMethod	1-n	DICOM_2011
(0028,3000)	SQ	ModalityLUTSequence	1	DICOM_2011
(0028,3002)	xs	LUTDescriptor	3	DICOM_2011
(0028,3003)	LO	LUTExplanation	1	DICOM_2011
(0028,3004)	LO	ModalityLUTType	1	DICOM_2011
(0028,3006)	lt	LUTData	1-n	DICOM_2011
(0028,3010)	SQ	VOILUTSequence	1	DICOM_2011
(0028,3110)	SQ	SoftcopyVOILUTSequence	1	DICOM_2011
(0028,6010)	US	RepresentativeFrameNumber	1	DICOM_2011
(0028,6020)	US	FrameNumbersOfInterest	1-n	DICOM_2011
(0028,6022)	LO	FrameOfInterestDescription	1-n	DICOM_2011
(0028,6023)	CS	FrameOfInterestType	1-n	DICOM_2011
(0028,6040)	US	RWavePointer	1-n	DICOM_2011
(0028,6100)	SQ	MaskSubtractionSequence	1	DICOM_2011
(0028,6101)	CS	MaskOperation	1	DICOM_2011
(0028,6102)	US	ApplicableFrameRange	2-2n	DICOM_2011
(0028,6110)	US	MaskFrameNumbers	1-n	DICOM_2011
(0028,6112)	US	ContrastFrameAveraging	1	DICOM_2011
(0028,6114)	FL	MaskSubPixelShift	2	DICOM_2011
(0028,6120)	SS	TIDOffset	1	DICOM_2011
(0028,6190)	ST	MaskOperationExplanation	1	DICOM_2011
(0028,7FE0)	UT	PixelDataProviderURL	1	DICOM_2011
(0028,9001)	UL	DataPointRows	1	DICOM_2011
(0028,9002)	UL	DataPointColumns	1	DICOM_2011
(0028,9003)	CS	SignalDomainColumns	1	DICOM_2011
(0028,9108)	CS	DataRepresentation	1	DICOM_2011
(0028,9110)	SQ	PixelMeasuresSequence	1	DICOM_2011
(0028,9132)	SQ	FrameVOILUTSequence	1	DICOM_2011
(0028,9145)	SQ	PixelValueTransformationSequence	1	DICOM_2011
(0028,9235)	CS	SignalDomainRows	1	DICOM_2011
(0028,9411)	FL	DisplayFilterPercentage	1	DICOM_2011
(0028,9415)	SQ	FramePixelShiftSequence	1	DICOM_2011
(0028,9416)	US	SubtractionItemID	1	DICOM_2011
(0028,9422)	SQ	PixelIntensityRelationshipLUTSequence	1	DICOM_2011
(0028,9443)	SQ	FramePixelDataPropertiesSequence	1	DICOM_2011
(0028,9444)	CS	GeometricalProperties	1	DICOM_2011
(0028,9445)	FL	GeometricMaximumDistortion	1	DICOM_2011
(0028,9446)	CS	ImageProcessingApplied	1-n	DICOM_2011
(0028,9454)	CS	MaskSelectionMode	1	DICOM_2011
(0028,9474)	CS	LUTFunction	1	DICOM_2011
(0028,9478)	FL	MaskVisibilityPercentage	1	DICOM_2011
(0028,9501)	SQ	PixelShiftSequence	1	DICOM_2011
(0028,9502)	SQ	RegionPixelShiftSequence	1	DICOM_2011
(0028,9503)	SS	VerticesOfTheRegion	2-2n	DICOM_2011
(0028,9505)	SQ	MultiFramePresentationSequence	1	DICOM_2011
(0028,9506)	US	PixelShiftFrameRange	2-2n	DICOM_2011
(0028,9507)	US	LUTFrameRange	2-2n	DICOM_2011
(0028,9520)	DS	ImageToEquipmentMappingMatrix	16	DICOM_2011
(0028,9537)	CS	EquipmentCoordinateSystemIdentification	1	DICOM_2011
(0032,1031)	SQ	RequestingPhysicianIdentificationSequence	1	DICOM_2011
(0032,1032)	PN	RequestingPhysician	1	DICOM_2011
(0032,1033)	LO	RequestingService	1	DICOM_2011
(0032,1034)	SQ	RequestingServiceCodeSequence	1	DICOM_2011
(0032,1060)	LO	RequestedProcedureDescription	1	DICOM_2011
(0032,1064)	SQ	RequestedProcedureCodeSequence	1	DICOM_2011
(0032,1070)	LO	RequestedContrastAgent	1	DICOM_2011
(0038,0004)	SQ	ReferencedPatientAliasSequence	1	DICOM_2011
(0038,0008)	CS	VisitStatusID	1	DICOM_2011
(0038,0010)	LO	AdmissionID	1	DICOM_2011
(0038,0014)	SQ	IssuerOfAdmissionIDSequence	1	DICOM_2011
(0038,0016)	LO	RouteOfAdmissions	1	DICOM_2011
(0038,0020)	DA	AdmittingDate	1	DICOM_2011
(0038,0021)	TM	AdmittingTime	1	DICOM_2011
(0038,0050)	LO	SpecialNeeds	1	DICOM_2011
(0038,0060)	LO	ServiceEpisodeID	1	DICOM_2011
(0038,0062)	LO	ServiceEpisodeDescription	1	DICOM_2011
(0038,0064)	SQ	IssuerOfServiceEpisodeIDSequence	1	DICOM_2011
(0038,0100)	SQ	PertinentDocumentsSequence	1	DICOM_2011
(0038,0300)	LO	CurrentPatientLocation	1	DICOM_2011
(0038,0400)	LO	PatientInstitutionResidence	1	DICOM_2011
(0038,0500)	LO	PatientState	1	DICOM_2011
(0038,0502)	SQ	PatientClinicalTrialParticipationSequence	1	DICOM_2011
(0038,4000)	LT	VisitComments	1	DICOM_2011
(003A,0004)	CS	WaveformOriginality	1	DICOM_2011
(003A,0005)	US	NumberOfWaveformChannels	1	DICOM_2011
(003A,0010)	UL	NumberOfWaveformSamples	1	DICOM_2011
(003A,001A)	DS	SamplingFrequency	1	DICOM_2011
(003A,0020)	SH	MultiplexGroupLabel	1	DICOM_2011
(003A,0200)	SQ	ChannelDefinitionSequence	1	DICOM_2011
(003A,0202)	IS	WaveformChannelNumber	1	DICOM_2011
(003A,0203)	SH	ChannelLabel	1	DICOM_2011
(003A,0205)	CS	ChannelStatus	1-n	DICOM_2011
(003A,0208)	SQ	ChannelSourceSequence	1	DICOM_2011
(003A,0209)	SQ	ChannelSourceModifiersSequence	1	DICOM_2011
(003A,020A)	SQ	SourceWaveformSequence	1	DICOM_2011
(003A,020C)	LO	ChannelDerivationDescription	1	DICOM_2011
(003A,0210)	DS	ChannelSensitivity	1	DICOM_2011
(003A,0211)	SQ	ChannelSensitivityUnitsSequence	1	DICOM_2011
(003A,0212)	DS	ChannelSensitivityCorrectionFactor	1	DICOM_2011
(003A,0213)	DS	ChannelBaseline	1	DICOM_2011
(003A,0214)	DS	ChannelTimeSkew	1	DICOM_2011
(003A,0215)	DS	ChannelSampleSkew	1	DICOM_2011
(003A,0218)	DS	ChannelOffset	1	DICOM_2011
(003A,021A)	US	WaveformBitsStored	1	DICOM_2011
(003A,0220)	DS	FilterLowFrequency	1	DICOM_2011
(003A,0221)	DS	FilterHighFrequency	1	DICOM_2011
(003A,0222)	DS	NotchFilterFrequency	1	DICOM_2011
(003A,0223)	DS	NotchFilterBandwidth	1	DICOM_2011
(003A,0230)	FL	WaveformDataDisplayScale	1	DICOM_2011
(003A,0231)	US	WaveformDisplayBackgroundCIELabValue	3	DICOM_2011
(003A,0240)	SQ	WaveformPresentationGroupSequence	1	DICOM_2011
(003A,0241)	US	PresentationGroupNumber	1	DICOM_2011
(003A,0242)	SQ	ChannelDisplaySequence	1	DICOM_2011
(003A,0244)	US	ChannelRecommendedDisplayCIELabValue	3	DICOM_2011
(003A,0245)	FL	ChannelPosition	1	DICOM_2011
(003A,0246)	CS	DisplayShadingFlag	1	DICOM_2011
(003A,0247)	FL	FractionalChannelDisplayScale	1	DICOM_2011
(003A,0248)	FL	AbsoluteChannelDisplayScale	1	DICOM_2011
(003A,0300)	SQ	MultiplexedAudioChannelsDescriptionCodeSequence	1	DICOM_2011
(003A,0301)	IS	ChannelIdentificationCode	1	DICOM_2011
(003A,0302)	CS	ChannelMode	1	DICOM_2011
(0040,0001)	AE	ScheduledStationAETitle	1-n	DICOM_2011
(0040,0002)	DA	ScheduledProcedureStepStartDate	1	DICOM_2011
(0040,0003)	TM	ScheduledProcedureStepStartTime	1	DICOM_2011
(0040,0004)	DA	ScheduledProcedureStepEndDate	1	DICOM_2011
(0040,0005)	TM	ScheduledProcedureStepEndTime	1	DICOM_2011
(0040,0006)	PN	ScheduledPerformingPhysicianName	1	DICOM_2011
(0040,0007)	LO	ScheduledProcedureStepDescription	1	DICOM_2011
(0040,0008)	SQ	ScheduledProtocolCodeSequence	1	DICOM_2011
(0040,0009)	SH	ScheduledProcedureStepID	1	DICOM_2011
(0040,000A)	SQ	StageCodeSequence	1	DICOM_2011
(0040,000B)	SQ	ScheduledPerformingPhysicianIdentificationSequence	1	DICOM_2011
(0040,0010)	SH	ScheduledStationName	1-n	DICOM_2011
(0040,0011)	SH	ScheduledProcedureStepLocation	1	DICOM_2011
(0040,0012)	LO	PreMedication	1	DICOM_2011
(0040,0020)	CS	ScheduledProcedureStepStatus	1	DICOM_2011
(0040,0026)	SQ	OrderPlacerIdentifierSequence	1	DICOM_2011
(0040,0027)	SQ	OrderFillerIdentifierSequence	1	DICOM_2011
(0040,0031)	UT	LocalNamespaceEntityID	1	DICOM_2011
(0040,0032)	UT	UniversalEntityID	1	DICOM_2011
(0040,0033)	CS	UniversalEntityIDType	1	DICOM_2011
(0040,0035)	CS	IdentifierTypeCode	1	DICOM_2011
(0040,0036)	SQ	AssigningFacilitySequence	1	DICOM_2011
(0040,0039)	SQ	AssigningJurisdictionCodeSequence	1	DICOM_2011
(0040,003A)	SQ	AssigningAgencyOrDepartmentCodeSequence	1	DICOM_2011
(0040,0100)	SQ	ScheduledProcedureStepSequence	1	DICOM_2011
(0040,0220)	SQ	ReferencedNonImageCompositeSOPInstanceSequence	1	DICOM_2011
(0040,0241)	AE	PerformedStationAETitle	1	DICOM_2011
(0040,0242)	SH	PerformedStationName	1	DICOM_2011
(0040,0243)	SH	PerformedLocation	1	DICOM_2011
(0040,0244)	DA	PerformedProcedureStepStartDate	1	DICOM_2011
(0040,0245)	TM	PerformedProcedureStepStartTime	1	DICOM_2011
(0040,0250)	DA	PerformedProcedureStepEndDate	1	DICOM_2011
(0040,0251)	TM	PerformedProcedureStepEndTime	1	DICOM_2011
(0040,0252)	CS	PerformedProcedureStepStatus	1	DICOM_2011
(0040,0253)	SH	PerformedProcedureStepID	1	DICOM_2011
(0040,0254)	LO	PerformedProcedureStepDescription	1	DICOM_2011
(0040,0255)	LO	PerformedProcedureTypeDescription	1	DICOM_2011
(0040,0260)	SQ	PerformedProtocolCodeSequence	1	DICOM_2011
(0040,0261)	CS	PerformedProtocolType	1	DICOM_2011
(0040,0270)	SQ	ScheduledStepAttributesSequence	1	DICOM_2011
(0040,0275)	SQ	RequestAttributesSequence	1	DICOM_2011
(0040,0280)	ST	CommentsOnThePerformedProcedureStep	1	DICOM_2011
(0040,0281)	SQ	PerformedProcedureStepDiscontinuationReasonCodeSequence	1	DICOM_2011
(0040,0293)	SQ	QuantitySequence	1	DICOM_2011
(0040,0294)	DS	Quantity	1	DICOM_2011
(0040,0295)	SQ	MeasuringUnitsSequence	1	DICOM_2011
(0040,0296)	SQ	BillingItemSequence	1	DICOM_2011
(0040,0300)	US	TotalTimeOfFluoroscopy	1	DICOM_2011
(0040,0301)	US	TotalNumberOfExposures	1	DICOM_2011
(0040,0302)	US	EntranceDose	1	DICOM_2011
(0040,0303)	US	ExposedArea	1-2	DICOM_2011
(0040,0306)	DS	DistanceSourceToEntrance	1	DICOM_2011
(0040,030E)	SQ	ExposureDoseSequence	1	DICOM_2011
(0040,0310)	ST	CommentsOnRadiationDose	1	DICOM_2011
(0040,0312)	DS	XRayOutput	1	DICOM_2011
(0040,0314)	DS	HalfValueLayer	1	DICOM_2011
(0040,0316)	DS	OrganDose	1	DICOM_2011
(0040,0318)	CS	OrganExposed	1	DICOM_2011
(0040,0320)	SQ	BillingProcedureStepSequence	1	DICOM_2011
(0040,0321)	SQ	FilmConsumptionSequence	1	DICOM_2011
(0040,0324)	SQ	BillingSuppliesAndDevicesSequence	1	DICOM_2011
(0040,0340)	SQ	PerformedSeriesSequence	1	DICOM_2011
(0040,0400)	LT	CommentsOnTheScheduledProcedureStep	1	DICOM_2011
(0040,0440)	SQ	ProtocolContextSequence	1	DICOM_2011
(0040,0441)	SQ	ContentItemModifierSequence	1	DICOM_2011
(0040,0500)	SQ	ScheduledSpecimenSequence	1	DICOM_2011
(0040,0512)	LO	ContainerIdentifier	1	DICOM_2011
(0040,0513)	SQ	IssuerOfTheContainerIdentifierSequence	1	DICOM_2011
(0040,0515)	SQ	AlternateContainerIdentifierSequence	1	DICOM_2011
(0040,0518)	SQ	ContainerTypeCodeSequence	1	DICOM_2011
(0040,051A)	LO	ContainerDescription	1	DICOM_2011
(0040,0520)	SQ	ContainerComponentSequence	1	DICOM_2011
(0040,0551)	LO	SpecimenIdentifier	1	DICOM_2011
(0040,0554)	UI	SpecimenUID	1	DICOM_2011
(0040,0555)	SQ	AcquisitionContextSequence	1	DICOM_2011
(0040,0556)	ST	AcquisitionContextDescription	1	DICOM_2011
(0040,0560)	SQ	SpecimenDescriptionSequence	1	DICOM_2011
(0040,0562)	SQ	IssuerOfTheSpecimenIdentifierSequence	1	DICOM_2011
(0040,059A)	SQ	SpecimenTypeCodeSequence	1	DICOM_2011
(0040,0600)	LO	SpecimenShortDescription	1	DICOM_2011
(0040,0602)	UT	SpecimenDetailedDescription	1	DICOM_2011
(0040,0610)	SQ	SpecimenPreparationSequence	1	DICOM_2011
(0040,0612)	SQ	SpecimenPreparationStepContentItemSequence	1	DICOM_2011
(0040,0620)	SQ	SpecimenLocalizationContentItemSequence	1	DICOM_2011
(0040,071A)	SQ	ImageCenterPointCoordinatesSequence	1	DICOM_2011
(0040,072A)	DS	XOffsetInSlideCoordinateSystem	1	DICOM_2011
(0040,073A)	DS	YOffsetInSlideCoordinateSystem	1	DICOM_2011
(0040,074A)	DS	ZOffsetInSlideCoordinateSystem	1	DICOM_2011
(0040,08EA)	SQ	MeasurementUnitsCodeSequence	1	DICOM_2011
(0040,1001)	SH	RequestedProcedureID	1	DICOM_2011
(0040,1002)	LO	ReasonForTheRequestedProcedure	1	DICOM_2011
(0040,1003)	SH	RequestedProcedurePriority	1	DICOM_2011
(0040,1004)	LO	PatientTransportArrangements	1	DICOM_2011
(0040,1005)	LO	RequestedProcedureLocation	1	DICOM_2011
(0040,1008)	LO	ConfidentialityCode	1	DICOM_2011
(0040,1009)	SH	ReportingPriority	1	DICOM_2011
(0040,100A)	SQ	ReasonForRequestedProcedureCodeSequence	1	DICOM_2011
(0040,1010)	PN	NamesOfIntendedRecipientsOfResults	1-n	DICOM_2011
(0040,1011)	SQ	IntendedRecipientsOfResultsIdentificationSequence	1	DICOM_2011
(0040,1012)	SQ	ReasonForPerformedProcedureCodeSequence	1	DICOM_2011
(0040,1101)	SQ	PersonIdentificationCodeSequence	1	DICOM_2011
(0040,1102)	ST	PersonAddress	1	DICOM_2011
(0040,1103)	LO	PersonTelephoneNumbers	1-n	DICOM_2011
(0040,1400)	LT	RequestedProcedureComments	1	DICOM_2011
(0040,2004)	DA	IssueDateOfImagingServiceRequest	1	DICOM_2011
(0040,2005)	TM	IssueTimeOfImagingServiceRequest	1	DICOM_2011
(0040,2008)	PN	OrderEnteredBy	1	DICOM_2011
(0040,2009)	SH	OrderEntererLocation	1	DICOM_2011
(0040,2010)	SH	OrderCallbackPhoneNumber	1	DICOM_2011
(0040,2016)	LO	PlacerOrderNumberImagingServiceRequest	1	DICOM_2011
(0040,2017)	LO	FillerOrderNumberImagingServiceRequest	1	DICOM_2011
(0040,2400)	LT	ImagingServiceRequestComments	1	DICOM_2011
(0040,3001)	LO	ConfidentialityConstraintOnPatientDataDescription	1	DICOM_2011
(0040,4001)	CS	GeneralPurposeScheduledProcedureStepStatus	1	DICOM_2011
(0040,4002)	CS	GeneralPurposePerformedProcedureStepStatus	1	DICOM_2011
(0040,4003)	CS	GeneralPurposeScheduledProcedureStepPriority	1	DICOM_2011
(0040,4004)	SQ	ScheduledProcessingApplicationsCodeSequence	1	DICOM_2011
(0040,4005)	DT	ScheduledProcedureStepStartDateTime	1	DICOM_2011
(0040,4006)	CS	MultipleCopiesFlag	1	DICOM_2011
(0040,4007)	SQ	PerformedProcessingApplicationsCodeSequence	1	DICOM_2011
(0040,4009)	SQ	HumanPerformerCodeSequence	1	DICOM_2011
(0040,4010)	DT	ScheduledProcedureStepModificationDateTime	1	DICOM_2011
(0040,4011)	DT	ExpectedCompletionDateTime	1	DICOM_2011
(0040,4015)	SQ	ResultingGeneralPurposePerformedProcedureStepsSequence	1	DICOM_2011
(0040,4016)	SQ	ReferencedGeneralPurposeScheduledProcedureStepSequence	1	DICOM_2011
(0040,4018)	SQ	ScheduledWorkitemCodeSequence	1	DICOM_2011
(0040,4019)	SQ	PerformedWorkitemCodeSequence	1	DICOM_2011
(0040,4020)	CS	InputAvailabilityFlag	1	DICOM_2011
(0040,4021)	SQ	InputInformationSequence	1	DICOM_2011
(0040,4022)	SQ	RelevantInformationSequence	1	DICOM_2011
(0040,4023)	UI	ReferencedGeneralPurposeScheduledProcedureStepTransactionUID	1	DICOM_2011
(0040,4025)	SQ	ScheduledStationNameCodeSequence	1	DICOM_2011
(0040,4026)	SQ	ScheduledStationClassCodeSequence	1	DICOM_2011
(0040,4027)	SQ	ScheduledStationGeographicLocationCodeSequence	1	DICOM_2011
(0040,4028)	SQ	PerformedStationNameCodeSequence	1	DICOM_2011
(0040,4029)	SQ	PerformedStationClassCodeSequence	1	DICOM_2011
(0040,4030)	SQ	PerformedStationGeographicLocationCodeSequence	1	DICOM_2011
(0040,4031)	SQ	RequestedSubsequentWorkitemCodeSequence	1	DICOM_2011
(0040,4032)	SQ	NonDICOMOutputCodeSequence	1	DICOM_2011
(0040,4033)	SQ	OutputInformationSequence	1	DICOM_2011
(0040,4034)	SQ	ScheduledHumanPerformersSequence	1	DICOM_2011
(0040,4035)	SQ	ActualHumanPerformersSequence	1	DICOM_2011
(0040,4036)	LO	HumanPerformerOrganization	1	DICOM_2011
(0040,4037)	PN	HumanPerformerName	1	DICOM_2011
(0040,4040)	CS	RawDataHandling	1	DICOM_2011
(0040,4041)	CS	InputReadinessState	1	DICOM_2011
(0040,4050)	DT	PerformedProcedureStepStartDateTime	1	DICOM_2011
(0040,4051)	DT	PerformedProcedureStepEndDateTime	1	DICOM_2011
(0040,4052)	DT	ProcedureStepCancellationDateTime	1	DICOM_2011
(0040,8302)	DS	EntranceDoseInmGy	1	DICOM_2011
(0040,9094)	SQ	ReferencedImageRealWorldValueMappingSequence	1	DICOM_2011
(0040,9096)	SQ	RealWorldValueMappingSequence	1	DICOM_2011
(0040,9098)	SQ	PixelValueMappingCodeSequence	1	DICOM_2011
(0040,9210)	SH	LUTLabel	1	DICOM_2011
(0040,9211)	xs	RealWorldValueLastValueMapped	1	DICOM_2011
(0040,9212)	FD	RealWorldValueLUTData	1-n	DICOM_2011
(0040,9216)	xs	RealWorldValueFirstValueMapped	1	DICOM_2011
(0040,9224)	FD	RealWorldValueIntercept	1	DICOM_2011
(0040,9225)	FD	RealWorldValueSlope	1	DICOM_2011
(0040,A010)	CS	RelationshipType	1	DICOM_2011
(0040,A027)	LO	VerifyingOrganization	1	DICOM_2011
(0040,A030)	DT	VerificationDateTime	1	DICOM_2011
(0040,A032)	DT	ObservationDateTime	1	DICOM_2011
(0040,A040)	CS	ValueType	1	DICOM_2011
(0040,A043)	SQ	ConceptNameCodeSequence	1	DICOM_2011
(0040,A050)	CS	ContinuityOfContent	1	DICOM_2011
(0040,A073)	SQ	VerifyingObserverSequence	1	DICOM_2011
(0040,A075)	PN	VerifyingObserverName	1	DICOM_2011
(0040,A078)	SQ	AuthorObserverSequence	1	DICOM_2011
(0040,A07A)	SQ	ParticipantSequence	1	DICOM_2011
(0040,A07C)	SQ	CustodialOrganizationSequence	1	DICOM_2011
(0040,A080)	CS	ParticipationType	1	DICOM_2011
(0040,A082)	DT	ParticipationDateTime	1	DICOM_2011
(0040,A084)	CS	ObserverType	1	DICOM_2011
(0040,A088)	SQ	VerifyingObserverIdentificationCodeSequence	1	DICOM_2011
(0040,A0B0)	US	ReferencedWaveformChannels	2-2n	DICOM_2011
(0040,A120)	DT	DateTime	1	DICOM_2011
(0040,A121)	DA	Date	1	DICOM_2011
(0040,A122)	TM	Time	1	DICOM_2011
(0040,A123)	PN	PersonName	1	DICOM_2011
(0040,A124)	UI	UID	1	DICOM_2011
(0040,A130)	CS	TemporalRangeType	1	DICOM_2011
(0040,A132)	UL	ReferencedSamplePositions	1-n	DICOM_2011
(0040,A136)	US	ReferencedFrameNumbers	1-n	DICOM_2011
(0040,A138)	DS	ReferencedTimeOffsets	1-n	DICOM_2011
(0040,A13A)	DT	ReferencedDateTime	1-n	DICOM_2011
(0040,A160)	UT	TextValue	1	DICOM_2011
(0040,A168)	SQ	ConceptCodeSequence	1	DICOM_2011
(0040,A170)	SQ	PurposeOfReferenceCodeSequence	1	DICOM_2011
(0040,A180)	US	AnnotationGroupNumber	1	DICOM_2011
(0040,A195)	SQ	ModifierCodeSequence	1	DICOM_2011
(0040,A300)	SQ	MeasuredValueSequence	1	DICOM_2011
(0040,A301)	SQ	NumericValueQualifierCodeSequence	1	DICOM_2011
(0040,A30A)	DS	NumericValue	1-n	DICOM_2011
(0040,A360)	SQ	PredecessorDocumentsSequence	1	DICOM_2011
(0040,A370)	SQ	ReferencedRequestSequence	1	DICOM_2011
(0040,A372)	SQ	PerformedProcedureCodeSequence	1	DICOM_2011
(0040,A375)	SQ	CurrentRequestedProcedureEvidenceSequence	1	DICOM_2011
(0040,A385)	SQ	PertinentOtherEvidenceSequence	1	DICOM_2011
(0040,A390)	SQ	HL7StructuredDocumentReferenceSequence	1	DICOM_2011
(0040,A491)	CS	CompletionFlag	1	DICOM_2011
(0040,A492)	LO	CompletionFlagDescription	1	DICOM_2011
(0040,A493)	CS	VerificationFlag	1	DICOM_2011
(0040,A494)	CS	ArchiveRequested	1	DICOM_2011
(0040,A496)	CS	PreliminaryFlag	1	DICOM_2011
(0040,A504)	SQ	ContentTemplateSequence	1	DICOM_2011
(0040,A525)	SQ	IdenticalDocumentsSequence	1	DICOM_2011
(0040,A730)	SQ	ContentSequence	1	DICOM_2011
(0040,B020)	SQ	WaveformAnnotationSequence	1	DICOM_2011
(0040,DB00)	CS	TemplateIdentifier	1	DICOM_2011
(0040,DB73)	UL	ReferencedContentItemIdentifier	1-n	DICOM_2011
(0040,E001)	ST	HL7InstanceIdentifier	1	DICOM_2011
(0040,E004)	DT	HL7DocumentEffectiveTime	1	DICOM_2011
(0040,E006)	SQ	HL7DocumentTypeCodeSequence	1	DICOM_2011
(0040,E008)	SQ	DocumentClassCodeSequence	1	DICOM_2011
(0040,E010)	UT	RetrieveURI	1	DICOM_2011
(0040,E011)	UI	RetrieveLocationUID	1	DICOM_2011
(0040,E020)	CS	TypeOfInstances	1	DICOM_2011
(0040,E021)	SQ	DICOMRetrievalSequence	1	DICOM_2011
(0040,E022)	SQ	DICOMMediaRetrievalSequence	1	DICOM_2011
(0040,E023)	SQ	WADORetrievalSequence	1	DICOM_2011
(0040,E024)	SQ	XDSRetrievalSequence	1	DICOM_2011
(0040,E030)	UI	RepositoryUniqueID	1	DICOM_2011
(0040,E031)	UI	HomeCommunityID	1	DICOM_2011
(0042,0010)	ST	DocumentTitle	1	DICOM_2011
(0042,0011)	OB	EncapsulatedDocument	1	DICOM_2011
(0042,0012)	LO	MIMETypeOfEncapsulatedDocument	1	DICOM_2011
(0042,0013)	SQ	SourceInstanceSequence	1	DICOM_2011
(0042,0014)	LO	ListOfMIMETypes	1-n	DICOM_2011
(0044,0001)	ST	ProductPackageIdentifier	1	DICOM_2011
(0044,0002)	CS	SubstanceAdministrationApproval	1	DICOM_2011
(0044,0003)	LT	ApprovalStatusFurtherDescription	1	DICOM_2011
(0044,0004)	DT	ApprovalStatusDateTime	1	DICOM_2011
(0044,0007)	SQ	ProductTypeCodeSequence	1	DICOM_2011
(0044,0008)	LO	ProductName	1-n	DICOM_2011
(0044,0009)	LT	ProductDescription	1	DICOM_2011
(0044,000A)	LO	ProductLotIdentifier	1	DICOM_2011
(0044,000B)	DT	ProductExpirationDateTime	1	DICOM_2011
(0044,0010)	DT	SubstanceAdministrationDateTime	1	DICOM_2011
(0044,0011)	LO	SubstanceAdministrationNotes	1	DICOM_2011
(0044,0012)	LO	SubstanceAdministrationDeviceID	1	DICOM_2011
(0044,0013)	SQ	ProductParameterSequence	1	DICOM_2011
(0044,0019)	SQ	SubstanceAdministrationParameterSequence	1	DICOM_2011
(0046,0012)	LO	LensDescription	1	DICOM_2011
(0046,0014)	SQ	RightLensSequence	1	DICOM_2011
(0046,0015)	SQ	LeftLensSequence	1	DICOM_2011
(0046,0016)	SQ	UnspecifiedLateralityLensSequence	1	DICOM_2011
(0046,0018)	SQ	CylinderSequence	1	DICOM_2011
(0046,0028)	SQ	PrismSequence	1	DICOM_2011
(0046,0030)	FD	HorizontalPrismPower	1	DICOM_2011
(0046,0032)	CS	HorizontalPrismBase	1	DICOM_2011
(0046,0034)	FD	VerticalPrismPower	1	DICOM_2011
(0046,0036)	CS	VerticalPrismBase	1	DICOM_2011
(0046,0038)	CS	LensSegmentType	1	DICOM_2011
(0046,0040)	FD	OpticalTransmittance	1	DICOM_2011
(0046,0042)	FD	ChannelWidth	1	DICOM_2011
(0046,0044)	FD	PupilSize	1	DICOM_2011
(0046,0046)	FD	CornealSize	1	DICOM_2011
(0046,0050)	SQ	AutorefractionRightEyeSequence	1	DICOM_2011
(0046,0052)	SQ	AutorefractionLeftEyeSequence	1	DICOM_2011
(0046,0060)	FD	DistancePupillaryDistance	1	DICOM_2011
(0046,0062)	FD	NearPupillaryDistance	1	DICOM_2011
(0046,0063)	FD	IntermediatePupillaryDistance	1	DICOM_2011
(0046,0064)	FD	OtherPupillaryDistance	1	DICOM_2011
(0046,0070)	SQ	KeratometryRightEyeSequence	1	DICOM_2011
(0046,0071)	SQ	KeratometryLeftEyeSequence	1	DICOM_2011
(0046,0074)	SQ	SteepKeratometricAxisSequence	1	DICOM_2011
(0046,0075)	FD	RadiusOfCurvature	1	DICOM_2011
(0046,0076)	FD	KeratometricPower	1	DICOM_2011
(0046,0077)	FD	KeratometricAxis	1	DICOM_2011
(0046,0080)	SQ	FlatKeratometricAxisSequence	1	DICOM_2011
(0046,0092)	CS	BackgroundColor	1	DICOM_2011
(0046,0094)	CS	Optotype	1	DICOM_2011
(0046,0095)	CS	OptotypePresentation	1	DICOM_2011
(0046,0097)	SQ	SubjectiveRefractionRightEyeSequence	1	DICOM_2011
(0046,0098)	SQ	SubjectiveRefractionLeftEyeSequence	1	DICOM_2011
(0046,0100)	SQ	AddNearSequence	1	DICOM_2011
(0046,0101)	SQ	AddIntermediateSequence	1	DICOM_2011
(0046,0102)	SQ	AddOtherSequence	1	DICOM_2011
(0046,0104)	FD	AddPower	1	DICOM_2011
(0046,0106)	FD	ViewingDistance	1	DICOM_2011
(0046,0121)	SQ	VisualAcuityTypeCodeSequence	1	DICOM_2011
(0046,0122)	SQ	VisualAcuityRightEyeSequence	1	DICOM_2011
(0046,0123)	SQ	VisualAcuityLeftEyeSequence	1	DICOM_2011
(0046,0124)	SQ	VisualAcuityBothEyesOpenSequence	1	DICOM_2011
(0046,0125)	CS	ViewingDistanceType	1	DICOM_2011
(0046,0135)	SS	VisualAcuityModifiers	2	DICOM_2011
(0046,0137)	FD	DecimalVisualAcuity	1	DICOM_2011
(0046,0139)	LO	OptotypeDetailedDefinition	1	DICOM_2011
(0046,0145)	SQ	ReferencedRefractiveMeasurementsSequence	1	DICOM_2011
(0046,0146)	FD	SpherePower	1	DICOM_2011
(0046,0147)	FD	CylinderPower	1	DICOM_2011
(0048,0001)	FL	ImagedVolumeWidth	1	DICOM_2011
(0048,0002)	FL	ImagedVolumeHeight	1	DICOM_2011
(0048,0003)	FL	ImagedVolumeDepth	1	DICOM_2011
(0048,0006)	UL	TotalPixelMatrixColumns	1	DICOM_2011
(0048,0007)	UL	TotalPixelMatrixRows	1	DICOM_2011
(0048,0008)	SQ	TotalPixelMatrixOriginSequence	1	DICOM_2011
(0048,0010)	CS	SpecimenLabelInImage	1	DICOM_2011
(0048,0011)	CS	FocusMethod	1	DICOM_2011
(0048,0012)	CS	ExtendedDepthOfField	1	DICOM_2011
(0048,0013)	US	NumberOfFocalPlanes	1	DICOM_2011
(0048,0014)	FL	DistanceBetweenFocalPlanes	1	DICOM_2011
(0048,0015)	US	RecommendedAbsentPixelCIELabValue	3	DICOM_2011
(0048,0100)	SQ	IlluminatorTypeCodeSequence	1	DICOM_2011
(0048,0102)	DS	ImageOrientationSlide	6	DICOM_2011
(0048,0105)	SQ	OpticalPathSequence	1	DICOM_2011
(0048,0106)	SH	OpticalPathIdentifier	1	DICOM_2011
(0048,0107)	ST	OpticalPathDescription	1	DICOM_2011
(0048,0108)	SQ	IlluminationColorCodeSequence	1	DICOM_2011
(0048,0110)	SQ	SpecimenReferenceSequence	1	DICOM_2011
(0048,0111)	DS	CondenserLensPower	1	DICOM_2011
(0048,0112)	DS	ObjectiveLensPower	1	DICOM_2011
(0048,0113)	DS	ObjectiveLensNumericalAperture	1	DICOM_2011
(0048,0120)	SQ	PaletteColorLookupTableSequence	1	DICOM_2011
(0048,0200)	SQ	ReferencedImageNavigationSequence	1	DICOM_2011
(0048,0201)	US	TopLeftHandCornerOfLocalizerArea	2	DICOM_2011
(0048,0202)	US	BottomRightHandCornerOfLocalizerArea	2	DICOM_2011
(0048,0207)	SQ	OpticalPathIdentificationSequence	1	DICOM_2011
(0048,021A)	SQ	PlanePositionSlideSequence	1	DICOM_2011
(0048,021E)	SL	ColumnPositionInTotalImagePixelMatrix	1	DICOM_2011
(0048,021F)	SL	RowPositionInTotalImagePixelMatrix	1	DICOM_2011
(0048,0301)	CS	PixelOriginInterpretation	1	DICOM_2011
(0050,0004)	CS	CalibrationImage	1	DICOM_2011
(0050,0010)	SQ	DeviceSequence	1	DICOM_2011
(0050,0012)	SQ	ContainerComponentTypeCodeSequence	1	DICOM_2011
(0050,0013)	FD	ContainerComponentThickness	1	DICOM_2011
(0050,0014)	DS	DeviceLength	1	DICOM_2011
(0050,0015)	FD	ContainerComponentWidth	1	DICOM_2011
(0050,0016)	DS	DeviceDiameter	1	DICOM_2011
(0050,0017)	CS	DeviceDiameterUnits	1	DICOM_2011
(0050,0018)	DS	DeviceVolume	1	DICOM_2011
(0050,0019)	DS	InterMarkerDistance	1	DICOM_2011
(0050,001A)	CS	ContainerComponentMaterial	1	DICOM_2011
(0050,001B)	LO	ContainerComponentID	1	DICOM_2011
(0050,001C)	FD	ContainerComponentLength	1	DICOM_2011
(0050,001D)	FD	ContainerComponentDiameter	1	DICOM_2011
(0050,001E)	LO	ContainerComponentDescription	1	DICOM_2011
(0050,0020)	LO	DeviceDescription	1	DICOM_2011
(0052,0001)	FL	ContrastBolusIngredientPercentByVolume	1	DICOM_2011
(0052,0002)	FD	OCTFocalDistance	1	DICOM_2011
(0052,0003)	FD	BeamSpotSize	1	DICOM_2011
(0052,0004)	FD	EffectiveRefractiveIndex	1	DICOM_2011
(0052,0006)	CS	OCTAcquisitionDomain	1	DICOM_2011
(0052,0007)	FD	OCTOpticalCenterWavelength	1	DICOM_2011
(0052,0008)	FD	AxialResolution	1	DICOM_2011
(0052,0009)	FD	RangingDepth	1	DICOM_2011
(0052,0011)	FD	ALineRate	1	DICOM_2011
(0052,0012)	US	ALinesPerFrame	1	DICOM_2011
(0052,0013)	FD	CatheterRotationalRate	1	DICOM_2011
(0052,0014)	FD	ALinePixelSpacing	1	DICOM_2011
(0052,0016)	SQ	ModeOfPercutaneousAccessSequence	1	DICOM_2011
(0052,0025)	SQ	IntravascularOCTFrameTypeSequence	1	DICOM_2011
(0052,0026)	CS	OCTZOffsetApplied	1	DICOM_2011
(0052,0027)	SQ	IntravascularFrameContentSequence	1	DICOM_2011
(0052,0028)	FD	IntravascularLongitudinalDistance	1	DICOM_2011
(0052,0029)	SQ	IntravascularOCTFrameContentSequence	1	DICOM_2011
(0052,0030)	SS	OCTZOffsetCorrection	1	DICOM_2011
(0052,0031)	CS	CatheterDirectionOfRotation	1	DICOM_2011
(0052,0033)	FD	SeamLineLocation	1	DICOM_2011
(0052,0034)	FD	FirstALineLocation	1	DICOM_2011
(0052,0036)	US	SeamLineIndex	1	DICOM_2011
(0052,0038)	US	NumberOfPaddedAlines	1	DICOM_2011
(0052,0039)	CS	InterpolationType	1	DICOM_2011
(0052,003A)	CS	RefractiveIndexApplied	1	DICOM_2011
(0054,0010)	US	EnergyWindowVector	1-n	DICOM_2011
(0054,0011)	US	NumberOfEnergyWindows	1	DICOM_2011
(0054,0012)	SQ	EnergyWindowInformationSequence	1	DICOM_2011
(0054,0013)	SQ	EnergyWindowRangeSequence	1	DICOM_2011
(0054,0014)	DS	EnergyWindowLowerLimit	1	DICOM_2011
(0054,0015)	DS	EnergyWindowUpperLimit	1	DICOM_2011
(0054,0016)	SQ	RadiopharmaceuticalInformationSequence	1	DICOM_2011
(0054,0017)	IS	ResidualSyringeCounts	1	DICOM_2011
(0054,0018)	SH	EnergyWindowName	1	DICOM_2011
(0054,0020)	US	DetectorVector	1-n	DICOM_2011
(0054,0021)	US	NumberOfDetectors	1	DICOM_2011
(0054,0022)	SQ	DetectorInformationSequence	1	DICOM_2011
(0054,0030)	US	PhaseVector	1-n	DICOM_2011
(0054,0031)	US	NumberOfPhases	1	DICOM_2011
(0054,0032)	SQ	PhaseInformationSequence	1	DICOM_2011
(0054,0033)	US	NumberOfFramesInPhase	1	DICOM_2011
(0054,0036)	IS	PhaseDelay	1	DICOM_2011
(0054,0038)	IS	PauseBetweenFrames	1	DICOM_2011
(0054,0039)	CS	PhaseDescription	1	DICOM_2011
(0054,0050)	US	RotationVector	1-n	DICOM_2011
(0054,0051)	US	NumberOfRotations	1	DICOM_2011
(0054,0052)	SQ	RotationInformationSequence	1	DICOM_2011
(0054,0053)	US	NumberOfFramesInRotation	1	DICOM_2011
(0054,0060)	US	RRIntervalVector	1-n	DICOM_2011
(0054,0061)	US	NumberOfRRIntervals	1	DICOM_2011
(0054,0062)	SQ	GatedInformationSequence	1	DICOM_2011
(0054,0063)	SQ	DataInformationSequence	1	DICOM_2011
(0054,0070)	US	TimeSlotVector	1-n	DICOM_2011
(0054,0071)	US	NumberOfTimeSlots	1	DICOM_2011
(0054,0072)	SQ	TimeSlotInformationSequence	1	DICOM_2011
(0054,0073)	DS	TimeSlotTime	1	DICOM_2011
(0054,0080)	US	SliceVector	1-n	DICOM_2011
(0054,0081)	US	NumberOfSlices	1	DICOM_2011
(0054,0090)	US	AngularViewVector	1-n	DICOM_2011
(0054,0100)	US	TimeSliceVector	1-n	DICOM_2011
(0054,0101)	US	NumberOfTimeSlices	1	DICOM_2011
(0054,0200)	DS	StartAngle	1	DICOM_2011
(0054,0202)	CS	TypeOfDetectorMotion	1	DICOM_2011
(0054,0210)	IS	TriggerVector	1-n	DICOM_2011
(0054,0211)	US	NumberOfTriggersInPhase	1	DICOM_2011
(0054,0220)	SQ	ViewCodeSequence	1	DICOM_2011
(0054,0222)	SQ	ViewModifierCodeSequence	1	DICOM_2011
(0054,0300)	SQ	RadionuclideCodeSequence	1	DICOM_2011
(0054,0302)	SQ	AdministrationRouteCodeSequence	1	DICOM_2011
(0054,0304)	SQ	RadiopharmaceuticalCodeSequence	1	DICOM_2011
(0054,0306)	SQ	CalibrationDataSequence	1	DICOM_2011
(0054,0308)	US	EnergyWindowNumber	1	DICOM_2011
(0054,0400)	SH	ImageID	1	DICOM_2011
(0054,0410)	SQ	PatientOrientationCodeSequence	1	DICOM_2011
(0054,0412)	SQ	PatientOrientationModifierCodeSequence	1	DICOM_2011
(0054,0414)	SQ	PatientGantryRelationshipCodeSequence	1	DICOM_2011
(0054,0500)	CS	SliceProgressionDirection	1	DICOM_2011
(0054,1000)	CS	SeriesType	2	DICOM_2011
(0054,1001)	CS	Units	1	DICOM_2011
(0054,1002)	CS	CountsSource	1	DICOM_2011
(0054,1004)	CS	ReprojectionMethod	1	DICOM_2011
(0054,1006)	CS	SUVType	1	DICOM_2011
(0054,1100)	CS	RandomsCorrectionMethod	1	DICOM_2011
(0054,1101)	LO	AttenuationCorrectionMethod	1	DICOM_2011
(0054,1102)	CS	DecayCorrection	1	DICOM_2011
(0054,1103)	LO	ReconstructionMethod	1	DICOM_2011
(0054,1104)	LO	DetectorLinesOfResponseUsed	1	DICOM_2011
(0054,1105)	LO	ScatterCorrectionMethod	1	DICOM_2011
(0054,1200)	DS	AxialAcceptance	1	DICOM_2011
(0054,1201)	IS	AxialMash	2	DICOM_2011
(0054,1202)	IS	TransverseMash	1	DICOM_2011
(0054,1203)	DS	DetectorElementSize	2	DICOM_2011
(0054,1210)	DS	CoincidenceWindowWidth	1	DICOM_2011
(0054,1220)	CS	SecondaryCountsType	1-n	DICOM_2011
(0054,1300)	DS	FrameReferenceTime	1	DICOM_2011
(0054,1310)	IS	PrimaryPromptsCountsAccumulated	1	DICOM_2011
(0054,1311)	IS	SecondaryCountsAccumulated	1-n	DICOM_2011
(0054,1320)	DS	SliceSensitivityFactor	1	DICOM_2011
(0054,1321)	DS	DecayFactor	1	DICOM_2011
(0054,1322)	DS	DoseCalibrationFactor	1	DICOM_2011
(0054,1323)	DS	ScatterFractionFactor	1	DICOM_2011
(0054,1324)	DS	DeadTimeFactor	1	DICOM_2011
(0054,1330)	US	ImageIndex	1	DICOM_2011
(0060,3000)	SQ	HistogramSequence	1	DICOM_2011
(0060,3002)	US	HistogramNumberOfBins	1	DICOM_2011
(0060,3004)	xs	HistogramFirstBinValue	1	DICOM_2011
(0060,3006)	xs	HistogramLastBinValue	1	DICOM_2011
(0060,3008)	US	HistogramBinWidth	1	DICOM_2011
(0060,3010)	LO	HistogramExplanation	1	DICOM_2011
(0060,3020)	UL	HistogramData	1-n	DICOM_2011
(0062,0001)	CS	SegmentationType	1	DICOM_2011
(0062,0002)	SQ	SegmentSequence	1	DICOM_2011
(0062,0003)	SQ	SegmentedPropertyCategoryCodeSequence	1	DICOM_2011
(0062,0004)	US	SegmentNumber	1	DICOM_2011
(0062,0005)	LO	SegmentLabel	1	DICOM_2011
(0062,0006)	ST	SegmentDescription	1	DICOM_2011
(0062,0008)	CS	SegmentAlgorithmType	1	DICOM_2011
(0062,0009)	LO	SegmentAlgorithmName	1	DICOM_2011
(0062,000A)	SQ	SegmentIdentificationSequence	1	DICOM_2011
(0062,000B)	US	ReferencedSegmentNumber	1-n	DICOM_2011
(0062,000C)	US	RecommendedDisplayGrayscaleValue	1	DICOM_2011
(0062,000D)	US	RecommendedDisplayCIELabValue	3	DICOM_2011
(0062,000E)	US	MaximumFractionalValue	1	DICOM_2011
(0062,000F)	SQ	SegmentedPropertyTypeCodeSequence	1	DICOM_2011
(0062,0010)	CS	SegmentationFractionalType	1	DICOM_2011
(0064,0002)	SQ	DeformableRegistrationSequence	1	DICOM_2011
(0064,0003)	UI	SourceFrameOfReferenceUID	1	DICOM_2011
(0064,0005)	SQ	DeformableRegistrationGridSequence	1	DICOM_2011
(0064,0007)	UL	GridDimensions	3	DICOM_2011
(0064,0008)	FD	GridResolution	3	DICOM_2011
(0064,0009)	OF	VectorGridData	1	DICOM_2011
(0064,000F)	SQ	PreDeformationMatrixRegistrationSequence	1	DICOM_2011
(0064,0010)	SQ	PostDeformationMatrixRegistrationSequence	1	DICOM_2011
(0066,0001)	UL	NumberOfSurfaces	1	DICOM_2011
(0066,0002)	SQ	SurfaceSequence	1	DICOM_2011
(0066,0003)	UL	SurfaceNumber	1	DICOM_2011
(0066,0004)	LT	SurfaceComments	1	DICOM_2011
(0066,0009)	CS	SurfaceProcessing	1	DICOM_2011
(0066,000A)	FL	SurfaceProcessingRatio	1	DICOM_2011
(0066,000B)	LO	SurfaceProcessingDescription	1	DICOM_2011
(0066,000C)	FL	RecommendedPresentationOpacity	1	DICOM_2011
(0066,000D)	CS	RecommendedPresentationType	1	DICOM_2011
(0066,000E)	CS	FiniteVolume	1	DICOM_2011
(0066,0010)	CS	Manifold	1	DICOM_2011
(0066,0011)	SQ	SurfacePointsSequence	1	DICOM_2011
(0066,0012)	SQ	SurfacePointsNormalsSequence	1	DICOM_2011
(0066,0013)	SQ	SurfaceMeshPrimitivesSequence	1	DICOM_2011
(0066,0015)	UL	NumberOfSurfacePoints	1	DICOM_2011
(0066,0016)	OF	PointCoordinatesData	1	DICOM_2011
(0066,0017)	FL	PointPositionAccuracy	3	DICOM_2011
(0066,0018)	FL	MeanPointDistance	1	DICOM_2011
(0066,0019)	FL	MaximumPointDistance	1	DICOM_2011
(0066,001A)	FL	PointsBoundingBoxCoordinates	6	DICOM_2011
(0066,001B)	FL	AxisOfRotation	3	DICOM_2011
(0066,001C)	FL	CenterOfRotation	3	DICOM_2011
(0066,001E)	UL	NumberOfVectors	1	DICOM_2011
(0066,001F)	US	VectorDimensionality	1	DICOM_2011
(0066,0020)	FL	VectorAccuracy	1-n	DICOM_2011
(0066,0021)	OF	VectorCoordinateData	1	DICOM_2011
(0066,0023)	OW	TrianglePointIndexList	1	DICOM_2011
(0066,0024)	OW	EdgePointIndexList	1	DICOM_2011
(0066,0025)	OW	VertexPointIndexList	1	DICOM_2011
(0066,0026)	SQ	TriangleStripSequence	1	DICOM_2011
(0066,0027)	SQ	TriangleFanSequence	1	DICOM_2011
(0066,0028)	SQ	LineSequence	1	DICOM_2011
(0066,0029)	OW	PrimitivePointIndexList	1	DICOM_2011
(0066,002A)	UL	SurfaceCount	1	DICOM_2011
(0066,002B)	SQ	ReferencedSurfaceSequence	1	DICOM_2011
(0066,002C)	UL	ReferencedSurfaceNumber	1	DICOM_2011
(0066,002D)	SQ	SegmentSurfaceGenerationAlgorithmIdentificationSequence	1	DICOM_2011
(0066,002E)	SQ	SegmentSurfaceSourceInstanceSequence	1	DICOM_2011
(0066,002F)	SQ	AlgorithmFamilyCodeSequence	1	DICOM_2011
(0066,0030)	SQ	AlgorithmNameCodeSequence	1	DICOM_2011
(0066,0031)	LO	AlgorithmVersion	1	DICOM_2011
(0066,0032)	LT	AlgorithmParameters	1	DICOM_2011
(0066,0034)	SQ	FacetSequence	1	DICOM_2011
(0066,0035)	SQ	SurfaceProcessingAlgorithmIdentificationSequence	1	DICOM_2011
(0066,0036)	LO	AlgorithmName	1	DICOM_2011
(0068,6210)	LO	ImplantSize	1	DICOM_2011
(0068,6221)	LO	ImplantTemplateVersion	1	DICOM_2011
(0068,6222)	SQ	ReplacedImplantTemplateSequence	1	DICOM_2011
(0068,6223)	CS	ImplantType	1	DICOM_2011
(0068,6224)	SQ	DerivationImplantTemplateSequence	1	DICOM_2011
(0068,6225)	SQ	OriginalImplantTemplateSequence	1	DICOM_2011
(0068,6226)	DT	EffectiveDateTime	1	DICOM_2011
(0068,6230)	SQ	ImplantTargetAnatomySequence	1	DICOM_2011
(0068,6260)	SQ	InformationFromManufacturerSequence	1	DICOM_2011
(0068,6265)	SQ	NotificationFromManufacturerSequence	1	DICOM_2011
(0068,6270)	DT	InformationIssueDateTime	1	DICOM_2011
(0068,6280)	ST	InformationSummary	1	DICOM_2011
(0068,62A0)	SQ	ImplantRegulatoryDisapprovalCodeSequence	1	DICOM_2011
(0068,62A5)	FD	OverallTemplateSpatialTolerance	1	DICOM_2011
(0068,62C0)	SQ	HPGLDocumentSequence	1	DICOM_2011
(0068,62D0)	US	HPGLDocumentID	1	DICOM_2011
(0068,62D5)	LO	HPGLDocumentLabel	1	DICOM_2011
(0068,62E0)	SQ	ViewOrientationCodeSequence	1	DICOM_2011
(0068,62F0)	FD	ViewOrientationModifier	9	DICOM_2011
(0068,62F2)	FD	HPGLDocumentScaling	1	DICOM_2011
(0068,6300)	OB	HPGLDocument	1	DICOM_2011
(0068,6310)	US	HPGLContourPenNumber	1	DICOM_2011
(0068,6320)	SQ	HPGLPenSequence	1	DICOM_2011
(0068,6330)	US	HPGLPenNumber	1	DICOM_2011
(0068,6340)	LO	HPGLPenLabel	1	DICOM_2011
(0068,6345)	ST	HPGLPenDescription	1	DICOM_2011
(0068,6346)	FD	RecommendedRotationPoint	2	DICOM_2011
(0068,6347)	FD	BoundingRectangle	4	DICOM_2011
(0068,6350)	US	ImplantTemplate3DModelSurfaceNumber	1-n	DICOM_2011
(0068,6360)	SQ	SurfaceModelDescriptionSequence	1	DICOM_2011
(0068,6380)	LO	SurfaceModelLabel	1	DICOM_2011
(0068,6390)	FD	SurfaceModelScalingFactor	1	DICOM_2011
(0068,63A0)	SQ	MaterialsCodeSequence	1	DICOM_2011
(0068,63A4)	SQ	CoatingMaterialsCodeSequence	1	DICOM_2011
(0068,63A8)	SQ	ImplantTypeCodeSequence	1	DICOM_2011
(0068,63AC)	SQ	FixationMethodCodeSequence	1	DICOM_2011
(0068,63B0)	SQ	MatingFeatureSetsSequence	1	DICOM_2011
(0068,63C0)	US	MatingFeatureSetID	1	DICOM_2011
(0068,63D0)	LO	MatingFeatureSetLabel	1	DICOM_2011
(0068,63E0)	SQ	MatingFeatureSequence	1	DICOM_2011
(0068,63F0)	US	MatingFeatureID	1	DICOM_2011
(0068,6400)	SQ	MatingFeatureDegreeOfFreedomSequence	1	DICOM_2011
(0068,6410)	US	DegreeOfFreedomID	1	DICOM_2011
(0068,6420)	CS	DegreeOfFreedomType	1	DICOM_2011
(0068,6430)	SQ	TwoDMatingFeatureCoordinatesSequence	1	DICOM_2011
(0068,6440)	US	ReferencedHPGLDocumentID	1	DICOM_2011
(0068,6450)	FD	TwoDMatingPoint	2	DICOM_2011
(0068,6460)	FD	TwoDMatingAxes	4	DICOM_2011
(0068,6470)	SQ	TwoDDegreeOfFreedomSequence	1	DICOM_2011
(0068,6490)	FD	ThreeDDegreeOfFreedomAxis	3	DICOM_2011
(0068,64A0)	FD	RangeOfFreedom	2	DICOM_2011
(0068,64C0)	FD	ThreeDMatingPoint	3	DICOM_2011
(0068,64D0)	FD	ThreeDMatingAxes	9	DICOM_2011
(0068,64F0)	FD	TwoDDegreeOfFreedomAxis	3	DICOM_2011
(0068,6500)	SQ	PlanningLandmarkPointSequence	1	DICOM_2011
(0068,6510)	SQ	PlanningLandmarkLineSequence	1	DICOM_2011
(0068,6520)	SQ	PlanningLandmarkPlaneSequence	1	DICOM_2011
(0068,6530)	US	PlanningLandmarkID	1	DICOM_2011
(0068,6540)	LO	PlanningLandmarkDescription	1	DICOM_2011
(0068,6545)	SQ	PlanningLandmarkIdentificationCodeSequence	1	DICOM_2011
(0068,6550)	SQ	TwoDPointCoordinatesSequence	1	DICOM_2011
(0068,6560)	FD	TwoDPointCoordinates	2	DICOM_2011
(0068,6590)	FD	ThreeDPointCoordinates	3	DICOM_2011
(0068,65A0)	SQ	TwoDLineCoordinatesSequence	1	DICOM_2011
(0068,65B0)	FD	TwoDLineCoordinates	4	DICOM_2011
(0068,65D0)	FD	ThreeDLineCoordinates	6	DICOM_2011
(0068,65E0)	SQ	TwoDPlaneCoordinatesSequence	1	DICOM_2011
(0068,65F0)	FD	TwoDPlaneIntersection	4	DICOM_2011
(0068,6610)	FD	ThreeDPlaneOrigin	3	DICOM_2011
(0068,6620)	FD	ThreeDPlaneNormal	3	DICOM_2011
(0070,0001)	SQ	GraphicAnnotationSequence	1	DICOM_2011
(0070,0002)	CS	GraphicLayer	1	DICOM_2011
(0070,0003)	CS	BoundingBoxAnnotationUnits	1	DICOM_2011
(0070,0004)	CS	AnchorPointAnnotationUnits	1	DICOM_2011
(0070,0005)	CS	GraphicAnnotationUnits	1	DICOM_2011
(0070,0006)	ST	UnformattedTextValue	1	DICOM_2011
(0070,0008)	SQ	TextObjectSequence	1	DICOM_2011
(0070,0009)	SQ	GraphicObjectSequence	1	DICOM_2011
(0070,0010)	FL	BoundingBoxTopLeftHandCorner	2	DICOM_2011
(0070,0011)	FL	BoundingBoxBottomRightHandCorner	2	DICOM_2011
(0070,0012)	CS	BoundingBoxTextHorizontalJustification	1	DICOM_2011
(0070,0014)	FL	AnchorPoint	2	DICOM_2011
(0070,0015)	CS	AnchorPointVisibility	1	DICOM_2011
(0070,0020)	US	GraphicDimensions	1	DICOM_2011
(0070,0021)	US	NumberOfGraphicPoints	1	DICOM_2011
(0070,0022)	FL	GraphicData	2-n	DICOM_2011
(0070,0023)	CS	GraphicType	1	DICOM_2011
(0070,0024)	CS	GraphicFilled	1	DICOM_2011
(0070,0041)	CS	ImageHorizontalFlip	1	DICOM_2011
(0070,0042)	US	ImageRotation	1	DICOM_2011
(0070,0052)	SL	DisplayedAreaTopLeftHandCorner	2	DICOM_2011
(0070,0053)	SL	DisplayedAreaBottomRightHandCorner	2	DICOM_2011
(0070,005A)	SQ	DisplayedAreaSelectionSequence	1	DICOM_2011
(0070,0060)	SQ	GraphicLayerSequence	1	DICOM_2011
(0070,0062)	IS	GraphicLayerOrder	1	DICOM_2011
(0070,0066)	US	GraphicLayerRecommendedDisplayGrayscaleValue	1	DICOM_2011
(0070,0068)	LO	GraphicLayerDescription	1	DICOM_2011
(0070,0080)	CS	ContentLabel	1	DICOM_2011
(0070,0081)	LO	ContentDescription	1	DICOM_2011
(0070,0082)	DA	PresentationCreationDate	1	DICOM_2011
(0070,0083)	TM	PresentationCreationTime	1	DICOM_2011
(0070,0084)	PN	ContentCreatorName	1	DICOM_2011
(0070,0086)	SQ	ContentCreatorIdentificationCodeSequence	1	DICOM_2011
(0070,0087)	SQ	AlternateContentDescriptionSequence	1	DICOM_2011
(0070,0100)	CS	PresentationSizeMode	1	DICOM_2011
(0070,0101)	DS	PresentationPixelSpacing	2	DICOM_2011
(0070,0102)	IS	PresentationPixelAspectRatio	2	DICOM_2011
(0070,0103)	FL	PresentationPixelMagnificationRatio	1	DICOM_2011
(0070,0207)	LO	GraphicGroupLabel	1	DICOM_2011
(0070,0208)	ST	GraphicGroupDescription	1	DICOM_2011
(0070,0209)	SQ	CompoundGraphicSequence	1	DICOM_2011
(0070,0226)	UL	CompoundGraphicInstanceID	1	DICOM_2011
(0070,0227)	LO	FontName	1	DICOM_2011
(0070,0228)	CS	FontNameType	1	DICOM_2011
(0070,0229)	LO	CSSFontName	1	DICOM_2011
(0070,0230)	FD	RotationAngle	1	DICOM_2011
(0070,0231)	SQ	TextStyleSequence	1	DICOM_2011
(0070,0232)	SQ	LineStyleSequence	1	DICOM_2011
(0070,0233)	SQ	FillStyleSequence	1	DICOM_2011
(0070,0234)	SQ	GraphicGroupSequence	1	DICOM_2011
(0070,0241)	US	TextColorCIELabValue	3	DICOM_2011
(0070,0242)	CS	HorizontalAlignment	1	DICOM_2011
(0070,0243)	CS	VerticalAlignment	1	DICOM_2011
(0070,0244)	CS	ShadowStyle	1	DICOM_2011
(0070,0245)	FL	ShadowOffsetX	1	DICOM_2011
(0070,0246)	FL	ShadowOffsetY	1	DICOM_2011
(0070,0247)	US	ShadowColorCIELabValue	3	DICOM_2011
(0070,0248)	CS	Underlined	1	DICOM_2011
(0070,0249)	CS	Bold	1	DICOM_2011
(0070,0250)	CS	Italic	1	DICOM_2011
(0070,0251)	US	PatternOnColorCIELabValue	3	DICOM_2011
(0070,0252)	US	PatternOffColorCIELabValue	3	DICOM_2011
(0070,0253)	FL	LineThickness	1	DICOM_2011
(0070,0254)	CS	LineDashingStyle	1	DICOM_2011
(0070,0255)	UL	LinePattern	1	DICOM_2011
(0070,0256)	OB	FillPattern	1	DICOM_2011
(0070,0257)	CS	FillMode	1	DICOM_2011
(0070,0258)	FL	ShadowOpacity	1	DICOM_2011
(0070,0261)	FL	GapLength	1	DICOM_2011
(0070,0262)	FL	DiameterOfVisibility	1	DICOM_2011
(0070,0273)	FL	RotationPoint	2	DICOM_2011
(0070,0274)	CS	TickAlignment	1	DICOM_2011
(0070,0278)	CS	ShowTickLabel	1	DICOM_2011
(0070,0279)	CS	TickLabelAlignment	1	DICOM_2011
(0070,0282)	CS	CompoundGraphicUnits	1	DICOM_2011
(0070,0284)	FL	PatternOnOpacity	1	DICOM_2011
(0070,0285)	FL	PatternOffOpacity	1	DICOM_2011
(0070,0287)	SQ	MajorTicksSequence	1	DICOM_2011
(0070,0288)	FL	TickPosition	1	DICOM_2011
(0070,0289)	SH	TickLabel	1	DICOM_2011
(0070,0294)	CS	CompoundGraphicType	1	DICOM_2011
(0070,0295)	UL	GraphicGroupID	1	DICOM_2011
(0070,0306)	CS	ShapeType	1	DICOM_2011
(0070,0308)	SQ	RegistrationSequence	1	DICOM_2011
(0070,0309)	SQ	MatrixRegistrationSequence	1	DICOM_2011
(0070,030A)	SQ	MatrixSequence	1	DICOM_2011
(0070,030C)	CS	FrameOfReferenceTransformationMatrixType	1	DICOM_2011
(0070,030D)	SQ	RegistrationTypeCodeSequence	1	DICOM_2011
(0070,030F)	ST	FiducialDescription	1	DICOM_2011
(0070,0310)	SH	FiducialIdentifier	1	DICOM_2011
(0070,0311)	SQ	FiducialIdentifierCodeSequence	1	DICOM_2011
(0070,0312)	FD	ContourUncertaintyRadius	1	DICOM_2011
(0070,0314)	SQ	UsedFiducialsSequence	1	DICOM_2011
(0070,0318)	SQ	GraphicCoordinatesDataSequence	1	DICOM_2011
(0070,031A)	UI	FiducialUID	1	DICOM_2011
(0070,031C)	SQ	FiducialSetSequence	1	DICOM_2011
(0070,031E)	SQ	FiducialSequence	1	DICOM_2011
(0070,0401)	US	GraphicLayerRecommendedDisplayCIELabValue	3	DICOM_2011
(0070,0402)	SQ	BlendingSequence	1	DICOM_2011
(0070,0403)	FL	RelativeOpacity	1	DICOM_2011
(0070,0404)	SQ	ReferencedSpatialRegistrationSequence	1	DICOM_2011
(0070,0405)	CS	BlendingPosition	1	DICOM_2011
(0072,0002)	SH	HangingProtocolName	1	DICOM_2011
(0072,0004)	LO	HangingProtocolDescription	1	DICOM_2011
(0072,0006)	CS	HangingProtocolLevel	1	DICOM_2011
(0072,0008)	LO	HangingProtocolCreator	1	DICOM_2011
(0072,000A)	DT	HangingProtocolCreationDateTime	1	DICOM_2011
(0072,000C)	SQ	HangingProtocolDefinitionSequence	1	DICOM_2011
(0072,000E)	SQ	HangingProtocolUserIdentificationCodeSequence	1	DICOM_2011
(0072,0010)	LO	HangingProtocolUserGroupName	1	DICOM_2011
(0072,0012)	SQ	SourceHangingProtocolSequence	1	DICOM_2011
(0072,0014)	US	NumberOfPriorsReferenced	1	DICOM_2011
(0072,0020)	SQ	ImageSetsSequence	1	DICOM_2011
(0072,0022)	SQ	ImageSetSelectorSequence	1	DICOM_2011
(0072,0024)	CS	ImageSetSelectorUsageFlag	1	DICOM_2011
(0072,0026)	AT	SelectorAttribute	1	DICOM_2011
(0072,0028)	US	SelectorValueNumber	1	DICOM_2011
(0072,0030)	SQ	TimeBasedImageSetsSequence	1	DICOM_2011
(0072,0032)	US	ImageSetNumber	1	DICOM_2011
(0072,0034)	CS	ImageSetSelectorCategory	1	DICOM_2011
(0072,0038)	US	RelativeTime	2	DICOM_2011
(0072,003A)	CS	RelativeTimeUnits	1	DICOM_2011
(0072,003C)	SS	AbstractPriorValue	2	DICOM_2011
(0072,003E)	SQ	AbstractPriorCodeSequence	1	DICOM_2011
(0072,0040)	LO	ImageSetLabel	1	DICOM_2011
(0072,0050)	CS	SelectorAttributeVR	1	DICOM_2011
(0072,0052)	AT	SelectorSequencePointer	1-n	DICOM_2011
(0072,0054)	LO	SelectorSequencePointerPrivateCreator	1-n	DICOM_2011
(0072,0056)	LO	SelectorAttributePrivateCreator	1	DICOM_2011
(0072,0060)	AT	SelectorATValue	1-n	DICOM_2011
(0072,0062)	CS	SelectorCSValue	1-n	DICOM_2011
(0072,0064)	IS	SelectorISValue	1-n	DICOM_2011
(0072,0066)	LO	SelectorLOValue	1-n	DICOM_2011
(0072,0068)	LT	SelectorLTValue	1	DICOM_2011
(0072,006A)	PN	SelectorPNValue	1-n	DICOM_2011
(0072,006C)	SH	SelectorSHValue	1-n	DICOM_2011
(0072,006E)	ST	SelectorSTValue	1	DICOM_2011
(0072,0070)	UT	SelectorUTValue	1	DICOM_2011
(0072,0072)	DS	SelectorDSValue	1-n	DICOM_2011
(0072,0074)	FD	SelectorFDValue	1-n	DICOM_2011
(0072,0076)	FL	SelectorFLValue	1-n	DICOM_2011
(0072,0078)	UL	SelectorULValue	1-n	DICOM_2011
(0072,007A)	US	SelectorUSValue	1-n	DICOM_2011
(0072,007C)	SL	SelectorSLValue	1-n	DICOM_2011
(0072,007E)	SS	SelectorSSValue	1-n	DICOM_2011
(0072,0080)	SQ	SelectorCodeSequenceValue	1	DICOM_2011
(0072,0100)	US	NumberOfScreens	1	DICOM_2011
(0072,0102)	SQ	NominalScreenDefinitionSequence	1	DICOM_2011
(0072,0104)	US	NumberOfVerticalPixels	1	DICOM_2011
(0072,0106)	US	NumberOfHorizontalPixels	1	DICOM_2011
(0072,0108)	FD	DisplayEnvironmentSpatialPosition	4	DICOM_2011
(0072,010A)	US	ScreenMinimumGrayscaleBitDepth	1	DICOM_2011
(0072,010C)	US	ScreenMinimumColorBitDepth	1	DICOM_2011
(0072,010E)	US	ApplicationMaximumRepaintTime	1	DICOM_2011
(0072,0200)	SQ	DisplaySetsSequence	1	DICOM_2011
(0072,0202)	US	DisplaySetNumber	1	DICOM_2011
(0072,0203)	LO	DisplaySetLabel	1	DICOM_2011
(0072,0204)	US	DisplaySetPresentationGroup	1	DICOM_2011
(0072,0206)	LO	DisplaySetPresentationGroupDescription	1	DICOM_2011
(0072,0208)	CS	PartialDataDisplayHandling	1	DICOM_2011
(0072,0210)	SQ	SynchronizedScrollingSequence	1	DICOM_2011
(0072,0212)	US	DisplaySetScrollingGroup	2-n	DICOM_2011
(0072,0214)	SQ	NavigationIndicatorSequence	1	DICOM_2011
(0072,0216)	US	NavigationDisplaySet	1	DICOM_2011
(0072,0218)	US	ReferenceDisplaySets	1-n	DICOM_2011
(0072,0300)	SQ	ImageBoxesSequence	1	DICOM_2011
(0072,0302)	US	ImageBoxNumber	1	DICOM_2011
(0072,0304)	CS	ImageBoxLayoutType	1	DICOM_2011
(0072,0306)	US	ImageBoxTileHorizontalDimension	1	DICOM_2011
(0072,0308)	US	ImageBoxTileVerticalDimension	1	DICOM_2011
(0072,0310)	CS	ImageBoxScrollDirection	1	DICOM_2011
(0072,0312)	CS	ImageBoxSmallScrollType	1	DICOM_2011
(0072,0314)	US	ImageBoxSmallScrollAmount	1	DICOM_2011
(0072,0316)	CS	ImageBoxLargeScrollType	1	DICOM_2011
(0072,0318)	US	ImageBoxLargeScrollAmount	1	DICOM_2011
(0072,0320)	US	ImageBoxOverlapPriority	1	DICOM_2011
(0072,0330)	FD	CineRelativeToRealTime	1	DICOM_2011
(0072,0400)	SQ	FilterOperationsSequence	1	DICOM_2011
(0072,0402)	CS	FilterByCategory	1	DICOM_2011
(0072,0404)	CS	FilterByAttributePresence	1	DICOM_2011
(0072,0406)	CS	FilterByOperator	1	DICOM_2011
(0072,0420)	US	StructuredDisplayBackgroundCIELabValue	3	DICOM_2011
(0072,0421)	US	EmptyImageBoxCIELabValue	3	DICOM_2011
(0072,0422)	SQ	StructuredDisplayImageBoxSequence	1	DICOM_2011
(0072,0424)	SQ	StructuredDisplayTextBoxSequence	1	DICOM_2011
(0072,0427)	SQ	ReferencedFirstFrameSequence	1	DICOM_2011
(0072,0430)	SQ	ImageBoxSynchronizationSequence	1	DICOM_2011
(0072,0432)	US	SynchronizedImageBoxList	2-n	DICOM_2011
(0072,0434)	CS	TypeOfSynchronization	1	DICOM_2011
(0072,0500)	CS	BlendingOperationType	1	DICOM_2011
(0072,0510)	CS	ReformattingOperationType	1	DICOM_2011
(0072,0512)	FD	ReformattingThickness	1	DICOM_2011
(0072,0514)	FD	ReformattingInterval	1	DICOM_2011
(0072,0516)	CS	ReformattingOperationInitialViewDirection	1	DICOM_2011
(0072,0520)	CS	ThreeDRenderingType	1-n	DICOM_2011
(0072,0600)	SQ	SortingOperationsSequence	1	DICOM_2011
(0072,0602)	CS	SortByCategory	1	DICOM_2011
(0072,0604)	CS	SortingDirection	1	DICOM_2011
(0072,0700)	CS	DisplaySetPatientOrientation	2	DICOM_2011
(0072,0702)	CS	VOIType	1	DICOM_2011
(0072,0704)	CS	PseudoColorType	1	DICOM_2011
(0072,0705)	SQ	PseudoColorPaletteInstanceReferenceSequence	1	DICOM_2011
(0072,0706)	CS	ShowGrayscaleInverted	1	DICOM_2011
(0072,0710)	CS	ShowImageTrueSizeFlag	1	DICOM_2011
(0072,0712)	CS	ShowGraphicAnnotationFlag	1	DICOM_2011
(0072,0714)	CS	ShowPatientDemographicsFlag	1	DICOM_2011
(0072,0716)	CS	ShowAcquisitionTechniquesFlag	1	DICOM_2011
(0072,0717)	CS	DisplaySetHorizontalJustification	1	DICOM_2011
(0072,0718)	CS	DisplaySetVerticalJustification	1	DICOM_2011
(0074,0120)	FD	ContinuationStartMeterset	1	DICOM_2011
(0074,0121)	FD	ContinuationEndMeterset	1	DICOM_2011
(0074,1000)	CS	ProcedureStepState	1	DICOM_2011
(0074,1002)	SQ	ProcedureStepProgressInformationSequence	1	DICOM_2011
(0074,1004)	DS	ProcedureStepProgress	1	DICOM_2011
(0074,1006)	ST	ProcedureStepProgressDescription	1	DICOM_2011
(0074,1008)	SQ	ProcedureStepCommunicationsURISequence	1	DICOM_2011
(0074,100A)	ST	ContactURI	1	DICOM_2011
(0074,100C)	LO	ContactDisplayName	1	DICOM_2011
(0074,100E)	SQ	ProcedureStepDiscontinuationReasonCodeSequence	1	DICOM_2011
(0074,1020)	SQ	BeamTaskSequence	1	DICOM_2011
(0074,1022)	CS	BeamTaskType	1	DICOM_2011
(0074,1026)	FD	TableTopVerticalAdjustedPosition	1	DICOM_2011
(0074,1027)	FD	TableTopLongitudinalAdjustedPosition	1	DICOM_2011
(0074,1028)	FD	TableTopLateralAdjustedPosition	1	DICOM_2011
(0074,102A)	FD	PatientSupportAdjustedAngle	1	DICOM_2011
(0074,102B)	FD	TableTopEccentricAdjustedAngle	1	DICOM_2011
(0074,102C)	FD	TableTopPitchAdjustedAngle	1	DICOM_2011
(0074,102D)	FD	TableTopRollAdjustedAngle	1	DICOM_2011
(0074,1030)	SQ	DeliveryVerificationImageSequence	1	DICOM_2011
(0074,1032)	CS	VerificationImageTiming	1	DICOM_2011
(0074,1034)	CS	DoubleExposureFlag	1	DICOM_2011
(0074,1036)	CS	DoubleExposureOrdering	1	DICOM_2011
(0074,1040)	SQ	RelatedReferenceRTImageSequence	1	DICOM_2011
(0074,1042)	SQ	GeneralMachineVerificationSequence	1	DICOM_2011
(0074,1044)	SQ	ConventionalMachineVerificationSequence	1	DICOM_2011
(0074,1046)	SQ	IonMachineVerificationSequence	1	DICOM_2011
(0074,1048)	SQ	FailedAttributesSequence	1	DICOM_2011
(0074,104A)	SQ	OverriddenAttributesSequence	1	DICOM_2011
(0074,104C)	SQ	ConventionalControlPointVerificationSequence	1	DICOM_2011
(0074,104E)	SQ	IonControlPointVerificationSequence	1	DICOM_2011
(0074,1050)	SQ	AttributeOccurrenceSequence	1	DICOM_2011
(0074,1052)	AT	AttributeOccurrencePointer	1	DICOM_2011
(0074,1054)	UL	AttributeItemSelector	1	DICOM_2011
(0074,1056)	LO	AttributeOccurrencePrivateCreator	1	DICOM_2011
(0074,1057)	IS	SelectorSequencePointerItems	1-n	DICOM_2011
(0074,1200)	CS	ScheduledProcedureStepPriority	1	DICOM_2011
(0074,1202)	LO	WorklistLabel	1	DICOM_2011
(0074,1204)	LO	ProcedureStepLabel	1	DICOM_2011
(0074,1210)	SQ	ScheduledProcessingParametersSequence	1	DICOM_2011
(0074,1212)	SQ	PerformedProcessingParametersSequence	1	DICOM_2011
(0074,1216)	SQ	UnifiedProcedureStepPerformedProcedureSequence	1	DICOM_2011
(0074,1224)	SQ	ReplacedProcedureStepSequence	1	DICOM_2011
(0074,1230)	LO	DeletionLock	1	DICOM_2011
(0074,1234)	AE	ReceivingAE	1	DICOM_2011
(0074,1236)	AE	RequestingAE	1	DICOM_2011
(0074,1238)	LT	ReasonForCancellation	1	DICOM_2011
(0074,1242)	CS	SCPStatus	1	DICOM_2011
(0074,1244)	CS	SubscriptionListStatus	1	DICOM_2011
(0074,1246)	CS	UnifiedProcedureStepListStatus	1	DICOM_2011
(0074,1324)	UL	BeamOrderIndex	1	DICOM_2011
(0074,1338)	FD	DoubleExposureMeterset	1	DICOM_2011
(0074,133A)	FD	DoubleExposureFieldDelta	4	DICOM_2011
(0076,0001)	LO	ImplantAssemblyTemplateName	1	DICOM_2011
(0076,0003)	LO	ImplantAssemblyTemplateIssuer	1	DICOM_2011
(0076,0006)	LO	ImplantAssemblyTemplateVersion	1	DICOM_2011
(0076,0008)	SQ	ReplacedImplantAssemblyTemplateSequence	1	DICOM_2011
(0076,000A)	CS	ImplantAssemblyTemplateType	1	DICOM_2011
(0076,000C)	SQ	OriginalImplantAssemblyTemplateSequence	1	DICOM_2011
(0076,000E)	SQ	DerivationImplantAssemblyTemplateSequence	1	DICOM_2011
(0076,0010)	SQ	ImplantAssemblyTemplateTargetAnatomySequence	1	DICOM_2011
(0076,0020)	SQ	ProcedureTypeCodeSequence	1	DICOM_2011
(0076,0030)	LO	SurgicalTechnique	1	DICOM_2011
(0076,0032)	SQ	ComponentTypesSequence	1	DICOM_2011
(0076,0034)	CS	ComponentTypeCodeSequence	1	DICOM_2011
(0076,0036)	CS	ExclusiveComponentType	1	DICOM_2011
(0076,0038)	CS	MandatoryComponentType	1	DICOM_2011
(0076,0040)	SQ	ComponentSequence	1	DICOM_2011
(0076,0055)	US	ComponentID	1	DICOM_2011
(0076,0060)	SQ	ComponentAssemblySequence	1	DICOM_2011
(0076,0070)	US	Component1ReferencedID	1	DICOM_2011
(0076,0080)	US	Component1ReferencedMatingFeatureSetID	1	DICOM_2011
(0076,0090)	US	Component1ReferencedMatingFeatureID	1	DICOM_2011
(0076,00A0)	US	Component2ReferencedID	1	DICOM_2011
(0076,00B0)	US	Component2ReferencedMatingFeatureSetID	1	DICOM_2011
(0076,00C0)	US	Component2ReferencedMatingFeatureID	1	DICOM_2011
(0078,0001)	LO	ImplantTemplateGroupName	1	DICOM_2011
(0078,0010)	ST	ImplantTemplateGroupDescription	1	DICOM_2011
(0078,0020)	LO	ImplantTemplateGroupIssuer	1	DICOM_2011
(0078,0024)	LO	ImplantTemplateGroupVersion	1	DICOM_2011
(0078,0026)	SQ	ReplacedImplantTemplateGroupSequence	1	DICOM_2011
(0078,0028)	SQ	ImplantTemplateGroupTargetAnatomySequence	1	DICOM_2011
(0078,002A)	SQ	ImplantTemplateGroupMembersSequence	1	DICOM_2011
(0078,002E)	US	ImplantTemplateGroupMemberID	1	DICOM_2011
(0078,0050)	FD	ThreeDImplantTemplateGroupMemberMatchingPoint	3	DICOM_2011
(0078,0060)	FD	ThreeDImplantTemplateGroupMemberMatchingAxes	9	DICOM_2011
(0078,0070)	SQ	ImplantTemplateGroupMemberMatching2DCoordinatesSequence	1	DICOM_2011
(0078,0090)	FD	TwoDImplantTemplateGroupMemberMatchingPoint	2	DICOM_2011
(0078,00A0)	FD	TwoDImplantTemplateGroupMemberMatchingAxes	4	DICOM_2011
(0078,00B0)	SQ	ImplantTemplateGroupVariationDimensionSequence	1	DICOM_2011
(0078,00B2)	LO	ImplantTemplateGroupVariationDimensionName	1	DICOM_2011
(0078,00B4)	SQ	ImplantTemplateGroupVariationDimensionRankSequence	1	DICOM_2011
(0078,00B6)	US	ReferencedImplantTemplateGroupMemberID	1	DICOM_2011
(0078,00B8)	US	ImplantTemplateGroupVariationDimensionRank	1	DICOM_2011
(0088,0130)	SH	StorageMediaFileSetID	1	DICOM_2011
(0088,0140)	UI	StorageMediaFileSetUID	1	DICOM_2011
(0088,0200)	SQ	IconImageSequence	1	DICOM_2011
(0100,0410)	CS	SOPInstanceStatus	1	DICOM_2011
(0100,0420)	DT	SOPAuthorizationDateTime	1	DICOM_2011
(0100,0424)	LT	SOPAuthorizationComment	1	DICOM_2011
(0100,0426)	LO	AuthorizationEquipmentCertificationNumber	1	DICOM_2011
(0400,0005)	US	MACIDNumber	1	DICOM_2011
(0400,0010)	UI	MACCalculationTransferSyntaxUID	1	DICOM_2011
(0400,0015)	CS	MACAlgorithm	1	DICOM_2011
(0400,0020)	AT	DataElementsSigned	1-n	DICOM_2011
(0400,0100)	UI	DigitalSignatureUID	1	DICOM_2011
(0400,0105)	DT	DigitalSignatureDateTime	1	DICOM_2011
(0400,0110)	CS	CertificateType	1	DICOM_2011
(0400,0115)	OB	CertificateOfSigner	1	DICOM_2011
(0400,0120)	OB	Signature	1	DICOM_2011
(0400,0305)	CS	CertifiedTimestampType	1	DICOM_2011
(0400,0310)	OB	CertifiedTimestamp	1	DICOM_2011
(0400,0401)	SQ	DigitalSignaturePurposeCodeSequence	1	DICOM_2011
(0400,0402)	SQ	ReferencedDigitalSignatureSequence	1	DICOM_2011
(0400,0403)	SQ	ReferencedSOPInstanceMACSequence	1	DICOM_2011
(0400,0404)	OB	MAC	1	DICOM_2011
(0400,0500)	SQ	EncryptedAttributesSequence	1	DICOM_2011
(0400,0510)	UI	EncryptedContentTransferSyntaxUID	1	DICOM_2011
(0400,0520)	OB	EncryptedContent	1	DICOM_2011
(0400,0550)	SQ	ModifiedAttributesSequence	1	DICOM_2011
(0400,0561)	SQ	OriginalAttributesSequence	1	DICOM_2011
(0400,0562)	DT	AttributeModificationDateTime	1	DICOM_2011
(0400,0563)	LO	ModifyingSystem	1	DICOM_2011
(0400,0564)	LO	SourceOfPreviousValues	1	DICOM_2011
(0400,0565)	CS	ReasonForTheAttributeModification	1	DICOM_2011
(2000,0010)	IS	NumberOfCopies	1	DICOM_2011
(2000,001E)	SQ	PrinterConfigurationSequence	1	DICOM_2011
(2000,0020)	CS	PrintPriority	1	DICOM_2011
(2000,0030)	CS	MediumType	1	DICOM_2011
(2000,0040)	CS	FilmDestination	1	DICOM_2011
(2000,0050)	LO	FilmSessionLabel	1	DICOM_2011
(2000,0060)	IS	MemoryAllocation	1	DICOM_2011
(2000,0061)	IS	MaximumMemoryAllocation	1	DICOM_2011
(2000,00A0)	US	MemoryBitDepth	1	DICOM_2011
(2000,00A1)	US	PrintingBitDepth	1	DICOM_2011
(2000,00A2)	SQ	MediaInstalledSequence	1	DICOM_2011
(2000,00A4)	SQ	OtherMediaAvailableSequence	1	DICOM_2011
(2000,00A8)	SQ	SupportedImageDisplayFormatsSequence	1	DICOM_2011
(2000,0500)	SQ	ReferencedFilmBoxSequence	1	DICOM_2011
(2010,0010)	ST	ImageDisplayFormat	1	DICOM_2011
(2010,0030)	CS	AnnotationDisplayFormatID	1	DICOM_2011
(2010,0040)	CS	FilmOrientation	1	DICOM_2011
(2010,0050)	CS	FilmSizeID	1	DICOM_2011
(2010,0052)	CS	PrinterResolutionID	1	DICOM_2011
(2010,0054)	CS	DefaultPrinterResolutionID	1	DICOM_2011
(2010,0060)	CS	MagnificationType	1	DICOM_2011
(2010,0080)	CS	SmoothingType	1	DICOM_2011
(2010,00A6)	CS	DefaultMagnificationType	1	DICOM_2011
(2010,00A7)	CS	OtherMagnificationTypesAvailable	1-n	DICOM_2011
(2010,00A8)	CS	DefaultSmoothingType	1	DICOM_2011
(2010,00A9)	CS	OtherSmoothingTypesAvailable	1-n	DICOM_2011
(2010,0100)	CS	BorderDensity	1	DICOM_2011
(2010,0110)	CS	EmptyImageDensity	1	DICOM_2011
(2010,0120)	US	MinDensity	1	DICOM_2011
(2010,0130)	US	MaxDensity	1	DICOM_2011
(2010,0140)	CS	Trim	1	DICOM_2011
(2010,0150)	ST	ConfigurationInformation	1	DICOM_2011
(2010,0152)	LT	ConfigurationInformationDescription	1	DICOM_2011
(2010,0154)	IS	MaximumCollatedFilms	1	DICOM_2011
(2010,015E)	US	Illumination	1	DICOM_2011
(2010,0160)	US	ReflectedAmbientLight	1	DICOM_2011
(2010,0376)	DS	PrinterPixelSpacing	2	DICOM_2011
(2010,0500)	SQ	ReferencedFilmSessionSequence	1	DICOM_2011
(2010,0510)	SQ	ReferencedImageBoxSequence	1	DICOM_2011
(2010,0520)	SQ	ReferencedBasicAnnotationBoxSequence	1	DICOM_2011
(2020,0010)	US	ImageBoxPosition	1	DICOM_2011
(2020,0020)	CS	Polarity	1	DICOM_2011
(2020,0030)	DS	RequestedImageSize	1	DICOM_2011
(2020,0040)	CS	RequestedDecimateCropBehavior	1	DICOM_2011
(2020,0050)	CS	RequestedResolutionID	1	DICOM_2011
(2020,00A0)	CS	RequestedImageSizeFlag	1	DICOM_2011
(2020,00A2)	CS	DecimateCropResult	1	DICOM_2011
(2020,0110)	SQ	BasicGrayscaleImageSequence	1	DICOM_2011
(2020,0111)	SQ	BasicColorImageSequence	1	DICOM_2011
(2030,0010)	US	AnnotationPosition	1	DICOM_2011
(2030,0020)	LO	TextString	1	DICOM_2011
(2050,0010)	SQ	PresentationLUTSequence	1	DICOM_2011
(2050,0020)	CS	PresentationLUTShape	1	DICOM_2011
(2050,0500)	SQ	ReferencedPresentationLUTSequence	1	DICOM_2011
(2100,0020)	CS	ExecutionStatus	1	DICOM_2011
(2100,0030)	CS	ExecutionStatusInfo	1	DICOM_2011
(2100,0040)	DA	CreationDate	1	DICOM_2011
(2100,0050)	TM	CreationTime	1	DICOM_2011
(2100,0070)	AE	Originator	1	DICOM_2011
(2100,0160)	SH	OwnerID	1	DICOM_2011
(2100,0170)	IS	NumberOfFilms	1	DICOM_2011
(2110,0010)	CS	PrinterStatus	1	DICOM_2011
(2110,0020)	CS	PrinterStatusInfo	1	DICOM_2011
(2110,0030)	LO	PrinterName	1	DICOM_2011
(2200,0001)	CS	LabelUsingInformationExtractedFromInstances	1	DICOM_2011
(2200,0002)	UT	LabelText	1	DICOM_2011
(2200,0003)	CS	LabelStyleSelection	1	DICOM_2011
(2200,0004)	LT	MediaDisposition	1	DICOM_2011
(2200,0005)	LT	BarcodeValue	1	DICOM_2011
(2200,0006)	CS	BarcodeSymbology	1	DICOM_2011
(2200,0007)	CS	AllowMediaSplitting	1	DICOM_2011
(2200,0008)	CS	IncludeNonDICOMObjects	1	DICOM_2011
(2200,0009)	CS	IncludeDisplayApplication	1	DICOM_2011
(2200,000A)	CS	PreserveCompositeInstancesAfterMediaCreation	1	DICOM_2011
(2200,000B)	US	TotalNumberOfPiecesOfMediaCreated	1	DICOM_2011
(2200,000C)	LO	RequestedMediaApplicationProfile	1	DICOM_2011
(2200,000D)	SQ	ReferencedStorageMediaSequence	1	DICOM_2011
(2200,000E)	AT	FailureAttributes	1-n	DICOM_2011
(2200,000F)	CS	AllowLossyCompression	1	DICOM_2011
(2200,0020)	CS	RequestPriority	1	DICOM_2011
(3002,0002)	SH	RTImageLabel	1	DICOM_2011
(3002,0003)	LO	RTImageName	1	DICOM_2011
(3002,0004)	ST	RTImageDescription	1	DICOM_2011
(3002,000A)	CS	ReportedValuesOrigin	1	DICOM_2011
(3002,000C)	CS	RTImagePlane	1	DICOM_2011
(3002,000D)	DS	XRayImageReceptorTranslation	3	DICOM_2011
(3002,000E)	DS	XRayImageReceptorAngle	1	DICOM_2011
(3002,0010)	DS	RTImageOrientation	6	DICOM_2011
(3002,0011)	DS	ImagePlanePixelSpacing	2	DICOM_2011
(3002,0012)	DS	RTImagePosition	2	DICOM_2011
(3002,0020)	SH	RadiationMachineName	1	DICOM_2011
(3002,0022)	DS	RadiationMachineSAD	1	DICOM_2011
(3002,0024)	DS	RadiationMachineSSD	1	DICOM_2011
(3002,0026)	DS	RTImageSID	1	DICOM_2011
(3002,0028)	DS	SourceToReferenceObjectDistance	1	DICOM_2011
(3002,0029)	IS	FractionNumber	1	DICOM_2011
(3002,0030)	SQ	ExposureSequence	1	DICOM_2011
(3002,0032)	DS	MetersetExposure	1	DICOM_2011
(3002,0034)	DS	DiaphragmPosition	4	DICOM_2011
(3002,0040)	SQ	FluenceMapSequence	1	DICOM_2011
(3002,0041)	CS	FluenceDataSource	1	DICOM_2011
(3002,0042)	DS	FluenceDataScale	1	DICOM_2011
(3002,0050)	SQ	PrimaryFluenceModeSequence	1	DICOM_2011
(3002,0051)	CS	FluenceMode	1	DICOM_2011
(3002,0052)	SH	FluenceModeID	1	DICOM_2011
(3004,0001)	CS	DVHType	1	DICOM_2011
(3004,0002)	CS	DoseUnits	1	DICOM_2011
(3004,0004)	CS	DoseType	1	DICOM_2011
(3004,0006)	LO	DoseComment	1	DICOM_2011
(3004,0008)	DS	NormalizationPoint	3	DICOM_2011
(3004,000A)	CS	DoseSummationType	1	DICOM_2011
(3004,000C)	DS	GridFrameOffsetVector	2-n	DICOM_2011
(3004,000E)	DS	DoseGridScaling	1	DICOM_2011
(3004,0010)	SQ	RTDoseROISequence	1	DICOM_2011
(3004,0012)	DS	DoseValue	1	DICOM_2011
(3004,0014)	CS	TissueHeterogeneityCorrection	1-3	DICOM_2011
(3004,0040)	DS	DVHNormalizationPoint	3	DICOM_2011
(3004,0042)	DS	DVHNormalizationDoseValue	1	DICOM_2011
(3004,0050)	SQ	DVHSequence	1	DICOM_2011
(3004,0052)	DS	DVHDoseScaling	1	DICOM_2011
(3004,0054)	CS	DVHVolumeUnits	1	DICOM_2011
(3004,0056)	IS	DVHNumberOfBins	1	DICOM_2011
(3004,0058)	DS	DVHData	2-2n	DICOM_2011
(3004,0060)	SQ	DVHReferencedROISequence	1	DICOM_2011
(3004,0062)	CS	DVHROIContributionType	1	DICOM_2011
(3004,0070)	DS	DVHMinimumDose	1	DICOM_2011
(3004,0072)	DS	DVHMaximumDose	1	DICOM_2011
(3004,0074)	DS	DVHMeanDose	1	DICOM_2011
(3006,0002)	SH	StructureSetLabel	1	DICOM_2011
(3006,0004)	LO	StructureSetName	1	DICOM_2011
(3006,0006)	ST	StructureSetDescription	1	DICOM_2011
(3006,0008)	DA	StructureSetDate	1	DICOM_2011
(3006,0009)	TM	StructureSetTime	1	DICOM_2011
(3006,0010)	SQ	ReferencedFrameOfReferenceSequence	1	DICOM_2011
(3006,0012)	SQ	RTReferencedStudySequence	1	DICOM_2011
(3006,0014)	SQ	RTReferencedSeriesSequence	1	DICOM_2011
(3006,0016)	SQ	ContourImageSequence	1	DICOM_2011
(3006,0020)	SQ	StructureSetROISequence	1	DICOM_2011
(3006,0022)	IS	ROINumber	1	DICOM_2011
(3006,0024)	UI	ReferencedFrameOfReferenceUID	1	DICOM_2011
(3006,0026)	LO	ROIName	1	DICOM_2011
(3006,0028)	ST	ROIDescription	1	DICOM_2011
(3006,002A)	IS	ROIDisplayColor	3	DICOM_2011
(3006,002C)	DS	ROIVolume	1	DICOM_2011
(3006,0030)	SQ	RTRelatedROISequence	1	DICOM_2011
(3006,0033)	CS	RTROIRelationship	1	DICOM_2011
(3006,0036)	CS	ROIGenerationAlgorithm	1	DICOM_2011
(3006,0038)	LO	ROIGenerationDescription	1	DICOM_2011
(3006,0039)	SQ	ROIContourSequence	1	DICOM_2011
(3006,0040)	SQ	ContourSequence	1	DICOM_2011
(3006,0042)	CS	ContourGeometricType	1	DICOM_2011
(3006,0044)	DS	ContourSlabThickness	1	DICOM_2011
(3006,0045)	DS	ContourOffsetVector	3	DICOM_2011
(3006,0046)	IS	NumberOfContourPoints	1	DICOM_2011
(3006,0048)	IS	ContourNumber	1	DICOM_2011
(3006,0049)	IS	AttachedContours	1-n	DICOM_2011
(3006,0050)	DS	ContourData	3-3n	DICOM_2011
(3006,0080)	SQ	RTROIObservationsSequence	1	DICOM_2011
(3006,0082)	IS	ObservationNumber	1	DICOM_2011
(3006,0084)	IS	ReferencedROINumber	1	DICOM_2011
(3006,0085)	SH	ROIObservationLabel	1	DICOM_2011
(3006,0086)	SQ	RTROIIdentificationCodeSequence	1	DICOM_2011
(3006,0088)	ST	ROIObservationDescription	1	DICOM_2011
(3006,00A0)	SQ	RelatedRTROIObservationsSequence	1	DICOM_2011
(3006,00A4)	CS	RTROIInterpretedType	1	DICOM_2011
(3006,00A6)	PN	ROIInterpreter	1	DICOM_2011
(3006,00B0)	SQ	ROIPhysicalPropertiesSequence	1	DICOM_2011
(3006,00B2)	CS	ROIPhysicalProperty	1	DICOM_2011
(3006,00B4)	DS	ROIPhysicalPropertyValue	1	DICOM_2011
(3006,00B6)	SQ	ROIElementalCompositionSequence	1	DICOM_2011
(3006,00B7)	US	ROIElementalCompositionAtomicNumber	1	DICOM_2011
(3006,00B8)	FL	ROIElementalCompositionAtomicMassFraction	1	DICOM_2011
(3006,00C0)	SQ	FrameOfReferenceRelationshipSequence	1	DICOM_2011
(3006,00C2)	UI	RelatedFrameOfReferenceUID	1	DICOM_2011
(3006,00C4)	CS	FrameOfReferenceTransformationType	1	DICOM_2011
(3006,00C6)	DS	FrameOfReferenceTransformationMatrix	16	DICOM_2011
(3006,00C8)	LO	FrameOfReferenceTransformationComment	1	DICOM_2011
(3008,0010)	SQ	MeasuredDoseReferenceSequence	1	DICOM_2011
(3008,0012)	ST	MeasuredDoseDescription	1	DICOM_2011
(3008,0014)	CS	MeasuredDoseType	1	DICOM_2011
(3008,0016)	DS	MeasuredDoseValue	1	DICOM_2011
(3008,0020)	SQ	TreatmentSessionBeamSequence	1	DICOM_2011
(3008,0021)	SQ	TreatmentSessionIonBeamSequence	1	DICOM_2011
(3008,0022)	IS	CurrentFractionNumber	1	DICOM_2011
(3008,0024)	DA	TreatmentControlPointDate	1	DICOM_2011
(3008,0025)	TM	TreatmentControlPointTime	1	DICOM_2011
(3008,002A)	CS	TreatmentTerminationStatus	1	DICOM_2011
(3008,002B)	SH	TreatmentTerminationCode	1	DICOM_2011
(3008,002C)	CS	TreatmentVerificationStatus	1	DICOM_2011
(3008,0030)	SQ	ReferencedTreatmentRecordSequence	1	DICOM_2011
(3008,0032)	DS	SpecifiedPrimaryMeterset	1	DICOM_2011
(3008,0033)	DS	SpecifiedSecondaryMeterset	1	DICOM_2011
(3008,0036)	DS	DeliveredPrimaryMeterset	1	DICOM_2011
(3008,0037)	DS	DeliveredSecondaryMeterset	1	DICOM_2011
(3008,003A)	DS	SpecifiedTreatmentTime	1	DICOM_2011
(3008,003B)	DS	DeliveredTreatmentTime	1	DICOM_2011
(3008,0040)	SQ	ControlPointDeliverySequence	1	DICOM_2011
(3008,0041)	SQ	IonControlPointDeliverySequence	1	DICOM_2011
(3008,0042)	DS	SpecifiedMeterset	1	DICOM_2011
(3008,0044)	DS	DeliveredMeterset	1	DICOM_2011
(3008,0045)	FL	MetersetRateSet	1	DICOM_2011
(3008,0046)	FL	MetersetRateDelivered	1	DICOM_2011
(3008,0047)	FL	ScanSpotMetersetsDelivered	1-n	DICOM_2011
(3008,0048)	DS	DoseRateDelivered	1	DICOM_2011
(3008,0050)	SQ	TreatmentSummaryCalculatedDoseReferenceSequence	1	DICOM_2011
(3008,0052)	DS	CumulativeDoseToDoseReference	1	DICOM_2011
(3008,0054)	DA	FirstTreatmentDate	1	DICOM_2011
(3008,0056)	DA	MostRecentTreatmentDate	1	DICOM_2011
(3008,005A)	IS	NumberOfFractionsDelivered	1	DICOM_2011
(3008,0060)	SQ	OverrideSequence	1	DICOM_2011
(3008,0061)	AT	ParameterSequencePointer	1	DICOM_2011
(3008,0062)	AT	OverrideParameterPointer	1	DICOM_2011
(3008,0063)	IS	ParameterItemIndex	1	DICOM_2011
(3008,0064)	IS	MeasuredDoseReferenceNumber	1	DICOM_2011
(3008,0065)	AT	ParameterPointer	1	DICOM_2011
(3008,0066)	ST	OverrideReason	1	DICOM_2011
(3008,0068)	SQ	CorrectedParameterSequence	1	DICOM_2011
(3008,006A)	FL	CorrectionValue	1	DICOM_2011
(3008,0070)	SQ	CalculatedDoseReferenceSequence	1	DICOM_2011
(3008,0072)	IS	CalculatedDoseReferenceNumber	1	DICOM_2011
(3008,0074)	ST	CalculatedDoseReferenceDescription	1	DICOM_2011
(3008,0076)	DS	CalculatedDoseReferenceDoseValue	1	DICOM_2011
(3008,0078)	DS	StartMeterset	1	DICOM_2011
(3008,007A)	DS	EndMeterset	1	DICOM_2011
(3008,0080)	SQ	ReferencedMeasuredDoseReferenceSequence	1	DICOM_2011
(3008,0082)	IS	ReferencedMeasuredDoseReferenceNumber	1	DICOM_2011
(3008,0090)	SQ	ReferencedCalculatedDoseReferenceSequence	1	DICOM_2011
(3008,0092)	IS	ReferencedCalculatedDoseReferenceNumber	1	DICOM_2011
(3008,00A0)	SQ	BeamLimitingDeviceLeafPairsSequence	1	DICOM_2011
(3008,00B0)	SQ	RecordedWedgeSequence	1	DICOM_2011
(3008,00C0)	SQ	RecordedCompensatorSequence	1	DICOM_2011
(3008,00D0)	SQ	RecordedBlockSequence	1	DICOM_2011
(3008,00E0)	SQ	TreatmentSummaryMeasuredDoseReferenceSequence	1	DICOM_2011
(3008,00F0)	SQ	RecordedSnoutSequence	1	DICOM_2011
(3008,00F2)	SQ	RecordedRangeShifterSequence	1	DICOM_2011
(3008,00F4)	SQ	RecordedLateralSpreadingDeviceSequence	1	DICOM_2011
(3008,00F6)	SQ	RecordedRangeModulatorSequence	1	DICOM_2011
(3008,0100)	SQ	RecordedSourceSequence	1	DICOM_2011
(3008,0105)	LO	SourceSerialNumber	1	DICOM_2011
(3008,0110)	SQ	TreatmentSessionApplicationSetupSequence	1	DICOM_2011
(3008,0116)	CS	ApplicationSetupCheck	1	DICOM_2011
(3008,0120)	SQ	RecordedBrachyAccessoryDeviceSequence	1	DICOM_2011
(3008,0122)	IS	ReferencedBrachyAccessoryDeviceNumber	1	DICOM_2011
(3008,0130)	SQ	RecordedChannelSequence	1	DICOM_2011
(3008,0132)	DS	SpecifiedChannelTotalTime	1	DICOM_2011
(3008,0134)	DS	DeliveredChannelTotalTime	1	DICOM_2011
(3008,0136)	IS	SpecifiedNumberOfPulses	1	DICOM_2011
(3008,0138)	IS	DeliveredNumberOfPulses	1	DICOM_2011
(3008,013A)	DS	SpecifiedPulseRepetitionInterval	1	DICOM_2011
(3008,013C)	DS	DeliveredPulseRepetitionInterval	1	DICOM_2011
(3008,0140)	SQ	RecordedSourceApplicatorSequence	1	DICOM_2011
(3008,0142)	IS	ReferencedSourceApplicatorNumber	1	DICOM_2011
(3008,0150)	SQ	RecordedChannelShieldSequence	1	DICOM_2011
(3008,0152)	IS	ReferencedChannelShieldNumber	1	DICOM_2011
(3008,0160)	SQ	BrachyControlPointDeliveredSequence	1	DICOM_2011
(3008,0162)	DA	SafePositionExitDate	1	DICOM_2011
(3008,0164)	TM	SafePositionExitTime	1	DICOM_2011
(3008,0166)	DA	SafePositionReturnDate	1	DICOM_2011
(3008,0168)	TM	SafePositionReturnTime	1	DICOM_2011
(3008,0200)	CS	CurrentTreatmentStatus	1	DICOM_2011
(3008,0202)	ST	TreatmentStatusComment	1	DICOM_2011
(3008,0220)	SQ	FractionGroupSummarySequence	1	DICOM_2011
(3008,0223)	IS	ReferencedFractionNumber	1	DICOM_2011
(3008,0224)	CS	FractionGroupType	1	DICOM_2011
(3008,0230)	CS	BeamStopperPosition	1	DICOM_2011
(3008,0240)	SQ	FractionStatusSummarySequence	1	DICOM_2011
(3008,0250)	DA	TreatmentDate	1	DICOM_2011
(3008,0251)	TM	TreatmentTime	1	DICOM_2011
(300A,0002)	SH	RTPlanLabel	1	DICOM_2011
(300A,0003)	LO	RTPlanName	1	DICOM_2011
(300A,0004)	ST	RTPlanDescription	1	DICOM_2011
(300A,0006)	DA	RTPlanDate	1	DICOM_2011
(300A,0007)	TM	RTPlanTime	1	DICOM_2011
(300A,0009)	LO	TreatmentProtocols	1-n	DICOM_2011
(300A,000A)	CS	PlanIntent	1	DICOM_2011
(300A,000B)	LO	TreatmentSites	1-n	DICOM_2011
(300A,000C)	CS	RTPlanGeometry	1	DICOM_2011
(300A,000E)	ST	PrescriptionDescription	1	DICOM_2011
(300A,0010)	SQ	DoseReferenceSequence	1	DICOM_2011
(300A,0012)	IS	DoseReferenceNumber	1	DICOM_2011
(300A,0013)	UI	DoseReferenceUID	1	DICOM_2011
(300A,0014)	CS	DoseReferenceStructureType	1	DICOM_2011
(300A,0015)	CS	NominalBeamEnergyUnit	1	DICOM_2011
(300A,0016)	LO	DoseReferenceDescription	1	DICOM_2011
(300A,0018)	DS	DoseReferencePointCoordinates	3	DICOM_2011
(300A,001A)	DS	NominalPriorDose	1	DICOM_2011
(300A,0020)	CS	DoseReferenceType	1	DICOM_2011
(300A,0021)	DS	ConstraintWeight	1	DICOM_2011
(300A,0022)	DS	DeliveryWarningDose	1	DICOM_2011
(300A,0023)	DS	DeliveryMaximumDose	1	DICOM_2011
(300A,0025)	DS	TargetMinimumDose	1	DICOM_2011
(300A,0026)	DS	TargetPrescriptionDose	1	DICOM_2011
(300A,0027)	DS	TargetMaximumDose	1	DICOM_2011
(300A,0028)	DS	TargetUnderdoseVolumeFraction	1	DICOM_2011
(300A,002A)	DS	OrganAtRiskFullVolumeDose	1	DICOM_2011
(300A,002B)	DS	OrganAtRiskLimitDose	1	DICOM_2011
(300A,002C)	DS	OrganAtRiskMaximumDose	1	DICOM_2011
(300A,002D)	DS	OrganAtRiskOverdoseVolumeFraction	1	DICOM_2011
(300A,0040)	SQ	ToleranceTableSequence	1	DICOM_2011
(300A,0042)	IS	ToleranceTableNumber	1	DICOM_2011
(300A,0043)	SH	ToleranceTableLabel	1	DICOM_2011
(300A,0044)	DS	GantryAngleTolerance	1	DICOM_2011
(300A,0046)	DS	BeamLimitingDeviceAngleTolerance	1	DICOM_2011
(300A,0048)	SQ	BeamLimitingDeviceToleranceSequence	1	DICOM_2011
(300A,004A)	DS	BeamLimitingDevicePositionTolerance	1	DICOM_2011
(300A,004B)	FL	SnoutPositionTolerance	1	DICOM_2011
(300A,004C)	DS	PatientSupportAngleTolerance	1	DICOM_2011
(300A,004E)	DS	TableTopEccentricAngleTolerance	1	DICOM_2011
(300A,004F)	FL	TableTopPitchAngleTolerance	1	DICOM_2011
(300A,0050)	FL	TableTopRollAngleTolerance	1	DICOM_2011
(300A,0051)	DS	TableTopVerticalPositionTolerance	1	DICOM_2011
(300A,0052)	DS	TableTopLongitudinalPositionTolerance	1	DICOM_2011
(300A,0053)	DS	TableTopLateralPositionTolerance	1	DICOM_2011
(300A,0055)	CS	RTPlanRelationship	1	DICOM_2011
(300A,0070)	SQ	FractionGroupSequence	1	DICOM_2011
(300A,0071)	IS	FractionGroupNumber	1	DICOM_2011
(300A,0072)	LO	FractionGroupDescription	1	DICOM_2011
(300A,0078)	IS	NumberOfFractionsPlanned	1	DICOM_2011
(300A,0079)	IS	NumberOfFractionPatternDigitsPerDay	1	DICOM_2011
(300A,007A)	IS	RepeatFractionCycleLength	1	DICOM_2011
(300A,007B)	LT	FractionPattern	1	DICOM_2011
(300A,0080)	IS	NumberOfBeams	1	DICOM_2011
(300A,0082)	DS	BeamDoseSpecificationPoint	3	DICOM_2011
(300A,0084)	DS	BeamDose	1	DICOM_2011
(300A,0086)	DS	BeamMeterset	1	DICOM_2011
(300A,0088)	FL	BeamDosePointDepth	1	DICOM_2011
(300A,0089)	FL	BeamDosePointEquivalentDepth	1	DICOM_2011
(300A,008A)	FL	BeamDosePointSSD	1	DICOM_2011
(300A,00A0)	IS	NumberOfBrachyApplicationSetups	1	DICOM_2011
(300A,00A2)	DS	BrachyApplicationSetupDoseSpecificationPoint	3	DICOM_2011
(300A,00A4)	DS	BrachyApplicationSetupDose	1	DICOM_2011
(300A,00B0)	SQ	BeamSequence	1	DICOM_2011
(300A,00B2)	SH	TreatmentMachineName	1	DICOM_2011
(300A,00B3)	CS	PrimaryDosimeterUnit	1	DICOM_2011
(300A,00B4)	DS	SourceAxisDistance	1	DICOM_2011
(300A,00B6)	SQ	BeamLimitingDeviceSequence	1	DICOM_2011
(300A,00B8)	CS	RTBeamLimitingDeviceType	1	DICOM_2011
(300A,00BA)	DS	SourceToBeamLimitingDeviceDistance	1	DICOM_2011
(300A,00BB)	FL	IsocenterToBeamLimitingDeviceDistance	1	DICOM_2011
(300A,00BC)	IS	NumberOfLeafJawPairs	1	DICOM_2011
(300A,00BE)	DS	LeafPositionBoundaries	3-n	DICOM_2011
(300A,00C0)	IS	BeamNumber	1	DICOM_2011
(300A,00C2)	LO	BeamName	1	DICOM_2011
(300A,00C3)	ST	BeamDescription	1	DICOM_2011
(300A,00C4)	CS	BeamType	1	DICOM_2011
(300A,00C6)	CS	RadiationType	1	DICOM_2011
(300A,00C7)	CS	HighDoseTechniqueType	1	DICOM_2011
(300A,00C8)	IS	ReferenceImageNumber	1	DICOM_2011
(300A,00CA)	SQ	PlannedVerificationImageSequence	1	DICOM_2011
(300A,00CC)	LO	ImagingDeviceSpecificAcquisitionParameters	1-n	DICOM_2011
(300A,00CE)	CS	TreatmentDeliveryType	1	DICOM_2011
(300A,00D0)	IS	NumberOfWedges	1	DICOM_2011
(300A,00D1)	SQ	WedgeSequence	1	DICOM_2011
(300A,00D2)	IS	WedgeNumber	1	DICOM_2011
(300A,00D3)	CS	WedgeType	1	DICOM_2011
(300A,00D4)	SH	WedgeID	1	DICOM_2011
(300A,00D5)	IS	WedgeAngle	1	DICOM_2011
(300A,00D6)	DS	WedgeFactor	1	DICOM_2011
(300A,00D7)	FL	TotalWedgeTrayWaterEquivalentThickness	1	DICOM_2011
(300A,00D8)	DS	WedgeOrientation	1	DICOM_2011
(300A,00D9)	FL	IsocenterToWedgeTrayDistance	1	DICOM_2011
(300A,00DA)	DS	SourceToWedgeTrayDistance	1	DICOM_2011
(300A,00DB)	FL	WedgeThinEdgePosition	1	DICOM_2011
(300A,00DC)	SH	BolusID	1	DICOM_2011
(300A,00DD)	ST	BolusDescription	1	DICOM_2011
(300A,00E0)	IS	NumberOfCompensators	1	DICOM_2011
(300A,00E1)	SH	MaterialID	1	DICOM_2011
(300A,00E2)	DS	TotalCompensatorTrayFactor	1	DICOM_2011
(300A,00E3)	SQ	CompensatorSequence	1	DICOM_2011
(300A,00E4)	IS	CompensatorNumber	1	DICOM_2011
(300A,00E5)	SH	CompensatorID	1	DICOM_2011
(300A,00E6)	DS	SourceToCompensatorTrayDistance	1	DICOM_2011
(300A,00E7)	IS	CompensatorRows	1	DICOM_2011
(300A,00E8)	IS	CompensatorColumns	1	DICOM_2011
(300A,00E9)	DS	CompensatorPixelSpacing	2	DICOM_2011
(300A,00EA)	DS	CompensatorPosition	2	DICOM_2011
(300A,00EB)	DS	CompensatorTransmissionData	1-n	DICOM_2011
(300A,00EC)	DS	CompensatorThicknessData	1-n	DICOM_2011
(300A,00ED)	IS	NumberOfBoli	1	DICOM_2011
(300A,00EE)	CS	CompensatorType	1	DICOM_2011
(300A,00F0)	IS	NumberOfBlocks	1	DICOM_2011
(300A,00F2)	DS	TotalBlockTrayFactor	1	DICOM_2011
(300A,00F3)	FL	TotalBlockTrayWaterEquivalentThickness	1	DICOM_2011
(300A,00F4)	SQ	BlockSequence	1	DICOM_2011
(300A,00F5)	SH	BlockTrayID	1	DICOM_2011
(300A,00F6)	DS	SourceToBlockTrayDistance	1	DICOM_2011
(300A,00F7)	FL	IsocenterToBlockTrayDistance	1	DICOM_2011
(300A,00F8)	CS	BlockType	1	DICOM_2011
(300A,00F9)	LO	AccessoryCode	1	DICOM_2011
(300A,00FA)	CS	BlockDivergence	1	DICOM_2011
(300A,00FB)	CS	BlockMountingPosition	1	DICOM_2011
(300A,00FC)	IS	BlockNumber	1	DICOM_2011
(300A,00FE)	LO	BlockName	1	DICOM_2011
(300A,0100)	DS	BlockThickness	1	DICOM_2011
(300A,0102)	DS	BlockTransmission	1	DICOM_2011
(300A,0104)	IS	BlockNumberOfPoints	1	DICOM_2011
(300A,0106)	DS	BlockData	2-2n	DICOM_2011
(300A,0107)	SQ	ApplicatorSequence	1	DICOM_2011
(300A,0108)	SH	ApplicatorID	1	DICOM_2011
(300A,0109)	CS	ApplicatorType	1	DICOM_2011
(300A,010A)	LO	ApplicatorDescription	1	DICOM_2011
(300A,010C)	DS	CumulativeDoseReferenceCoefficient	1	DICOM_2011
(300A,010E)	DS	FinalCumulativeMetersetWeight	1	DICOM_2011
(300A,0110)	IS	NumberOfControlPoints	1	DICOM_2011
(300A,0111)	SQ	ControlPointSequence	1	DICOM_2011
(300A,0112)	IS	ControlPointIndex	1	DICOM_2011
(300A,0114)	DS	NominalBeamEnergy	1	DICOM_2011
(300A,0115)	DS	DoseRateSet	1	DICOM_2011
(300A,0116)	SQ	WedgePositionSequence	1	DICOM_2011
(300A,0118)	CS	WedgePosition	1	DICOM_2011
(300A,011A)	SQ	BeamLimitingDevicePositionSequence	1	DICOM_2011
(300A,011C)	DS	LeafJawPositions	2-2n	DICOM_2011
(300A,011E)	DS	GantryAngle	1	DICOM_2011
(300A,011F)	CS	GantryRotationDirection	1	DICOM_2011
(300A,0120)	DS	BeamLimitingDeviceAngle	1	DICOM_2011
(300A,0121)	CS	BeamLimitingDeviceRotationDirection	1	DICOM_2011
(300A,0122)	DS	PatientSupportAngle	1	DICOM_2011
(300A,0123)	CS	PatientSupportRotationDirection	1	DICOM_2011
(300A,0124)	DS	TableTopEccentricAxisDistance	1	DICOM_2011
(300A,0125)	DS	TableTopEccentricAngle	1	DICOM_2011
(300A,0126)	CS	TableTopEccentricRotationDirection	1	DICOM_2011
(300A,0128)	DS	TableTopVerticalPosition	1	DICOM_2011
(300A,0129)	DS	TableTopLongitudinalPosition	1	DICOM_2011
(300A,012A)	DS	TableTopLateralPosition	1	DICOM_2011
(300A,012C)	DS	IsocenterPosition	3	DICOM_2011
(300A,012E)	DS	SurfaceEntryPoint	3	DICOM_2011
(300A,0130)	DS	SourceToSurfaceDistance	1	DICOM_2011
(300A,0134)	DS	CumulativeMetersetWeight	1	DICOM_2011
(300A,0140)	FL	TableTopPitchAngle	1	DICOM_2011
(300A,0142)	CS	TableTopPitchRotationDirection	1	DICOM_2011
(300A,0144)	FL	TableTopRollAngle	1	DICOM_2011
(300A,0146)	CS	TableTopRollRotationDirection	1	DICOM_2011
(300A,0148)	FL	HeadFixationAngle	1	DICOM_2011
(300A,014A)	FL	GantryPitchAngle	1	DICOM_2011
(300A,014C)	CS	GantryPitchRotationDirection	1	DICOM_2011
(300A,014E)	FL	GantryPitchAngleTolerance	1	DICOM_2011
(300A,0180)	SQ	PatientSetupSequence	1	DICOM_2011
(300A,0182)	IS	PatientSetupNumber	1	DICOM_2011
(300A,0183)	LO	PatientSetupLabel	1	DICOM_2011
(300A,0184)	LO	PatientAdditionalPosition	1	DICOM_2011
(300A,0190)	SQ	FixationDeviceSequence	1	DICOM_2011
(300A,0192)	CS	FixationDeviceType	1	DICOM_2011
(300A,0194)	SH	FixationDeviceLabel	1	DICOM_2011
(300A,0196)	ST	FixationDeviceDescription	1	DICOM_2011
(300A,0198)	SH	FixationDevicePosition	1	DICOM_2011
(300A,0199)	FL	FixationDevicePitchAngle	1	DICOM_2011
(300A,019A)	FL	FixationDeviceRollAngle	1	DICOM_2011
(300A,01A0)	SQ	ShieldingDeviceSequence	1	DICOM_2011
(300A,01A2)	CS	ShieldingDeviceType	1	DICOM_2011
(300A,01A4)	SH	ShieldingDeviceLabel	1	DICOM_2011
(300A,01A6)	ST	ShieldingDeviceDescription	1	DICOM_2011
(300A,01A8)	SH	ShieldingDevicePosition	1	DICOM_2011
(300A,01B0)	CS	SetupTechnique	1	DICOM_2011
(300A,01B2)	ST	SetupTechniqueDescription	1	DICOM_2011
(300A,01B4)	SQ	SetupDeviceSequence	1	DICOM_2011
(300A,01B6)	CS	SetupDeviceType	1	DICOM_2011
(300A,01B8)	SH	SetupDeviceLabel	1	DICOM_2011
(300A,01BA)	ST	SetupDeviceDescription	1	DICOM_2011
(300A,01BC)	DS	SetupDeviceParameter	1	DICOM_2011
(300A,01D0)	ST	SetupReferenceDescription	1	DICOM_2011
(300A,01D2)	DS	TableTopVerticalSetupDisplacement	1	DICOM_2011
(300A,01D4)	DS	TableTopLongitudinalSetupDisplacement	1	DICOM_2011
(300A,01D6)	DS	TableTopLateralSetupDisplacement	1	DICOM_2011
(300A,0200)	CS	BrachyTreatmentTechnique	1	DICOM_2011
(300A,0202)	CS	BrachyTreatmentType	1	DICOM_2011
(300A,0206)	SQ	TreatmentMachineSequence	1	DICOM_2011
(300A,0210)	SQ	SourceSequence	1	DICOM_2011
(300A,0212)	IS	SourceNumber	1	DICOM_2011
(300A,0214)	CS	SourceType	1	DICOM_2011
(300A,0216)	LO	SourceManufacturer	1	DICOM_2011
(300A,0218)	DS	ActiveSourceDiameter	1	DICOM_2011
(300A,021A)	DS	ActiveSourceLength	1	DICOM_2011
(300A,0222)	DS	SourceEncapsulationNominalThickness	1	DICOM_2011
(300A,0224)	DS	SourceEncapsulationNominalTransmission	1	DICOM_2011
(300A,0226)	LO	SourceIsotopeName	1	DICOM_2011
(300A,0228)	DS	SourceIsotopeHalfLife	1	DICOM_2011
(300A,0229)	CS	SourceStrengthUnits	1	DICOM_2011
(300A,022A)	DS	ReferenceAirKermaRate	1	DICOM_2011
(300A,022B)	DS	SourceStrength	1	DICOM_2011
(300A,022C)	DA	SourceStrengthReferenceDate	1	DICOM_2011
(300A,022E)	TM	SourceStrengthReferenceTime	1	DICOM_2011
(300A,0230)	SQ	ApplicationSetupSequence	1	DICOM_2011
(300A,0232)	CS	ApplicationSetupType	1	DICOM_2011
(300A,0234)	IS	ApplicationSetupNumber	1	DICOM_2011
(300A,0236)	LO	ApplicationSetupName	1	DICOM_2011
(300A,0238)	LO	ApplicationSetupManufacturer	1	DICOM_2011
(300A,0240)	IS	TemplateNumber	1	DICOM_2011
(300A,0242)	SH	TemplateType	1	DICOM_2011
(300A,0244)	LO	TemplateName	1	DICOM_2011
(300A,0250)	DS	TotalReferenceAirKerma	1	DICOM_2011
(300A,0260)	SQ	BrachyAccessoryDeviceSequence	1	DICOM_2011
(300A,0262)	IS	BrachyAccessoryDeviceNumber	1	DICOM_2011
(300A,0263)	SH	BrachyAccessoryDeviceID	1	DICOM_2011
(300A,0264)	CS	BrachyAccessoryDeviceType	1	DICOM_2011
(300A,0266)	LO	BrachyAccessoryDeviceName	1	DICOM_2011
(300A,026A)	DS	BrachyAccessoryDeviceNominalThickness	1	DICOM_2011
(300A,026C)	DS	BrachyAccessoryDeviceNominalTransmission	1	DICOM_2011
(300A,0280)	SQ	ChannelSequence	1	DICOM_2011
(300A,0282)	IS	ChannelNumber	1	DICOM_2011
(300A,0284)	DS	ChannelLength	1	DICOM_2011
(300A,0286)	DS	ChannelTotalTime	1	DICOM_2011
(300A,0288)	CS	SourceMovementType	1	DICOM_2011
(300A,028A)	IS	NumberOfPulses	1	DICOM_2011
(300A,028C)	DS	PulseRepetitionInterval	1	DICOM_2011
(300A,0290)	IS	SourceApplicatorNumber	1	DICOM_2011
(300A,0291)	SH	SourceApplicatorID	1	DICOM_2011
(300A,0292)	CS	SourceApplicatorType	1	DICOM_2011
(300A,0294)	LO	SourceApplicatorName	1	DICOM_2011
(300A,0296)	DS	SourceApplicatorLength	1	DICOM_2011
(300A,0298)	LO	SourceApplicatorManufacturer	1	DICOM_2011
(300A,029C)	DS	SourceApplicatorWallNominalThickness	1	DICOM_2011
(300A,029E)	DS	SourceApplicatorWallNominalTransmission	1	DICOM_2011
(300A,02A0)	DS	SourceApplicatorStepSize	1	DICOM_2011
(300A,02A2)	IS	TransferTubeNumber	1	DICOM_2011
(300A,02A4)	DS	TransferTubeLength	1	DICOM_2011
(300A,02B0)	SQ	ChannelShieldSequence	1	DICOM_2011
(300A,02B2)	IS	ChannelShieldNumber	1	DICOM_2011
(300A,02B3)	SH	ChannelShieldID	1	DICOM_2011
(300A,02B4)	LO	ChannelShieldName	1	DICOM_2011
(300A,02B8)	DS	ChannelShieldNominalThickness	1	DICOM_2011
(300A,02BA)	DS	ChannelShieldNominalTransmission	1	DICOM_2011
(300A,02C8)	DS	FinalCumulativeTimeWeight	1	DICOM_2011
(300A,02D0)	SQ	BrachyControlPointSequence	1	DICOM_2011
(300A,02D2)	DS	ControlPointRelativePosition	1	DICOM_2011
(300A,02D4)	DS	ControlPoint3DPosition	3	DICOM_2011
(300A,02D6)	DS	CumulativeTimeWeight	1	DICOM_2011
(300A,02E0)	CS	CompensatorDivergence	1	DICOM_2011
(300A,02E1)	CS	CompensatorMountingPosition	1	DICOM_2011
(300A,02E2)	DS	SourceToCompensatorDistance	1-n	DICOM_2011
(300A,02E3)	FL	TotalCompensatorTrayWaterEquivalentThickness	1	DICOM_2011
(300A,02E4)	FL	IsocenterToCompensatorTrayDistance	1	DICOM_2011
(300A,02E5)	FL	CompensatorColumnOffset	1	DICOM_2011
(300A,02E6)	FL	IsocenterToCompensatorDistances	1-n	DICOM_2011
(300A,02E7)	FL	CompensatorRelativeStoppingPowerRatio	1	DICOM_2011
(300A,02E8)	FL	CompensatorMillingToolDiameter	1	DICOM_2011
(300A,02EA)	SQ	IonRangeCompensatorSequence	1	DICOM_2011
(300A,02EB)	LT	CompensatorDescription	1	DICOM_2011
(300A,0302)	IS	RadiationMassNumber	1	DICOM_2011
(300A,0304)	IS	RadiationAtomicNumber	1	DICOM_2011
(300A,0306)	SS	RadiationChargeState	1	DICOM_2011
(300A,0308)	CS	ScanMode	1	DICOM_2011
(300A,030A)	FL	VirtualSourceAxisDistances	2	DICOM_2011
(300A,030C)	SQ	SnoutSequence	1	DICOM_2011
(300A,030D)	FL	SnoutPosition	1	DICOM_2011
(300A,030F)	SH	SnoutID	1	DICOM_2011
(300A,0312)	IS	NumberOfRangeShifters	1	DICOM_2011
(300A,0314)	SQ	RangeShifterSequence	1	DICOM_2011
(300A,0316)	IS	RangeShifterNumber	1	DICOM_2011
(300A,0318)	SH	RangeShifterID	1	DICOM_2011
(300A,0320)	CS	RangeShifterType	1	DICOM_2011
(300A,0322)	LO	RangeShifterDescription	1	DICOM_2011
(300A,0330)	IS	NumberOfLateralSpreadingDevices	1	DICOM_2011
(300A,0332)	SQ	LateralSpreadingDeviceSequence	1	DICOM_2011
(300A,0334)	IS	LateralSpreadingDeviceNumber	1	DICOM_2011
(300A,0336)	SH	LateralSpreadingDeviceID	1	DICOM_2011
(300A,0338)	CS	LateralSpreadingDeviceType	1	DICOM_2011
(300A,033A)	LO	LateralSpreadingDeviceDescription	1	DICOM_2011
(300A,033C)	FL	LateralSpreadingDeviceWaterEquivalentThickness	1	DICOM_2011
(300A,0340)	IS	NumberOfRangeModulators	1	DICOM_2011
(300A,0342)	SQ	RangeModulatorSequence	1	DICOM_2011
(300A,0344)	IS	RangeModulatorNumber	1	DICOM_2011
(300A,0346)	SH	RangeModulatorID	1	DICOM_2011
(300A,0348)	CS	RangeModulatorType	1	DICOM_2011
(300A,034A)	LO	RangeModulatorDescription	1	DICOM_2011
(300A,034C)	SH	BeamCurrentModulationID	1	DICOM_2011
(300A,0350)	CS	PatientSupportType	1	DICOM_2011
(300A,0352)	SH	PatientSupportID	1	DICOM_2011
(300A,0354)	LO	PatientSupportAccessoryCode	1	DICOM_2011
(300A,0356)	FL	FixationLightAzimuthalAngle	1	DICOM_2011
(300A,0358)	FL	FixationLightPolarAngle	1	DICOM_2011
(300A,035A)	FL	MetersetRate	1	DICOM_2011
(300A,0360)	SQ	RangeShifterSettingsSequence	1	DICOM_2011
(300A,0362)	LO	RangeShifterSetting	1	DICOM_2011
(300A,0364)	FL	IsocenterToRangeShifterDistance	1	DICOM_2011
(300A,0366)	FL	RangeShifterWaterEquivalentThickness	1	DICOM_2011
(300A,0370)	SQ	LateralSpreadingDeviceSettingsSequence	1	DICOM_2011
(300A,0372)	LO	LateralSpreadingDeviceSetting	1	DICOM_2011
(300A,0374)	FL	IsocenterToLateralSpreadingDeviceDistance	1	DICOM_2011
(300A,0380)	SQ	RangeModulatorSettingsSequence	1	DICOM_2011
(300A,0382)	FL	RangeModulatorGatingStartValue	1	DICOM_2011
(300A,0384)	FL	RangeModulatorGatingStopValue	1	DICOM_2011
(300A,0386)	FL	RangeModulatorGatingStartWaterEquivalentThickness	1	DICOM_2011
(300A,0388)	FL	RangeModulatorGatingStopWaterEquivalentThickness	1	DICOM_2011
(300A,038A)	FL	IsocenterToRangeModulatorDistance	1	DICOM_2011
(300A,0390)	SH	ScanSpotTuneID	1	DICOM_2011
(300A,0392)	IS	NumberOfScanSpotPositions	1	DICOM_2011
(300A,0394)	FL	ScanSpotPositionMap	1-n	DICOM_2011
(300A,0396)	FL	ScanSpotMetersetWeights	1-n	DICOM_2011
(300A,0398)	FL	ScanningSpotSize	2	DICOM_2011
(300A,039A)	IS	NumberOfPaintings	1	DICOM_2011
(300A,03A0)	SQ	IonToleranceTableSequence	1	DICOM_2011
(300A,03A2)	SQ	IonBeamSequence	1	DICOM_2011
(300A,03A4)	SQ	IonBeamLimitingDeviceSequence	1	DICOM_2011
(300A,03A6)	SQ	IonBlockSequence	1	DICOM_2011
(300A,03A8)	SQ	IonControlPointSequence	1	DICOM_2011
(300A,03AA)	SQ	IonWedgeSequence	1	DICOM_2011
(300A,03AC)	SQ	IonWedgePositionSequence	1	DICOM_2011
(300A,0401)	SQ	ReferencedSetupImageSequence	1	DICOM_2011
(300A,0402)	ST	SetupImageComment	1	DICOM_2011
(300A,0410)	SQ	MotionSynchronizationSequence	1	DICOM_2011
(300A,0412)	FL	ControlPointOrientation	3	DICOM_2011
(300A,0420)	SQ	GeneralAccessorySequence	1	DICOM_2011
(300A,0421)	SH	GeneralAccessoryID	1	DICOM_2011
(300A,0422)	ST	GeneralAccessoryDescription	1	DICOM_2011
(300A,0423)	CS	GeneralAccessoryType	1	DICOM_2011
(300A,0424)	IS	GeneralAccessoryNumber	1	DICOM_2011
(300A,0431)	SQ	ApplicatorGeometrySequence	1	DICOM_2011
(300A,0432)	CS	ApplicatorApertureShape	1	DICOM_2011
(300A,0433)	FL	ApplicatorOpening	1	DICOM_2011
(300A,0434)	FL	ApplicatorOpeningX	1	DICOM_2011
(300A,0435)	FL	ApplicatorOpeningY	1	DICOM_2011
(300A,0436)	FL	SourceToApplicatorMountingPositionDistance	1	DICOM_2011
(300C,0002)	SQ	ReferencedRTPlanSequence	1	DICOM_2011
(300C,0004)	SQ	ReferencedBeamSequence	1	DICOM_2011
(300C,0006)	IS	ReferencedBeamNumber	1	DICOM_2011
(300C,0007)	IS	ReferencedReferenceImageNumber	1	DICOM_2011
(300C,0008)	DS	StartCumulativeMetersetWeight	1	DICOM_2011
(300C,0009)	DS	EndCumulativeMetersetWeight	1	DICOM_2011
(300C,000A)	SQ	ReferencedBrachyApplicationSetupSequence	1	DICOM_2011
(300C,000C)	IS	ReferencedBrachyApplicationSetupNumber	1	DICOM_2011
(300C,000E)	IS	ReferencedSourceNumber	1	DICOM_2011
(300C,0020)	SQ	ReferencedFractionGroupSequence	1	DICOM_2011
(300C,0022)	IS	ReferencedFractionGroupNumber	1	DICOM_2011
(300C,0040)	SQ	ReferencedVerificationImageSequence	1	DICOM_2011
(300C,0042)	SQ	ReferencedReferenceImageSequence	1	DICOM_2011
(300C,0050)	SQ	ReferencedDoseReferenceSequence	1	DICOM_2011
(300C,0051)	IS	ReferencedDoseReferenceNumber	1	DICOM_2011
(300C,0055)	SQ	BrachyReferencedDoseReferenceSequence	1	DICOM_2011
(300C,0060)	SQ	ReferencedStructureSetSequence	1	DICOM_2011
(300C,006A)	IS	ReferencedPatientSetupNumber	1	DICOM_2011
(300C,0080)	SQ	ReferencedDoseSequence	1	DICOM_2011
(300C,00A0)	IS	ReferencedToleranceTableNumber	1	DICOM_2011
(300C,00B0)	SQ	ReferencedBolusSequence	1	DICOM_2011
(300C,00C0)	IS	ReferencedWedgeNumber	1	DICOM_2011
(300C,00D0)	IS	ReferencedCompensatorNumber	1	DICOM_2011
(300C,00E0)	IS	ReferencedBlockNumber	1	DICOM_2011
(300C,00F0)	IS	ReferencedControlPointIndex	1	DICOM_2011
(300C,00F2)	SQ	ReferencedControlPointSequence	1	DICOM_2011
(300C,00F4)	IS	ReferencedStartControlPointIndex	1	DICOM_2011
(300C,00F6)	IS	ReferencedStopControlPointIndex	1	DICOM_2011
(300C,0100)	IS	ReferencedRangeShifterNumber	1	DICOM_2011
(300C,0102)	IS	ReferencedLateralSpreadingDeviceNumber	1	DICOM_2011
(300C,0104)	IS	ReferencedRangeModulatorNumber	1	DICOM_2011
(300E,0002)	CS	ApprovalStatus	1	DICOM_2011
(300E,0004)	DA	ReviewDate	1	DICOM_2011
(300E,0005)	TM	ReviewTime	1	DICOM_2011
(300E,0008)	PN	ReviewerName	1	DICOM_2011
(4010,0001)	CS	LowEnergyDetectors	1	DICOM_2011
(4010,0002)	CS	HighEnergyDetectors	1	DICOM_2011
(4010,0004)	SQ	DetectorGeometrySequence	1	DICOM_2011
(4010,1001)	SQ	ThreatROIVoxelSequence	1	DICOM_2011
(4010,1004)	FL	ThreatROIBase	3	DICOM_2011
(4010,1005)	FL	ThreatROIExtents	3	DICOM_2011
(4010,1006)	OB	ThreatROIBitmap	1	DICOM_2011
(4010,1007)	SH	RouteSegmentID	1	DICOM_2011
(4010,1008)	CS	GantryType	1	DICOM_2011
(4010,1009)	CS	OOIOwnerType	1	DICOM_2011
(4010,100A)	SQ	RouteSegmentSequence	1	DICOM_2011
(4010,1010)	US	PotentialThreatObjectID	1	DICOM_2011
(4010,1011)	SQ	ThreatSequence	1	DICOM_2011
(4010,1012)	CS	ThreatCategory	1	DICOM_2011
(4010,1013)	LT	ThreatCategoryDescription	1	DICOM_2011
(4010,1014)	CS	ATDAbilityAssessment	1	DICOM_2011
(4010,1015)	CS	ATDAssessmentFlag	1	DICOM_2011
(4010,1016)	FL	ATDAssessmentProbability	1	DICOM_2011
(4010,1017)	FL	Mass	1	DICOM_2011
(4010,1018)	FL	Density	1	DICOM_2011
(4010,1019)	FL	ZEffective	1	DICOM_2011
(4010,101A)	SH	BoardingPassID	1	DICOM_2011
(4010,101B)	FL	CenterOfMass	3	DICOM_2011
(4010,101C)	FL	CenterOfPTO	3	DICOM_2011
(4010,101D)	FL	BoundingPolygon	6-n	DICOM_2011
(4010,101E)	SH	RouteSegmentStartLocationID	1	DICOM_2011
(4010,101F)	SH	RouteSegmentEndLocationID	1	DICOM_2011
(4010,1020)	CS	RouteSegmentLocationIDType	1	DICOM_2011
(4010,1021)	CS	AbortReason	1-n	DICOM_2011
(4010,1023)	FL	VolumeOfPTO	1	DICOM_2011
(4010,1024)	CS	AbortFlag	1	DICOM_2011
(4010,1025)	DT	RouteSegmentStartTime	1	DICOM_2011
(4010,1026)	DT	RouteSegmentEndTime	1	DICOM_2011
(4010,1027)	CS	TDRType	1	DICOM_2011
(4010,1028)	CS	InternationalRouteSegment	1	DICOM_2011
(4010,1029)	LO	ThreatDetectionAlgorithmandVersion	1-n	DICOM_2011
(4010,102A)	SH	AssignedLocation	1	DICOM_2011
(4010,102B)	DT	AlarmDecisionTime	1	DICOM_2011
(4010,1031)	CS	AlarmDecision	1	DICOM_2011
(4010,1033)	US	NumberOfTotalObjects	1	DICOM_2011
(4010,1034)	US	NumberOfAlarmObjects	1	DICOM_2011
(4010,1037)	SQ	PTORepresentationSequence	1	DICOM_2011
(4010,1038)	SQ	ATDAssessmentSequence	1	DICOM_2011
(4010,1039)	CS	TIPType	1	DICOM_2011
(4010,103A)	CS	DICOSVersion	1	DICOM_2011
(4010,1041)	DT	OOIOwnerCreationTime	1	DICOM_2011
(4010,1042)	CS	OOIType	1	DICOM_2011
(4010,1043)	FL	OOISize	3	DICOM_2011
(4010,1044)	CS	AcquisitionStatus	1	DICOM_2011
(4010,1045)	SQ	BasisMaterialsCodeSequence	1	DICOM_2011
(4010,1046)	CS	PhantomType	1	DICOM_2011
(4010,1047)	SQ	OOIOwnerSequence	1	DICOM_2011
(4010,1048)	CS	ScanType	1	DICOM_2011
(4010,1051)	LO	ItineraryID	1	DICOM_2011
(4010,1052)	SH	ItineraryIDType	1	DICOM_2011
(4010,1053)	LO	ItineraryIDAssigningAuthority	1	DICOM_2011
(4010,1054)	SH	RouteID	1	DICOM_2011
(4010,1055)	SH	RouteIDAssigningAuthority	1	DICOM_2011
(4010,1056)	CS	InboundArrivalType	1	DICOM_2011
(4010,1058)	SH	CarrierID	1	DICOM_2011
(4010,1059)	CS	CarrierIDAssigningAuthority	1	DICOM_2011
(4010,1060)	FL	SourceOrientation	3	DICOM_2011
(4010,1061)	FL	SourcePosition	3	DICOM_2011
(4010,1062)	FL	BeltHeight	1	DICOM_2011
(4010,1064)	SQ	AlgorithmRoutingCodeSequence	1	DICOM_2011
(4010,1067)	CS	TransportClassification	1	DICOM_2011
(4010,1068)	LT	OOITypeDescriptor	1	DICOM_2011
(4010,1069)	FL	TotalProcessingTime	1	DICOM_2011
(4010,106C)	OB	DetectorCalibrationData	1	DICOM_2011
(4FFE,0001)	SQ	MACParametersSequence	1	DICOM_2011
(5200,9229)	SQ	SharedFunctionalGroupsSequence	1	DICOM_2011
(5200,9230)	SQ	PerFrameFunctionalGroupsSequence	1	DICOM_2011
(5400,0100)	SQ	WaveformSequence	1	DICOM_2011
(5400,0110)	ox	ChannelMinimumValue	1	DICOM_2011
(5400,0112)	ox	ChannelMaximumValue	1	DICOM_2011
(5400,1004)	US	WaveformBitsAllocated	1	DICOM_2011
(5400,1006)	CS	WaveformSampleInterpretation	1	DICOM_2011
(5400,100A)	ox	WaveformPaddingValue	1	DICOM_2011
(5400,1010)	ox	WaveformData	1	DICOM_2011
(5600,0010)	OF	FirstOrderPhaseCorrectionAngle	1	DICOM_2011
(5600,0020)	OF	SpectroscopyData	1	DICOM_2011
(6000-60FF,0010)	US	OverlayRows	1	DICOM_2011
(6000-60FF,0011)	US	OverlayColumns	1	DICOM_2011
(6000-60FF,0015)	IS	NumberOfFramesInOverlay	1	DICOM_2011
(6000-60FF,0022)	LO	OverlayDescription	1	DICOM_2011
(6000-60FF,0040)	CS	OverlayType	1	DICOM_2011
(6000-60FF,0045)	LO	OverlaySubtype	1	DICOM_2011
(6000-60FF,0050)	SS	OverlayOrigin	2	DICOM_2011
(6000-60FF,0051)	US	ImageFrameOrigin	1	DICOM_2011
(6000-60FF,0100)	US	OverlayBitsAllocated	1	DICOM_2011
(6000-60FF,0102)	US	OverlayBitPosition	1	DICOM_2011
(6000-60FF,1001)	CS	OverlayActivationLayer	1	DICOM_2011
(6000-60FF,1301)	IS	ROIArea	1	DICOM_2011
(6000-60FF,1302)	DS	ROIMean	1	DICOM_2011
(6000-60FF,1303)	DS	ROIStandardDeviation	1	DICOM_2011
(6000-60FF,1500)	LO	OverlayLabel	1	DICOM_2011
(6000-60FF,3000)	ox	OverlayData	1	DICOM_2011
(7FE0,0010)	ox	PixelData	1	DICOM_2011
(FFFA,FFFA)	SQ	DigitalSignaturesSequence	1	DICOM_2011
(FFFC,FFFC)	OB	DataSetTrailingPadding	1	DICOM_2011
(FFFE,E000)	na	Item	1	DICOM_2011
(FFFE,E00D)	na	ItemDelimitationItem	1	DICOM_2011
(FFFE,E0DD)	na	SequenceDelimitationItem	1	DICOM_2011
#
#---------------------------------------------------------------------------
#
# Supplement 152 (Ophthalmic Thickness Map Storage SOP Class)
#
(0022,1415)	CS	OphthalmicMappingDeviceType	1	Supplement_152
(0022,1420)	SQ	AcquisitionTypeMethodSequence	1	Supplement_152
(0022,1423)	SQ	AcquisitonMethodAlgorithmSequence	1	Supplement_152
(0022,1436)	SQ	OphthalmicThicknessMapTypeCodeSequence	1	Supplement_152
(0022,1443)	SQ	OphthalmicThicknessMappingNormalsSequence	1	Supplement_152
(0022,1445)	SQ	RetinalThicknessDefinitionCodeSequence	1	Supplement_152
(0022,1450)	SQ	PixelValueMappingtoCodedConceptSequence	1	Supplement_152
(0022,1452)	xs	MappedPixelValue	1	Supplement_152
(0022,1454)	LO	PixelValueMappingExplanation	1	Supplement_152
(0022,1458)	SQ	OphthalmicThicknessMapQualityThresholdSequence	1	Supplement_152
(0022,1460)	FL	OphthalmicThicknessMapThresholdQualityRating	1	Supplement_152
(0022,1463)	FL	AnatomicStructureReferencePoint	2	Supplement_152
(0022,1465)	SQ	RegistrationtoLocalizerSequence	1	Supplement_152
(0022,1466)	CS	RegisteredLocalizerUnits	1	Supplement_152
(0022,1467)	FL	RegisteredTopLeftHandCorner	2	Supplement_152
(0022,1468)	FL	RegisteredBottomRightHandCorner	2	Supplement_152
(0022,1470)	SQ	OphthalmicThicknessMapQualityRatingSequence	1	Supplement_152
(0022,1472)	SQ	RelevantOPTAttributesSequence	1	Supplement_152
#
#---------------------------------------------------------------------------
#
# Supplement 158 (Retirement of General Purpose Worklist and Procedure Step)
#
(0040,4001)	CS	RETIRED_GeneralPurposeScheduledProcedureStepStatus	1	Supplement_158/retired
(0040,4002)	CS	RETIRED_GeneralPurposePerformedProcedureStepStatus	1	Supplement_158/retired
(0040,4003)	CS	RETIRED_GeneralPurposeScheduledProcedureStepPriority	1	Supplement_158/retired
(0040,4004)	SQ	RETIRED_ScheduledProcessingApplicationsCodeSequence	1	Supplement_158/retired
(0040,4006)	CS	RETIRED_MultipleCopiesFlag	1	Supplement_158/retired
(0040,4015)	SQ	RETIRED_ResultingGeneralPurposePerformedProcedureStepsSequence	1	Supplement_158/retired
(0040,4016)	SQ	RETIRED_ReferencedGeneralPurposeScheduledProcedureStepSequence	1	Supplement_158/retired
(0040,4022)	SQ	RETIRED_RelevantInformationSequence	1	Supplement_158/retired
(0040,4023)	UI	RETIRED_ReferencedGeneralPurposeScheduledProcedureStepTransactionUID	1	Supplement_158/retired
(0040,4031)	SQ	RETIRED_RequestedSubsequentWorkitemCodeSequence	1	Supplement_158/retired
(0040,4032)	SQ	RETIRED_NonDICOMOutputCodeSequence	1	Supplement_158/retired
#
#---------------------------------------------------------------------------
#
# Correction Items 1064, 1123, 1137, 1138, 1147, 1188, 1200, 1204, 1216
#
(0040,A161)	FD	FloatingPointValue	1-n	CP_1064
(0040,A162)	SL	RationalNumeratorValue	1-n	CP_1064
(0040,A163)	UL	RationalDenominatorValue	1-n	CP_1064
#
(0010,0200)	CS	QualityControlSubject	1	CP_1123
#
(3006,0018)	SQ	PredecessorStructureSetSequence	1	CP_1137
#
(300A,0088)	FL	RETIRED_BeamDosePointDepth	1	CP_1138/retired
(300A,0089)	FL	RETIRED_BeamDosePointEquivalentDepth	1	CP_1138/retired
(300A,008A)	FL	RETIRED_BeamDosePointSSD	1	CP_1138/retired
(300A,008C)	SQ	BeamDoseVerificationControlPointSequence	1	CP_1138
(300A,008D)	FL	AverageBeamDosePointDepth	1	CP_1138
(300A,008E)	FL	AverageBeamDosePointEquivalentDepth	1	CP_1138
(300A,008F)	FL	AverageBeamDosePointSSD	1	CP_1138
#
(0040,A171)	UI	ObservationUID	1	CP_1147
#
(300A,00EF)	SH	CompensatorTrayID	1	CP_1188
#
(0066,0037)	FL	RecommendedPointRadius	1	CP_1200
(0066,0038)	FL	RecommendedLineThickness	1	CP_1200
#
(300A,021B)	SH	SourceModel	1	CP_1204
(300A,021C)	LO	SourceDescription	1	CP_1204
#
(0008,0015)	DT	InstanceCoercionDateTime	1	CP_1216
#
#---------------------------------------------------------------------------
#
# Private Creator Data Elements
#
(0009-o-ffff,0000)	UL	PrivateGroupLength	1	PRIVATE
(0009-o-ffff,0010-u-00ff)	LO	PrivateCreator	1	PRIVATE
(0001-o-0007,0000)	UL	IllegalGroupLength	1	ILLEGAL
(0001-o-0007,0010-u-00ff)	LO	IllegalPrivateCreator	1	ILLEGAL
#
#---------------------------------------------------------------------------
#
# A "catch all" for group length elements
#
(0000-u-ffff,0000)	UL	GenericGroupLength	1	GENERIC
#
#---------------------------------------------------------------------------
#
# Retired data elements from ACR/NEMA 2 (1988)
#
(0000,0001)	UL	ACR_NEMA_CommandGroupLengthToEnd	1	ACR/NEMA2
(0000,0010)	CS	ACR_NEMA_CommandRecognitionCode	1	ACR/NEMA2
(0000,0200)	LO	ACR_NEMA_Initiator	1	ACR/NEMA2
(0000,0300)	LO	ACR_NEMA_Receiver	1	ACR/NEMA2
(0000,0400)	LO	ACR_NEMA_FindLocation	1	ACR/NEMA2
(0000,0850)	US	ACR_NEMA_NumberOfMatches	1	ACR/NEMA2
(0000,0860)	US	ACR_NEMA_ResponseSequenceNumber	1	ACR/NEMA2
(0000,4000)	LO	ACR_NEMA_DialogReceiver	1	ACR/NEMA2
(0000,4010)	LO	ACR_NEMA_TerminalType	1	ACR/NEMA2
(0000,5010)	LO	ACR_NEMA_MessageSetID	1	ACR/NEMA2
(0000,5020)	LO	ACR_NEMA_EndMessageSet	1	ACR/NEMA2
(0000,5110)	LO	ACR_NEMA_DisplayFormat	1	ACR/NEMA2
(0000,5120)	LO	ACR_NEMA_PagePositionID	1	ACR/NEMA2
(0000,5130)	LO	ACR_NEMA_TextFormatID	1	ACR/NEMA2
(0000,5140)	CS	ACR_NEMA_NormalReverse	1	ACR/NEMA2
(0000,5150)	CS	ACR_NEMA_AddGrayScale	1	ACR/NEMA2
(0000,5160)	CS	ACR_NEMA_Borders	1	ACR/NEMA2
(0000,5170)	IS	ACR_NEMA_Copies	1	ACR/NEMA2
(0000,5180)	LO	ACR_NEMA_MagnificationType	1	ACR/NEMA2
(0000,5190)	LO	ACR_NEMA_Erase	1-n	ACR/NEMA2
(0000,51A0)	CS	ACR_NEMA_Print	1	ACR/NEMA2
(0000,51B0)	US	ACR_NEMA_Overlays	1-n	ACR/NEMA2
(0008,0001)	UL	ACR_NEMA_IdentifyingGroupLengthToEnd	1	ACR/NEMA2
(0008,0010)	LO	ACR_NEMA_RecognitionCode	1	ACR/NEMA2
(0008,0040)	US	ACR_NEMA_OldDataSetType	1	ACR/NEMA2
(0008,0041)	LO	ACR_NEMA_DataSetSubtype	1	ACR/NEMA2
(0008,1000)	LO	ACR_NEMA_NetworkID	1	ACR/NEMA2
(0008,4000)	LT	ACR_NEMA_IdentifyingComments	1-n	ACR/NEMA2
(0010,1050)	LT	ACR_NEMA_InsurancePlanIdentification	1-n	ACR/NEMA2
(0018,1240)	IS	ACR_NEMA_UpperLowerPixelValues	1-n	ACR/NEMA2
(0018,4000)	LT	ACR_NEMA_AcquisitionComments	1-n	ACR/NEMA2
(0018,5030)	DS	ACR_NEMA_DynamicRange	1	ACR/NEMA2
(0018,5040)	DS	ACR_NEMA_TotalGain	1	ACR/NEMA2
(0020,0030)	DS	ACR_NEMA_ImagePosition	3	ACR/NEMA2
(0020,0035)	DS	ACR_NEMA_ImageOrientation	6	ACR/NEMA2
(0020,0050)	DS	ACR_NEMA_Location	1	ACR/NEMA2
(0020,0070)	LO	ACR_NEMA_ImageGeometryType	1	ACR/NEMA2
(0020,0080)	LO	ACR_NEMA_MaskingImage	1-n	ACR/NEMA2
(0020,1001)	IS	ACR_NEMA_AcquisitionsInSeries	1	ACR/NEMA2
(0020,1003)	IS	ACR_NEMA_ImagesInSeries	1	ACR/NEMA2
(0020,1005)	IS	ACR_NEMA_ImagesInStudy	1	ACR/NEMA2
(0020,1020)	LO	ACR_NEMA_Reference	1-n	ACR/NEMA2
(0020,3100-31FF)	LO	ACR_NEMA_SourceImageID	1-n	ACR/NEMA2
(0020,3401)	LO	ACR_NEMA_ModifyingDeviceID	1	ACR/NEMA2
(0020,3402)	LO	ACR_NEMA_ModifiedImageID	1	ACR/NEMA2
(0020,3403)	DA	ACR_NEMA_ModifiedImageDate	1	ACR/NEMA2
(0020,3404)	LO	ACR_NEMA_ModifyingDeviceManufacturer	1	ACR/NEMA2
(0020,3405)	TM	ACR_NEMA_ModifiedImageTime	1	ACR/NEMA2
(0020,3406)	LO	ACR_NEMA_ModifiedImageDescription	1	ACR/NEMA2
(0020,5000)	AT	ACR_NEMA_OriginalImageIdentification	1-n	ACR/NEMA2
(0020,5002)	LO	ACR_NEMA_OriginalImageIdentificationNomenclature	1-n	ACR/NEMA2
(0028,0005)	US	ACR_NEMA_ImageDimensions	1	ACR/NEMA2
(0028,0040)	CS	ACR_NEMA_ImageFormat	1	ACR/NEMA2
(0028,0050)	LO	ACR_NEMA_ManipulatedImage	1-n	ACR/NEMA2
(0028,0060)	CS	ACR_NEMA_CompressionCode	1	ACR/NEMA2
(0028,0104)	xs	ACR_NEMA_SmallestValidPixelValue	1	ACR/NEMA2
(0028,0105)	xs	ACR_NEMA_LargestValidPixelValue	1	ACR/NEMA2
(0028,0200)	US	ACR_NEMA_ImageLocation	1	ACR/NEMA2
(0028,1080)	CS	ACR_NEMA_GrayScale	1	ACR/NEMA2
(0028,1100)	xs	ACR_NEMA_GrayLookupTableDescriptor	3	ACR/NEMA2
(0028,1200)	xs	ACR_NEMA_GrayLookupTableData	1-n	ACR/NEMA2
(0028,4000)	LT	ACR_NEMA_ImagePresentationComments	1-n	ACR/NEMA2
(4000,0000)	UL	ACR_NEMA_TextGroupLength	1	ACR/NEMA2
(4000,0010)	LT	ACR_NEMA_TextArbitrary	1-n	ACR/NEMA2
(4000,4000)	LT	ACR_NEMA_TextComments	1-n	ACR/NEMA2
(6000-60ff,0110)	CS	ACR_NEMA_OverlayFormat	1	ACR/NEMA2
(6000-60ff,0200)	US	ACR_NEMA_OverlayLocation	1	ACR/NEMA2
(6000-60ff,4000)	LT	ACR_NEMA_OverlayComments	1-n	ACR/NEMA2
#
#---------------------------------------------------------------------------
#
# Retired data elements from the ACR/NEMA 2 compression enhancements
#
(0028,005F)	CS	ACR_NEMA_2C_CompressionRecognitionCode	1	ACR/NEMA2C
(0028,0061)	SH	ACR_NEMA_2C_CompressionOriginator	1	ACR/NEMA2C
(0028,0062)	SH	ACR_NEMA_2C_CompressionLabel	1	ACR/NEMA2C
(0028,0063)	SH	ACR_NEMA_2C_CompressionDescription	1	ACR/NEMA2C
(0028,0065)	CS	ACR_NEMA_2C_CompressionSequence	1-n	ACR/NEMA2C
(0028,0066)	AT	ACR_NEMA_2C_CompressionStepPointers	1-n	ACR/NEMA2C
(0028,0068)	US	ACR_NEMA_2C_RepeatInterval	1	ACR/NEMA2C
(0028,0069)	US	ACR_NEMA_2C_BitsGrouped	1	ACR/NEMA2C
(0028,0070)	US	ACR_NEMA_2C_PerimeterTable	1-n	ACR/NEMA2C
(0028,0071)	xs	ACR_NEMA_2C_PerimeterValue	1	ACR/NEMA2C
(0028,0080)	US	ACR_NEMA_2C_PredictorRows	1	ACR/NEMA2C
(0028,0081)	US	ACR_NEMA_2C_PredictorColumns	1	ACR/NEMA2C
(0028,0082)	US	ACR_NEMA_2C_PredictorConstants	1-n	ACR/NEMA2C
(0028,0090)	CS	ACR_NEMA_2C_BlockedPixels	1	ACR/NEMA2C
(0028,0091)	US	ACR_NEMA_2C_BlockRows	1	ACR/NEMA2C
(0028,0092)	US	ACR_NEMA_2C_BlockColumns	1	ACR/NEMA2C
(0028,0093)	US	ACR_NEMA_2C_RowOverlap	1	ACR/NEMA2C
(0028,0094)	US	ACR_NEMA_2C_ColumnOverlap	1	ACR/NEMA2C
(0028,0400)	CS	ACR_NEMA_2C_TransformLabel	1	ACR/NEMA2C
(0028,0401)	CS	ACR_NEMA_2C_TransformVersionNumber	1	ACR/NEMA2C
(0028,0402)	US	ACR_NEMA_2C_NumberOfTransformSteps	1	ACR/NEMA2C
(0028,0403)	CS	ACR_NEMA_2C_SequenceOfCompressedData	1-n	ACR/NEMA2C
(0028,0404)	AT	ACR_NEMA_2C_DetailsOfCoefficients	1-n	ACR/NEMA2C
(0028,0410)	US	ACR_NEMA_2C_RowsForNthOrderCoefficients	1	ACR/NEMA2C
(0028,0411)	US	ACR_NEMA_2C_ColumnsForNthOrderCoefficients	1	ACR/NEMA2C
(0028,0412)	CS	ACR_NEMA_2C_CoefficientCoding	1-n	ACR/NEMA2C
(0028,0413)	AT	ACR_NEMA_2C_CoefficientCodingPointers	1-n	ACR/NEMA2C
(0028,0700)	CS	ACR_NEMA_2C_DCTLabel	1	ACR/NEMA2C
(0028,0701)	CS	ACR_NEMA_2C_DataBlockDescription	1-n	ACR/NEMA2C
(0028,0702)	AT	ACR_NEMA_2C_DataBlock	1-n	ACR/NEMA2C
(0028,0710)	US	ACR_NEMA_2C_NormalizationFactorFormat	1	ACR/NEMA2C
(0028,0720)	US	ACR_NEMA_2C_ZonalMapNumberFormat	1	ACR/NEMA2C
(0028,0721)	AT	ACR_NEMA_2C_ZonalMapLocation	1-n	ACR/NEMA2C
(0028,0722)	US	ACR_NEMA_2C_ZonalMapFormat	1	ACR/NEMA2C
(0028,0730)	US	ACR_NEMA_2C_AdaptiveMapFormat	1	ACR/NEMA2C
(0028,0740)	US	ACR_NEMA_2C_CodeNumberFormat	1	ACR/NEMA2C
(0028,0800)	CS	ACR_NEMA_2C_CodeLabel	1-n	ACR/NEMA2C
(0028,0802)	US	ACR_NEMA_2C_NumberOfTables	1	ACR/NEMA2C
(0028,0803)	AT	ACR_NEMA_2C_CodeTableLocation	1-n	ACR/NEMA2C
(0028,0804)	US	ACR_NEMA_2C_BitsForCodeWord	1	ACR/NEMA2C
(0028,0808)	AT	ACR_NEMA_2C_ImageDataLocation	1-n	ACR/NEMA2C
(1000,0000)	UL	ACR_NEMA_2C_CodeTableGroupLength	1	ACR/NEMA2C
(1000,0010)	US	ACR_NEMA_2C_EscapeTriplet	3	ACR/NEMA2C
(1000,0011)	US	ACR_NEMA_2C_RunLengthTriplet	3	ACR/NEMA2C
(1000,0012)	US	ACR_NEMA_2C_HuffmanTableSize	1	ACR/NEMA2C
(1000,0013)	US	ACR_NEMA_2C_HuffmanTableTriplet	3	ACR/NEMA2C
(1000,0014)	US	ACR_NEMA_2C_ShiftTableSize	1	ACR/NEMA2C
(1000,0015)	US	ACR_NEMA_2C_ShiftTableTriplet	3	ACR/NEMA2C
(1010,0000)	UL	ACR_NEMA_2C_ZonalMapGroupLength	1	ACR/NEMA2C
(1010,0004)	US	ACR_NEMA_2C_ZonalMap	1-n	ACR/NEMA2C
(6000-60ff,0060)	CS	ACR_NEMA_2C_OverlayCompressionCode	1	ACR/NEMA2C
(6000-60ff,0061)	SH	ACR_NEMA_2C_OverlayCompressionOriginator	1	ACR/NEMA2C
(6000-60ff,0062)	SH	ACR_NEMA_2C_OverlayCompressionLabel	1	ACR/NEMA2C
(6000-60ff,0063)	SH	ACR_NEMA_2C_OverlayCompressionDescription	1	ACR/NEMA2C
(6000-60ff,0066)	AT	ACR_NEMA_2C_OverlayCompressionStepPointers	1-n	ACR/NEMA2C
(6000-60ff,0068)	US	ACR_NEMA_2C_OverlayRepeatInterval	1	ACR/NEMA2C
(6000-60ff,0069)	US	ACR_NEMA_2C_OverlayBitsGrouped	1	ACR/NEMA2C
(6000-60ff,0800)	CS	ACR_NEMA_2C_OverlayCodeLabel	1-n	ACR/NEMA2C
(6000-60ff,0802)	US	ACR_NEMA_2C_OverlayNumberOfTables	1	ACR/NEMA2C
(6000-60ff,0803)	AT	ACR_NEMA_2C_OverlayCodeTableLocation	1-n	ACR/NEMA2C
(6000-60ff,0804)	US	ACR_NEMA_2C_OverlayBitsForCodeWord	1	ACR/NEMA2C
(7F00-7fff,0000)	UL	ACR_NEMA_2C_VariablePixelDataGroupLength	1	ACR/NEMA2C
(7F00-7fff,0010)	ox	ACR_NEMA_2C_VariablePixelData	1	ACR/NEMA2C
(7F00-7fff,0011)	AT	ACR_NEMA_2C_VariableNextDataGroup	1	ACR/NEMA2C
(7F00-7fff,0020)	OW	ACR_NEMA_2C_VariableCoefficientsSDVN	1-n	ACR/NEMA2C
(7F00-7fff,0030)	OW	ACR_NEMA_2C_VariableCoefficientsSDHN	1-n	ACR/NEMA2C
(7F00-7fff,0040)	OW	ACR_NEMA_2C_VariableCoefficientsSDDN	1-n	ACR/NEMA2C
(7FE0,0020)	OW	ACR_NEMA_2C_CoefficientsSDVN	1-n	ACR/NEMA2C
(7FE0,0030)	OW	ACR_NEMA_2C_CoefficientsSDHN	1-n	ACR/NEMA2C
(7FE0,0040)	OW	ACR_NEMA_2C_CoefficientsSDDN	1-n	ACR/NEMA2C
#
#---------------------------------------------------------------------------
#
# Retired data elements from previous editions of the DICOM standard
#
(0000,0001)	UL	RETIRED_CommandLengthToEnd	1	DICOM/retired
(0000,0010)	SH	RETIRED_CommandRecognitionCode	1	DICOM/retired
(0000,0200)	AE	RETIRED_Initiator	1	DICOM/retired
(0000,0300)	AE	RETIRED_Receiver	1	DICOM/retired
(0000,0400)	AE	RETIRED_FindLocation	1	DICOM/retired
(0000,0850)	US	RETIRED_NumberOfMatches	1	DICOM/retired
(0000,0860)	US	RETIRED_ResponseSequenceNumber	1	DICOM/retired
(0000,4000)	AT	RETIRED_DialogReceiver	1	DICOM/retired
(0000,4010)	AT	RETIRED_TerminalType	1	DICOM/retired
(0000,5010)	SH	RETIRED_MessageSetID	1	DICOM/retired
(0000,5020)	SH	RETIRED_EndMessageID	1	DICOM/retired
(0000,5110)	AT	RETIRED_DisplayFormat	1	DICOM/retired
(0000,5120)	AT	RETIRED_PagePositionID	1	DICOM/retired
(0000,5130)	CS	RETIRED_TextFormatID	1	DICOM/retired
(0000,5140)	CS	RETIRED_NormalReverse	1	DICOM/retired
(0000,5150)	CS	RETIRED_AddGrayScale	1	DICOM/retired
(0000,5160)	CS	RETIRED_Borders	1	DICOM/retired
(0000,5170)	IS	RETIRED_Copies	1	DICOM/retired
(0000,5180)	CS	RETIRED_CommandMagnificationType	1	DICOM/retired
(0000,5190)	CS	RETIRED_Erase	1	DICOM/retired
(0000,51A0)	CS	RETIRED_Print	1	DICOM/retired
(0000,51B0)	US	RETIRED_Overlays	1-n	DICOM/retired
(0004,1504)	up	RETIRED_MRDRDirectoryRecordOffset	1	DICOM/retired
(0004,1600)	UL	RETIRED_NumberOfReferences	1	DICOM/retired
(0008,0001)	UL	RETIRED_LengthToEnd	1	DICOM/retired
(0008,0010)	SH	RETIRED_RecognitionCode	1	DICOM/retired
(0008,0024)	DA	RETIRED_OverlayDate	1	DICOM/retired
(0008,0025)	DA	RETIRED_CurveDate	1	DICOM/retired
(0008,0034)	TM	RETIRED_OverlayTime	1	DICOM/retired
(0008,0035)	TM	RETIRED_CurveTime	1	DICOM/retired
(0008,0040)	US	RETIRED_DataSetType	1	DICOM/retired
(0008,0041)	LO	RETIRED_DataSetSubtype	1	DICOM/retired
(0008,0042)	CS	RETIRED_NuclearMedicineSeriesType	1	DICOM/retired
(0008,1000)	AE	RETIRED_NetworkID	1	DICOM/retired
(0008,1100)	SQ	RETIRED_ReferencedResultsSequence	1	DICOM/retired
(0008,1130)	SQ	RETIRED_ReferencedOverlaySequence	1	DICOM/retired
(0008,1145)	SQ	RETIRED_ReferencedCurveSequence	1	DICOM/retired
(0008,2110)	CS	RETIRED_LossyImageCompressionRetired	1	DICOM/retired
(0008,2200)	CS	RETIRED_TransducerPosition	1	DICOM/retired
(0008,2204)	CS	RETIRED_TransducerOrientation	1	DICOM/retired
(0008,2208)	CS	RETIRED_AnatomicStructure	1	DICOM/retired
(0008,2240)	SQ	RETIRED_TransducerPositionSequence	1	DICOM/retired
(0008,2242)	SQ	RETIRED_TransducerPositionModifierSequence	1	DICOM/retired
(0008,2244)	SQ	RETIRED_TransducerOrientationSequence	1	DICOM/retired
(0008,2246)	SQ	RETIRED_TransducerOrientationModifierSequence	1	DICOM/retired
(0008,2251)	SQ	RETIRED_AnatomicStructureSpaceOrRegionCodeSequenceTrial	1	DICOM/retired
(0008,2253)	SQ	RETIRED_AnatomicPortalOfEntranceCodeSequenceTrial	1	DICOM/retired
(0008,2255)	SQ	RETIRED_AnatomicApproachDirectionCodeSequenceTrial	1	DICOM/retired
(0008,2256)	ST	RETIRED_AnatomicPerspectiveDescriptionTrial	1	DICOM/retired
(0008,2257)	SQ	RETIRED_AnatomicPerspectiveCodeSequenceTrial	1	DICOM/retired
(0008,2258)	ST	RETIRED_AnatomicLocationOfExaminingInstrumentDescriptionTrial	1	DICOM/retired
(0008,2259)	SQ	RETIRED_AnatomicLocationOfExaminingInstrumentCodeSequenceTrial	1	DICOM/retired
(0008,225A)	SQ	RETIRED_AnatomicStructureSpaceOrRegionModifierCodeSequenceTrial	1	DICOM/retired
(0008,225C)	SQ	RETIRED_OnAxisBackgroundAnatomicStructureCodeSequenceTrial	1	DICOM/retired
(0008,4000)	LT	RETIRED_IdentifyingComments	1	DICOM/retired
(0010,1050)	LO	RETIRED_InsurancePlanIdentification	1-n	DICOM/retired
(0018,0030)	LO	RETIRED_Radionuclide	1-n	DICOM/retired
(0018,0032)	DS	RETIRED_EnergyWindowCenterline	1	DICOM/retired
(0018,0033)	DS	RETIRED_EnergyWindowTotalWidth	1-n	DICOM/retired
(0018,0037)	CS	RETIRED_TherapyType	1	DICOM/retired
(0018,0039)	CS	RETIRED_TherapyDescription	1	DICOM/retired
(0018,1011)	LO	RETIRED_HardcopyCreationDeviceID	1	DICOM/retired
(0018,1017)	LO	RETIRED_HardcopyDeviceManufacturer	1	DICOM/retired
(0018,101A)	LO	RETIRED_HardcopyDeviceSoftwareVersion	1-n	DICOM/retired
(0018,101B)	LO	RETIRED_HardcopyDeviceManufacturerModelName	1	DICOM/retired
(0018,1141)	DS	RETIRED_AngularPosition	1	DICOM/retired
(0018,1146)	DS	RETIRED_RotationOffset	1-n	DICOM/retired
(0018,1240)	IS	RETIRED_UpperLowerPixelValues	1-n	DICOM/retired
(0018,4000)	LT	RETIRED_AcquisitionComments	1	DICOM/retired
(0018,5021)	LO	RETIRED_PostprocessingFunction	1	DICOM/retired
(0018,5030)	DS	RETIRED_DynamicRange	1	DICOM/retired
(0018,5040)	DS	RETIRED_TotalGain	1	DICOM/retired
(0018,5210)	DS	RETIRED_ImageTransformationMatrix	6	DICOM/retired
(0018,5212)	DS	RETIRED_ImageTranslationVector	3	DICOM/retired
(0018,6038)	UL	RETIRED_DopplerSampleVolumeXPositionRetired	1	DICOM/retired
(0018,603A)	UL	RETIRED_DopplerSampleVolumeYPositionRetired	1	DICOM/retired
(0018,603C)	UL	RETIRED_TMLinePositionX0Retired	1	DICOM/retired
(0018,603E)	UL	RETIRED_TMLinePositionY0Retired	1	DICOM/retired
(0018,6040)	UL	RETIRED_TMLinePositionX1Retired	1	DICOM/retired
(0018,6042)	UL	RETIRED_TMLinePositionY1Retired	1	DICOM/retired
(0018,9096)	FD	RETIRED_ParallelReductionFactorInPlaneRetired	1	DICOM/retired
(0018,9166)	CS	RETIRED_BulkMotionStatus	1	DICOM/retired
(0018,9195)	FD	RETIRED_ChemicalShiftMinimumIntegrationLimitInHz	1	DICOM/retired
(0018,9196)	FD	RETIRED_ChemicalShiftMaximumIntegrationLimitInHz	1	DICOM/retired
(0020,0014)	IS	RETIRED_IsotopeNumber	1	DICOM/retired
(0020,0015)	IS	RETIRED_PhaseNumber	1	DICOM/retired
(0020,0016)	IS	RETIRED_IntervalNumber	1	DICOM/retired
(0020,0017)	IS	RETIRED_TimeSlotNumber	1	DICOM/retired
(0020,0018)	IS	RETIRED_AngleNumber	1	DICOM/retired
(0020,0022)	IS	RETIRED_OverlayNumber	1	DICOM/retired
(0020,0024)	IS	RETIRED_CurveNumber	1	DICOM/retired
(0020,0026)	IS	RETIRED_LUTNumber	1	DICOM/retired
(0020,0030)	DS	RETIRED_ImagePosition	3	DICOM/retired
(0020,0035)	DS	RETIRED_ImageOrientation	6	DICOM/retired
(0020,0050)	DS	RETIRED_Location	1	DICOM/retired
(0020,0070)	LO	RETIRED_ImageGeometryType	1	DICOM/retired
(0020,0080)	CS	RETIRED_MaskingImage	1-n	DICOM/retired
(0020,00AA)	IS	RETIRED_ReportNumber	1	DICOM/retired
(0020,1000)	IS	RETIRED_SeriesInStudy	1	DICOM/retired
(0020,1001)	IS	RETIRED_AcquisitionsInSeries	1	DICOM/retired
(0020,1003)	IS	RETIRED_ImagesInSeries	1	DICOM/retired
(0020,1004)	IS	RETIRED_AcquisitionsInStudy	1	DICOM/retired
(0020,1005)	IS	RETIRED_ImagesInStudy	1	DICOM/retired
(0020,1020)	LO	RETIRED_Reference	1-n	DICOM/retired
(0020,1070)	IS	RETIRED_OtherStudyNumbers	1-n	DICOM/retired
(0020,3100-31FF)	CS	RETIRED_SourceImageIDs	1-n	DICOM/retired
(0020,3401)	CS	RETIRED_ModifyingDeviceID	1	DICOM/retired
(0020,3402)	CS	RETIRED_ModifiedImageID	1	DICOM/retired
(0020,3403)	DA	RETIRED_ModifiedImageDate	1	DICOM/retired
(0020,3404)	LO	RETIRED_ModifyingDeviceManufacturer	1	DICOM/retired
(0020,3405)	TM	RETIRED_ModifiedImageTime	1	DICOM/retired
(0020,3406)	LO	RETIRED_ModifiedImageDescription	1	DICOM/retired
(0020,5000)	AT	RETIRED_OriginalImageIdentification	1-n	DICOM/retired
(0020,5002)	LO	RETIRED_OriginalImageIdentificationNomenclature	1-n	DICOM/retired
(0028,0005)	US	RETIRED_ImageDimensions	1	DICOM/retired
(0028,0012)	US	RETIRED_Planes	1	DICOM/retired
(0028,0040)	CS	RETIRED_ImageFormat	1	DICOM/retired
(0028,0050)	LO	RETIRED_ManipulatedImage	1-n	DICOM/retired
(0028,005F)	LO	RETIRED_CompressionRecognitionCode	1	DICOM/retired
(0028,0060)	CS	RETIRED_CompressionCode	1	DICOM/retired
(0028,0061)	SH	RETIRED_CompressionOriginator	1	DICOM/retired
(0028,0062)	LO	RETIRED_CompressionLabel	1	DICOM/retired
(0028,0063)	SH	RETIRED_CompressionDescription	1	DICOM/retired
(0028,0065)	CS	RETIRED_CompressionSequence	1-n	DICOM/retired
(0028,0066)	AT	RETIRED_CompressionStepPointers	1-n	DICOM/retired
(0028,0068)	US	RETIRED_RepeatInterval	1	DICOM/retired
(0028,0069)	US	RETIRED_BitsGrouped	1	DICOM/retired
(0028,0070)	US	RETIRED_PerimeterTable	1-n	DICOM/retired
(0028,0071)	xs	RETIRED_PerimeterValue	1	DICOM/retired
(0028,0080)	US	RETIRED_PredictorRows	1	DICOM/retired
(0028,0081)	US	RETIRED_PredictorColumns	1	DICOM/retired
(0028,0082)	US	RETIRED_PredictorConstants	1-n	DICOM/retired
(0028,0090)	CS	RETIRED_BlockedPixels	1	DICOM/retired
(0028,0091)	US	RETIRED_BlockRows	1	DICOM/retired
(0028,0092)	US	RETIRED_BlockColumns	1	DICOM/retired
(0028,0093)	US	RETIRED_RowOverlap	1	DICOM/retired
(0028,0094)	US	RETIRED_ColumnOverlap	1	DICOM/retired
(0028,0104)	xs	RETIRED_SmallestValidPixelValue	1	DICOM/retired
(0028,0105)	xs	RETIRED_LargestValidPixelValue	1	DICOM/retired
(0028,0110)	xs	RETIRED_SmallestImagePixelValueInPlane	1	DICOM/retired
(0028,0111)	xs	RETIRED_LargestImagePixelValueInPlane	1	DICOM/retired
(0028,0200)	US	RETIRED_ImageLocation	1	DICOM/retired
(0028,0400)	LO	RETIRED_TransformLabel	1	DICOM/retired
(0028,0401)	LO	RETIRED_TransformVersionNumber	1	DICOM/retired
(0028,0402)	US	RETIRED_NumberOfTransformSteps	1	DICOM/retired
(0028,0403)	LO	RETIRED_SequenceOfCompressedData	1-n	DICOM/retired
(0028,0404)	AT	RETIRED_DetailsOfCoefficients	1-n	DICOM/retired
#(0028,04X0)	US	RETIRED_RowsForNthOrderCoefficients	1	DICOM/retired
#(0028,04X1)	US	RETIRED_ColumnsForNthOrderCoefficients	1	DICOM/retired
#(0028,04X2)	LO	RETIRED_CoefficientCoding	1-n	DICOM/retired
#(0028,04X3)	AT	RETIRED_CoefficientCodingPointers	1-n	DICOM/retired
(0028,0700)	LO	RETIRED_DCTLabel	1	DICOM/retired
(0028,0701)	CS	RETIRED_DataBlockDescription	1-n	DICOM/retired
(0028,0702)	AT	RETIRED_DataBlock	1-n	DICOM/retired
(0028,0710)	US	RETIRED_NormalizationFactorFormat	1	DICOM/retired
(0028,0720)	US	RETIRED_ZonalMapNumberFormat	1	DICOM/retired
(0028,0721)	AT	RETIRED_ZonalMapLocation	1-n	DICOM/retired
(0028,0722)	US	RETIRED_ZonalMapFormat	1	DICOM/retired
(0028,0730)	US	RETIRED_AdaptiveMapFormat	1	DICOM/retired
(0028,0740)	US	RETIRED_CodeNumberFormat	1	DICOM/retired
#(0028,08X0)	CS	RETIRED_CodeLabel	1-n	DICOM/retired
#(0028,08X2)	US	RETIRED_NumberOfTables	1	DICOM/retired
#(0028,08X3)	AT	RETIRED_CodeTableLocation	1-n	DICOM/retired
#(0028,08X4)	US	RETIRED_BitsForCodeWord	1	DICOM/retired
#(0028,08X8)	AT	RETIRED_ImageDataLocation	1-n	DICOM/retired
(0028,1080)	CS	RETIRED_GrayScale	1	DICOM/retired
(0028,1100)	xs	RETIRED_GrayLookupTableDescriptor	3	DICOM/retired
(0028,1111)	xs	RETIRED_LargeRedPaletteColorLookupTableDescriptor	4	DICOM/retired
(0028,1112)	xs	RETIRED_LargeGreenPaletteColorLookupTableDescriptor	4	DICOM/retired
(0028,1113)	xs	RETIRED_LargeBluePaletteColorLookupTableDescriptor	4	DICOM/retired
(0028,1200)	lt	RETIRED_GrayLookupTableData	1-n	DICOM/retired
(0028,1211)	OW	RETIRED_LargeRedPaletteColorLookupTableData	1	DICOM/retired
(0028,1212)	OW	RETIRED_LargeGreenPaletteColorLookupTableData	1	DICOM/retired
(0028,1213)	OW	RETIRED_LargeBluePaletteColorLookupTableData	1	DICOM/retired
(0028,1214)	UI	RETIRED_LargePaletteColorLookupTableUID	1	DICOM/retired
(0028,4000)	LT	RETIRED_ImagePresentationComments	1	DICOM/retired
(0028,5000)	SQ	RETIRED_BiPlaneAcquisitionSequence	1	DICOM/retired
(0028,6030)	US	RETIRED_MaskPointers	1-n	DICOM/retired
(0028,9099)	US	RETIRED_LargestMonochromePixelValue	1	DICOM/retired
(0032,000A)	CS	RETIRED_StudyStatusID	1	DICOM/retired
(0032,000C)	CS	RETIRED_StudyPriorityID	1	DICOM/retired
(0032,0012)	LO	RETIRED_StudyIDIssuer	1	DICOM/retired
(0032,0032)	DA	RETIRED_StudyVerifiedDate	1	DICOM/retired
(0032,0033)	TM	RETIRED_StudyVerifiedTime	1	DICOM/retired
(0032,0034)	DA	RETIRED_StudyReadDate	1	DICOM/retired
(0032,0035)	TM	RETIRED_StudyReadTime	1	DICOM/retired
(0032,1000)	DA	RETIRED_ScheduledStudyStartDate	1	DICOM/retired
(0032,1001)	TM	RETIRED_ScheduledStudyStartTime	1	DICOM/retired
(0032,1010)	DA	RETIRED_ScheduledStudyStopDate	1	DICOM/retired
(0032,1011)	TM	RETIRED_ScheduledStudyStopTime	1	DICOM/retired
(0032,1020)	LO	RETIRED_ScheduledStudyLocation	1	DICOM/retired
(0032,1021)	AE	RETIRED_ScheduledStudyLocationAETitle	1-n	DICOM/retired
(0032,1030)	LO	RETIRED_ReasonForStudy	1	DICOM/retired
(0032,1040)	DA	RETIRED_StudyArrivalDate	1	DICOM/retired
(0032,1041)	TM	RETIRED_StudyArrivalTime	1	DICOM/retired
(0032,1050)	DA	RETIRED_StudyCompletionDate	1	DICOM/retired
(0032,1051)	TM	RETIRED_StudyCompletionTime	1	DICOM/retired
(0032,1055)	CS	RETIRED_StudyComponentStatusID	1	DICOM/retired
(0032,4000)	LT	RETIRED_StudyComments	1	DICOM/retired
(0038,0011)	LO	RETIRED_IssuerOfAdmissionID	1	DICOM/retired
(0038,001A)	DA	RETIRED_ScheduledAdmissionDate	1	DICOM/retired
(0038,001B)	TM	RETIRED_ScheduledAdmissionTime	1	DICOM/retired
(0038,001C)	DA	RETIRED_ScheduledDischargeDate	1	DICOM/retired
(0038,001D)	TM	RETIRED_ScheduledDischargeTime	1	DICOM/retired
(0038,001E)	LO	RETIRED_ScheduledPatientInstitutionResidence	1	DICOM/retired
(0038,0030)	DA	RETIRED_DischargeDate	1	DICOM/retired
(0038,0032)	TM	RETIRED_DischargeTime	1	DICOM/retired
(0038,0040)	LO	RETIRED_DischargeDiagnosisDescription	1	DICOM/retired
(0038,0044)	SQ	RETIRED_DischargeDiagnosisCodeSequence	1	DICOM/retired
(0038,0061)	LO	RETIRED_IssuerOfServiceEpisodeID	1	DICOM/retired
(0040,0307)	DS	RETIRED_DistanceSourceToSupport	1	DICOM/retired
(0040,0330)	SQ	RETIRED_ReferencedProcedureStepSequence	1	DICOM/retired
(0040,050A)	LO	RETIRED_SpecimenAccessionNumber	1	DICOM/retired
(0040,0550)	SQ	RETIRED_SpecimenSequence	1	DICOM/retired
(0040,0552)	SQ	RETIRED_SpecimenDescriptionSequenceTrial	1	DICOM/retired
(0040,0553)	ST	RETIRED_SpecimenDescriptionTrial	1	DICOM/retired
(0040,06FA)	LO	RETIRED_SlideIdentifier	1	DICOM/retired
(0040,08D8)	SQ	RETIRED_PixelSpacingSequence	1	DICOM/retired
(0040,08DA)	SQ	RETIRED_CoordinateSystemAxisCodeSequence	1	DICOM/retired
(0040,09F8)	SQ	RETIRED_VitalStainCodeSequenceTrial	1	DICOM/retired
(0040,1006)	SH	RETIRED_PlacerOrderNumberProcedure	1	DICOM/retired
(0040,1007)	SH	RETIRED_FillerOrderNumberProcedure	1	DICOM/retired
(0040,1060)	LO	RETIRED_RequestedProcedureDescriptionTrial	1	DICOM/retired
(0040,2001)	LO	RETIRED_ReasonForTheImagingServiceRequest	1	DICOM/retired
(0040,2006)	SH	RETIRED_PlacerOrderNumberImagingServiceRequestRetired	1	DICOM/retired
(0040,2007)	SH	RETIRED_FillerOrderNumberImagingServiceRequestRetired	1	DICOM/retired
(0040,A007)	CS	RETIRED_FindingsFlagTrial	1	DICOM/retired
(0040,A020)	SQ	RETIRED_FindingsSequenceTrial	1	DICOM/retired
(0040,A021)	UI	RETIRED_FindingsGroupUIDTrial	1	DICOM/retired
(0040,A022)	UI	RETIRED_ReferencedFindingsGroupUIDTrial	1	DICOM/retired
(0040,A023)	DA	RETIRED_FindingsGroupRecordingDateTrial	1	DICOM/retired
(0040,A024)	TM	RETIRED_FindingsGroupRecordingTimeTrial	1	DICOM/retired
(0040,A026)	SQ	RETIRED_FindingsSourceCategoryCodeSequenceTrial	1	DICOM/retired
(0040,A028)	SQ	RETIRED_DocumentingOrganizationIdentifierCodeSequenceTrial	1	DICOM/retired
(0040,A047)	LO	RETIRED_MeasurementPrecisionDescriptionTrial	1	DICOM/retired
(0040,A057)	CS	RETIRED_UrgencyOrPriorityAlertsTrial	1-n	DICOM/retired
(0040,A060)	LO	RETIRED_SequencingIndicatorTrial	1	DICOM/retired
(0040,A066)	SQ	RETIRED_DocumentIdentifierCodeSequenceTrial	1	DICOM/retired
(0040,A067)	PN	RETIRED_DocumentAuthorTrial	1	DICOM/retired
(0040,A068)	SQ	RETIRED_DocumentAuthorIdentifierCodeSequenceTrial	1	DICOM/retired
(0040,A070)	SQ	RETIRED_IdentifierCodeSequenceTrial	1	DICOM/retired
(0040,A074)	OB	RETIRED_ObjectBinaryIdentifierTrial	1	DICOM/retired
(0040,A076)	SQ	RETIRED_DocumentingObserverIdentifierCodeSequenceTrial	1	DICOM/retired
(0040,A085)	SQ	RETIRED_ProcedureIdentifierCodeSequenceTrial	1	DICOM/retired
(0040,A089)	OB	RETIRED_ObjectDirectoryBinaryIdentifierTrial	1	DICOM/retired
(0040,A090)	SQ	RETIRED_EquivalentCDADocumentSequence	1	DICOM/retired
(0040,A110)	DA	RETIRED_DateOfDocumentOrVerbalTransactionTrial	1	DICOM/retired
(0040,A112)	TM	RETIRED_TimeOfDocumentCreationOrVerbalTransactionTrial	1	DICOM/retired
(0040,A125)	CS	RETIRED_ReportStatusIDTrial	2	DICOM/retired
(0040,A167)	SQ	RETIRED_ObservationCategoryCodeSequenceTrial	1	DICOM/retired
(0040,A16A)	ST	RETIRED_BibliographicCitationTrial	1	DICOM/retired
# the following attribute is defined as SQ in the DICOM standard,
# it was defined as CS in Supplement 23 (frozen draft) from 1997.
#(0040,A170)	CS	RETIRED_ObservationClassTrial	1	DICOM/retired
# the following attribute has been reintroduced with CP-1147
#(0040,A171)	UI	RETIRED_ObservationUIDTrial	1	DICOM/retired
(0040,A172)	UI	RETIRED_ReferencedObservationUIDTrial	1	DICOM/retired
(0040,A173)	CS	RETIRED_ReferencedObservationClassTrial	1	DICOM/retired
(0040,A174)	CS	RETIRED_ReferencedObjectObservationClassTrial	1	DICOM/retired
(0040,A192)	DA	RETIRED_ObservationDateTrial	1	DICOM/retired
(0040,A193)	TM	RETIRED_ObservationTimeTrial	1	DICOM/retired
(0040,A194)	CS	RETIRED_MeasurementAutomationTrial	1	DICOM/retired
(0040,A224)	ST	RETIRED_IdentificationDescriptionTrial	1	DICOM/retired
(0040,A290)	CS	RETIRED_CoordinatesSetGeometricTypeTrial	1	DICOM/retired
(0040,A296)	SQ	RETIRED_AlgorithmCodeSequenceTrial	1	DICOM/retired
(0040,A297)	ST	RETIRED_AlgorithmDescriptionTrial	1	DICOM/retired
(0040,A29A)	SL	RETIRED_PixelCoordinatesSetTrial	2-2n	DICOM/retired
(0040,A307)	PN	RETIRED_CurrentObserverTrial	1	DICOM/retired
(0040,A313)	SQ	RETIRED_ReferencedAccessionSequenceTrial	1	DICOM/retired
(0040,A33A)	ST	RETIRED_ReportStatusCommentTrial	1	DICOM/retired
(0040,A340)	SQ	RETIRED_ProcedureContextSequenceTrial	1	DICOM/retired
(0040,A352)	PN	RETIRED_VerbalSourceTrial	1	DICOM/retired
(0040,A353)	ST	RETIRED_AddressTrial	1	DICOM/retired
(0040,A354)	LO	RETIRED_TelephoneNumberTrial	1	DICOM/retired
(0040,A358)	SQ	RETIRED_VerbalSourceIdentifierCodeSequenceTrial	1	DICOM/retired
(0040,A380)	SQ	RETIRED_ReportDetailSequenceTrial	1	DICOM/retired
(0040,A402)	UI	RETIRED_ObservationSubjectUIDTrial	1	DICOM/retired
(0040,A403)	CS	RETIRED_ObservationSubjectClassTrial	1	DICOM/retired
(0040,A404)	SQ	RETIRED_ObservationSubjectTypeCodeSequenceTrial	1	DICOM/retired
(0040,A600)	CS	RETIRED_ObservationSubjectContextFlagTrial	1	DICOM/retired
(0040,A601)	CS	RETIRED_ObserverContextFlagTrial	1	DICOM/retired
(0040,A603)	CS	RETIRED_ProcedureContextFlagTrial	1	DICOM/retired
(0040,A731)	SQ	RETIRED_RelationshipSequenceTrial	1	DICOM/retired
(0040,A732)	SQ	RETIRED_RelationshipTypeCodeSequenceTrial	1	DICOM/retired
(0040,A744)	SQ	RETIRED_LanguageCodeSequenceTrial	1	DICOM/retired
(0040,A992)	ST	RETIRED_UniformResourceLocatorTrial	1	DICOM/retired
(0040,DB06)	DT	RETIRED_TemplateVersion	1	DICOM/retired
(0040,DB07)	DT	RETIRED_TemplateLocalVersion	1	DICOM/retired
(0040,DB0B)	CS	RETIRED_TemplateExtensionFlag	1	DICOM/retired
(0040,DB0C)	UI	RETIRED_TemplateExtensionOrganizationUID	1	DICOM/retired
(0040,DB0D)	UI	RETIRED_TemplateExtensionCreatorUID	1	DICOM/retired
(0054,1400)	CS	RETIRED_CountsIncluded	1-n	DICOM/retired
(0054,1401)	CS	RETIRED_DeadTimeCorrectionFlag	1	DICOM/retired
(0070,0040)	IS	RETIRED_ImageRotationRetired	1	DICOM/retired
(0070,0050)	US	RETIRED_DisplayedAreaTopLeftHandCornerTrial	2	DICOM/retired
(0070,0051)	US	RETIRED_DisplayedAreaBottomRightHandCornerTrial	2	DICOM/retired
(0070,0067)	US	RETIRED_GraphicLayerRecommendedDisplayRGBValue	3	DICOM/retired
(0074,1024)	IS	RETIRED_BeamOrderIndexTrial	1	DICOM/retired
(0074,1038)	DS	RETIRED_DoubleExposureMetersetTrial	1	DICOM/retired
(0074,103A)	DS	RETIRED_DoubleExposureFieldDeltaTrial	4	DICOM/retired
(0074,1220)	SQ	RETIRED_RelatedProcedureStepSequence	1	DICOM/retired
(0074,1222)	LO	RETIRED_ProcedureStepRelationshipType	1	DICOM/retired
(0088,0904)	LO	RETIRED_TopicTitle	1	DICOM/retired
(0088,0906)	ST	RETIRED_TopicSubject	1	DICOM/retired
(0088,0910)	LO	RETIRED_TopicAuthor	1	DICOM/retired
(0088,0912)	LO	RETIRED_TopicKeywords	1-32	DICOM/retired
#(1000,XXX0)	US	RETIRED_EscapeTriplet	3	DICOM/retired
#(1000,XXX0)	US	RETIRED_RunLengthTriplet	3	DICOM/retired
#(1000,XXX0)	US	RETIRED_HuffmanTableSize	1	DICOM/retired
#(1000,XXX0)	US	RETIRED_HuffmanTableTriplet	3	DICOM/retired
#(1000,XXX0)	US	RETIRED_ShiftTableSize	1	DICOM/retired
#(1000,XXX0)	US	RETIRED_ShiftTableTriplet	3	DICOM/retired
#(1010,XXXX)	US	RETIRED_ZonalMap	1-n	DICOM/retired
(2000,0062)	CS	RETIRED_ColorImagePrintingFlag	1	DICOM/retired
(2000,0063)	CS	RETIRED_CollationFlag	1	DICOM/retired
(2000,0065)	CS	RETIRED_AnnotationFlag	1	DICOM/retired
(2000,0067)	CS	RETIRED_ImageOverlayFlag	1	DICOM/retired
(2000,0069)	CS	RETIRED_PresentationLUTFlag	1	DICOM/retired
(2000,006A)	CS	RETIRED_ImageBoxPresentationLUTFlag	1	DICOM/retired
(2000,0510)	SQ	RETIRED_ReferencedStoredPrintSequence	1	DICOM/retired
(2020,0130)	SQ	RETIRED_ReferencedImageOverlayBoxSequence	1	DICOM/retired
(2020,0140)	SQ	RETIRED_ReferencedVOILUTBoxSequence	1	DICOM/retired
(2040,0010)	SQ	RETIRED_ReferencedOverlayPlaneSequence	1	DICOM/retired
(2040,0011)	US	RETIRED_ReferencedOverlayPlaneGroups	1-99	DICOM/retired
(2040,0020)	SQ	RETIRED_OverlayPixelDataSequence	1	DICOM/retired
(2040,0060)	CS	RETIRED_OverlayMagnificationType	1	DICOM/retired
(2040,0070)	CS	RETIRED_OverlaySmoothingType	1	DICOM/retired
(2040,0072)	CS	RETIRED_OverlayOrImageMagnification	1	DICOM/retired
(2040,0074)	US	RETIRED_MagnifyToNumberOfColumns	1	DICOM/retired
(2040,0080)	CS	RETIRED_OverlayForegroundDensity	1	DICOM/retired
(2040,0082)	CS	RETIRED_OverlayBackgroundDensity	1	DICOM/retired
(2040,0090)	CS	RETIRED_OverlayMode	1	DICOM/retired
(2040,0100)	CS	RETIRED_ThresholdDensity	1	DICOM/retired
(2040,0500)	SQ	RETIRED_ReferencedImageBoxSequenceRetired	1	DICOM/retired
(2100,0010)	SH	RETIRED_PrintJobID	1	DICOM/retired
(2100,0140)	AE	RETIRED_DestinationAE	1	DICOM/retired
(2100,0500)	SQ	RETIRED_ReferencedPrintJobSequencePullStoredPrint	1	DICOM/retired
(2110,0099)	SH	RETIRED_PrintQueueID	1	DICOM/retired
(2120,0010)	CS	RETIRED_QueueStatus	1	DICOM/retired
(2120,0050)	SQ	RETIRED_PrintJobDescriptionSequence	1	DICOM/retired
(2120,0070)	SQ	RETIRED_ReferencedPrintJobSequence	1	DICOM/retired
(2130,0010)	SQ	RETIRED_PrintManagementCapabilitiesSequence	1	DICOM/retired
(2130,0015)	SQ	RETIRED_PrinterCharacteristicsSequence	1	DICOM/retired
(2130,0030)	SQ	RETIRED_FilmBoxContentSequence	1	DICOM/retired
(2130,0040)	SQ	RETIRED_ImageBoxContentSequence	1	DICOM/retired
(2130,0050)	SQ	RETIRED_AnnotationContentSequence	1	DICOM/retired
(2130,0060)	SQ	RETIRED_ImageOverlayBoxContentSequence	1	DICOM/retired
(2130,0080)	SQ	RETIRED_PresentationLUTContentSequence	1	DICOM/retired
(2130,00A0)	SQ	RETIRED_ProposedStudySequence	1	DICOM/retired
(2130,00C0)	SQ	RETIRED_OriginalImageSequence	1	DICOM/retired
(4000,0010)	LT	RETIRED_Arbitrary	1	DICOM/retired
(4000,4000)	LT	RETIRED_TextComments	1	DICOM/retired
(4008,0040)	SH	RETIRED_ResultsID	1	DICOM/retired
(4008,0042)	LO	RETIRED_ResultsIDIssuer	1	DICOM/retired
(4008,0050)	SQ	RETIRED_ReferencedInterpretationSequence	1	DICOM/retired
(4008,00FF)	CS	RETIRED_ReportProductionStatusTrial	1	DICOM/retired
(4008,0100)	DA	RETIRED_InterpretationRecordedDate	1	DICOM/retired
(4008,0101)	TM	RETIRED_InterpretationRecordedTime	1	DICOM/retired
(4008,0102)	PN	RETIRED_InterpretationRecorder	1	DICOM/retired
(4008,0103)	LO	RETIRED_ReferenceToRecordedSound	1	DICOM/retired
(4008,0108)	DA	RETIRED_InterpretationTranscriptionDate	1	DICOM/retired
(4008,0109)	TM	RETIRED_InterpretationTranscriptionTime	1	DICOM/retired
(4008,010A)	PN	RETIRED_InterpretationTranscriber	1	DICOM/retired
(4008,010B)	ST	RETIRED_InterpretationText	1	DICOM/retired
(4008,010C)	PN	RETIRED_InterpretationAuthor	1	DICOM/retired
(4008,0111)	SQ	RETIRED_InterpretationApproverSequence	1	DICOM/retired
(4008,0112)	DA	RETIRED_InterpretationApprovalDate	1	DICOM/retired
(4008,0113)	TM	RETIRED_InterpretationApprovalTime	1	DICOM/retired
(4008,0114)	PN	RETIRED_PhysicianApprovingInterpretation	1	DICOM/retired
(4008,0115)	LT	RETIRED_InterpretationDiagnosisDescription	1	DICOM/retired
(4008,0117)	SQ	RETIRED_InterpretationDiagnosisCodeSequence	1	DICOM/retired
(4008,0118)	SQ	RETIRED_ResultsDistributionListSequence	1	DICOM/retired
(4008,0119)	PN	RETIRED_DistributionName	1	DICOM/retired
(4008,011A)	LO	RETIRED_DistributionAddress	1	DICOM/retired
(4008,0200)	SH	RETIRED_InterpretationID	1	DICOM/retired
(4008,0202)	LO	RETIRED_InterpretationIDIssuer	1	DICOM/retired
(4008,0210)	CS	RETIRED_InterpretationTypeID	1	DICOM/retired
(4008,0212)	CS	RETIRED_InterpretationStatusID	1	DICOM/retired
(4008,0300)	ST	RETIRED_Impressions	1	DICOM/retired
(4008,4000)	ST	RETIRED_ResultsComments	1	DICOM/retired
(5000-50FF,0005)	US	RETIRED_CurveDimensions	1	DICOM/retired
(5000-50FF,0010)	US	RETIRED_NumberOfPoints	1	DICOM/retired
(5000-50FF,0020)	CS	RETIRED_TypeOfData	1	DICOM/retired
(5000-50FF,0022)	LO	RETIRED_CurveDescription	1	DICOM/retired
(5000-50FF,0030)	SH	RETIRED_AxisUnits	1-n	DICOM/retired
(5000-50FF,0040)	SH	RETIRED_AxisLabels	1-n	DICOM/retired
(5000-50FF,0103)	US	RETIRED_DataValueRepresentation	1	DICOM/retired
(5000-50FF,0104)	US	RETIRED_MinimumCoordinateValue	1-n	DICOM/retired
(5000-50FF,0105)	US	RETIRED_MaximumCoordinateValue	1-n	DICOM/retired
(5000-50FF,0106)	SH	RETIRED_CurveRange	1-n	DICOM/retired
(5000-50FF,0110)	US	RETIRED_CurveDataDescriptor	1-n	DICOM/retired
(5000-50FF,0112)	US	RETIRED_CoordinateStartValue	1-n	DICOM/retired
(5000-50FF,0114)	US	RETIRED_CoordinateStepValue	1-n	DICOM/retired
(5000-50FF,1001)	CS	RETIRED_CurveActivationLayer	1	DICOM/retired
(5000-50FF,2000)	US	RETIRED_AudioType	1	DICOM/retired
(5000-50FF,2002)	US	RETIRED_AudioSampleFormat	1	DICOM/retired
(5000-50FF,2004)	US	RETIRED_NumberOfChannels	1	DICOM/retired
(5000-50FF,2006)	UL	RETIRED_NumberOfSamples	1	DICOM/retired
(5000-50FF,2008)	UL	RETIRED_SampleRate	1	DICOM/retired
(5000-50FF,200A)	UL	RETIRED_TotalTime	1	DICOM/retired
(5000-50FF,200C)	ox	RETIRED_AudioSampleData	1	DICOM/retired
(5000-50FF,200E)	LT	RETIRED_AudioComments	1	DICOM/retired
(5000-50FF,2500)	LO	RETIRED_CurveLabel	1	DICOM/retired
(5000-50FF,2600)	SQ	RETIRED_CurveReferencedOverlaySequence	1	DICOM/retired
(5000-50FF,2610)	US	RETIRED_CurveReferencedOverlayGroup	1	DICOM/retired
(5000-50FF,3000)	ox	RETIRED_CurveData	1	DICOM/retired
(6000-60FF,0012)	US	RETIRED_OverlayPlanes	1	DICOM/retired
(6000-60FF,0052)	US	RETIRED_OverlayPlaneOrigin	1	DICOM/retired
(6000-60FF,0060)	CS	RETIRED_OverlayCompressionCode	1	DICOM/retired
(6000-60FF,0061)	SH	RETIRED_OverlayCompressionOriginator	1	DICOM/retired
(6000-60FF,0062)	SH	RETIRED_OverlayCompressionLabel	1	DICOM/retired
(6000-60FF,0063)	CS	RETIRED_OverlayCompressionDescription	1	DICOM/retired
(6000-60FF,0066)	AT	RETIRED_OverlayCompressionStepPointers	1-n	DICOM/retired
(6000-60FF,0068)	US	RETIRED_OverlayRepeatInterval	1	DICOM/retired
(6000-60FF,0069)	US	RETIRED_OverlayBitsGrouped	1	DICOM/retired
(6000-60FF,0110)	CS	RETIRED_OverlayFormat	1	DICOM/retired
(6000-60FF,0200)	US	RETIRED_OverlayLocation	1	DICOM/retired
(6000-60FF,0800)	CS	RETIRED_OverlayCodeLabel	1-n	DICOM/retired
(6000-60FF,0802)	US	RETIRED_OverlayNumberOfTables	1	DICOM/retired
(6000-60FF,0803)	AT	RETIRED_OverlayCodeTableLocation	1-n	DICOM/retired
(6000-60FF,0804)	US	RETIRED_OverlayBitsForCodeWord	1	DICOM/retired
(6000-60FF,1100)	US	RETIRED_OverlayDescriptorGray	1	DICOM/retired
(6000-60FF,1101)	US	RETIRED_OverlayDescriptorRed	1	DICOM/retired
(6000-60FF,1102)	US	RETIRED_OverlayDescriptorGreen	1	DICOM/retired
(6000-60FF,1103)	US	RETIRED_OverlayDescriptorBlue	1	DICOM/retired
(6000-60FF,1200)	US	RETIRED_OverlaysGray	1-n	DICOM/retired
(6000-60FF,1201)	US	RETIRED_OverlaysRed	1-n	DICOM/retired
(6000-60FF,1202)	US	RETIRED_OverlaysGreen	1-n	DICOM/retired
(6000-60FF,1203)	US	RETIRED_OverlaysBlue	1-n	DICOM/retired
(6000-60FF,4000)	LT	RETIRED_OverlayComments	1	DICOM/retired
(7FE0,0020)	OW	RETIRED_CoefficientsSDVN	1	DICOM/retired
(7FE0,0030)	OW	RETIRED_CoefficientsSDHN	1	DICOM/retired
(7FE0,0040)	OW	RETIRED_CoefficientsSDDN	1	DICOM/retired
(7F00-7FFF,0010)	ox	RETIRED_VariablePixelData	1	DICOM/retired
(7F00-7FFF,0011)	US	RETIRED_VariableNextDataGroup	1	DICOM/retired
(7F00-7FFF,0020)	OW	RETIRED_VariableCoefficientsSDVN	1	DICOM/retired
(7F00-7FFF,0030)	OW	RETIRED_VariableCoefficientsSDHN	1	DICOM/retired
(7F00-7FFF,0040)	OW	RETIRED_VariableCoefficientsSDDN	1	DICOM/retired
#
# EOF
#
"""

def main():
    with open("tag_definitions.go", "w") as out:
        generate(out)

main()
